/* cdfaltmap                                                         */
/* This program converts the 30 sec USAF terrain data to Candis      */
/* format.                                                           */
/* cdfaltmap lat_low lat_high lon_low lon_hi < alt_map_data > out    */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

static int status,flag;          /* eof? and misc info */
static long ssize,fno;           /* number of elements to send */
static char buffer[HBMAX][LINE]; /* the cdf header */
static float *sbuff,*vbuff;      /* buffer for static and variable data */
static float bad,badlim;         /* values for bad and badlim */
static short row[8471][121];     /* contains 121 rows of lat */
static unsigned short *map;
static unsigned short map2[14675];


main(argc,argv)
int argc;
char *argv[];
{
  int nmrd,loop1,loop2,x,y,a,b;  /* misc. looping and return values */
  int nlon,nlat;                 /* size of lat and lon */
  int latb,lonb;                 /* lat and lon value of sw corner of block */
  int latst,latend,lonst,lonend; /* range of lat and lon */
  int off,loc,lastlatw;          /* misc (offsets) */
  long nels,nelv;                /* number of elements in slices */
  char cl[LINE];                 /* comments and misc */
  float *dout;                   /* pointers for data position */
  float latl,lath,latt;          /* low lat and high lat */
  float lonl,lonh,temp;          /* same for lon */
  unsigned short a1,a2,a3,a4;

/* get command line arguments */
  if (argc != 5) {
    fprintf(stderr,"usage: cdfaltmap lat_low lat_high long_low long_high\n");
    exit(1);
  }
  latl=atof(argv[1]);
  lath=atof(argv[2]);
  lonl=atof(argv[3]);
  lonh=atof(argv[4]);
/* check values of lats and longs to insure they are within boundries */
  if ((latl<23.0)||(latl>51.0)||(lath<23.0)||(lath>51.0)||(lonl < -130.0)||
      (lonl > -60.0)||(lonh < -130.0)||(lonh > -60.0)) {
    fprintf(stderr,"cdfaltmap: error with area to extract\n");
    fprintf(stderr,"           -130 < longitude < -60\n");
    fprintf(stderr,"            23  < latitude  < 51\n");
    exit(1);
  }
  if (latl > lath) {
    temp=latl;
    latl=lath;
    lath=temp;
  }
  if (lonl > lonh) {
    temp=lonl;
    lonl=lonh;
    lonh=temp;
  }
/* set values for bad and badlim */
  bad = 1000.0;
  badlim = 999.0;
/* find the size of each 1x1 degree block and allocate memory for it */
  scanf("%8ld",&fno);
  map=(unsigned short*)malloc((fno/2)*sizeof(unsigned short));
/* make a header buffer */
  nullhdr(buffer,HBMAX,"float");
  addcline(buffer,HBMAX,"cdfaltmap:\n");
  sprintf(cl,"  %s %s %s %s\n",argv[1],argv[2],argv[3],argv[4]);
  addcline(buffer,HBMAX,cl);
  addpar(buffer,HBMAX,"bad","1000.0");
  addpar(buffer,HBMAX,"badlim","999.0");
  loc=(int) ((lonl+130.0)*120.0);
  lonl=(float) (loc/120.0)-130.0;
  sprintf(cl,"%f\0",lonl);
  addpar(buffer,HBMAX,"lon0",cl);
  sprintf(cl,"%f\0",(1.0/120.0));
  addpar(buffer,HBMAX,"dlon",cl);
/* determine the number of latitudes and longitudes */
  latst=(int) (((latl-23.0)*120.0)+.5);
  latend=(int) (((lath-23.0)*120.0)+.5);
  lonst=(int) (((lonl+130.0)*120.0)+.5);
  lonend=(int) (((lonh+130.0)*120.0)+.5);
  nlon=lonend-lonst+1;
  nlat=latend-latst+1;
  lastlatw=latst-1;
/* add field names */
  addfld(buffer,HBMAX,'s',"lon",20.0,4000.0,'s',1,"lon",nlon);
  addfld(buffer,HBMAX,'v',"lat",20.0,4000.0,'s',0);
  addfld(buffer,HBMAX,'v',"alt",20.0,4000.0,'s',1,"lon",nlon);
/* send header to stdout */
  puthdr(stdout,buffer,HBMAX);
/* make the static slice and variable slice */
  nels=elemcnt(buffer,HBMAX,'s');
  sbuff=getbuff(nels);
  nelv=elemcnt(buffer,HBMAX,'v');
/* place data into static slice and output it */
  dout=getptr(buffer,HBMAX,sbuff,'s','d',"lon");
  for (loop1=0;loop1<nlon;loop1++)
    dout[loop1] = ((loop1+lonst)/120.0)-130.0;
  putslice(stdout,nels,sbuff);
  vbuff=getbuff(nelv);
/* set up the array (row[x][y]) that holds the line of 1x1 blocks */
  crow(lonst,lonend+1);
  flag=0;
  nmrd=fread(map,sizeof(unsigned short),fno/2,stdin);
  while ((nmrd != 0)&&(flag != -1)) {
    scanf("%8ld",&fno);
/* get the lat and lon the the 1x1 block (sw corner) */
    latb=map[0]-90;
    lonb=map[1]-360;
    if (latb > lath)
      flag = -1;
/* make sure that the 1x1 block is with the given paramaters */
/* if it is then place data in row[x][y] else skip it */
    loc=(int) ((lonb+130.0)*120.0+.5);
    if (((latb+1) > latl)&&(latb < lath)&&((lonb+1) > lonl)&&(lonb < lonh)) {
      b=0;
      map2[b++]=(getbits(map[10],7,8) << 4)+getbits(map[11],15,4);
      map2[b++]=getbits(map[11],11,12);
      for (a=12;a<10993;a += 3) {
        unpack(map[a],map[a+1],map[a+2],&a1,&a2,&a3,&a4);
        map2[b++]=a1;
        map2[b++]=a2;
        map2[b++]=a3;
        map2[b++]=a4;
      }
/* transfer the data in map2 to row[x][y] */
      for (y=0;y<120;y++) {
        for (x=0;x<120;x++) {
          off=x+y*121;
          row[x+loc][y]=map2[off];
        }
      }
    }
/* get the next block */
    nmrd=fread(map,sizeof(unsigned short),fno/2,stdin);
    if (nmrd != 0)
      loc=map[0]-90;
/* if the latitude of the next 1x1 block is not the same as previous one */
/* then output the ones currently in row then continue */
    if (loc != latb) {
      dout=getptr(buffer,HBMAX,vbuff,'v','d',"alt");
      for (y=0;y<120;y++) {
        loop2=0;
        latt=(float) (latb+y/120.0);
        off=(latb-23)*120+y;
        if (((lastlatw+1) < off)&&(latt <= lath)) {
          while ((off-1) > lastlatw) {
            buff((lastlatw/120.0+23.0),nlon+1,nelv,HBMAX);
            lastlatw++;
          }
        }
        if (latt > lath)
          flag = -1;
        if ((latt >= latl)&&(lath >= latt)&&(off != lastlatw)) {
          for (x=lonst;x<(nlon+lonst);x++) {
            if (row[x][y] == 0)
              dout[loop2]=bad;
            else
              dout[loop2] = (float) (row[x][y]*20.0-4000.0)/3281.0;
            loop2++;
          }
          dout=getptr(buffer,HBMAX,vbuff,'v','d',"lat");
          dout[0]=latt;
          putslice(stdout,nelv,vbuff);
          lastlatw++;
        }
      }
      crow(lonst,lonend+1);
    }
  }
  exit(0);
}

/* extract 16bit shorts from the 12bit data */
unpack(s1,s2,s3,el1,el2,el3,el4)
unsigned short s1,s2,s3;
unsigned short *el1,*el2,*el3,*el4;
{
  *el1 = getbits(s1,15,12);
  *el2 = (getbits(s1,3,4) << 8)+getbits(s2,15,8);
  *el3 = (getbits(s2,7,8) << 4)+getbits(s3,15,4);
  *el4 = getbits(s3,11,12);
  return(OK);
}

/* get sets of bits */
getbits(x,p,n)
unsigned short x,p,n;
{
  return((x >> (p+1-n)) & ~(~0 << n));
}

/* clear row */
crow(x1,x2)
int x1,x2;
{
  int lx,ly;

  for (lx=x1;lx<x2;lx++) {
    for (ly=0;ly<121;ly++)
      row[lx][ly] = 0;
  }
  return(OK);
}

/* if a line of constant latitude is missing then send a slice w/bad values */
buff(lat,nl,nel,max)
int max,nl;
long nel;
float lat;
{
  int l;
  float *di;
  
  di=getptr(buffer,max,vbuff,'v','d',"alt");
  for (l=0;l<nl;l++)
    di[l]=bad;
  di=getptr(buffer,max,vbuff,'v','d',"lat");
    di[0]=lat;
  putslice(stdout,nel,vbuff);
  return(OK);
}
