/* cdfdb -- This program converts the specified field to or from
 * decibel representation.
 */

#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"
#define TINY 1.e-30

static char hbuff[HBMAX][LINE];        /* header buffer */
static char cline[LINE];               /* general character line */
static long nsbuff,nvbuff;             /* lengths of slice buffers */
static float *sbuff,*vbuff;            /* slice buffers */
static float *modfld;                  /* field to be modified */
static int decibel;        /* flag indicating to decibel conversion */
static float scale;                    /* scale factor */
static float temp;                     /* float temporary */
static float bad,badlim;               /* bad data parameters */
static struct field *fp;               /* field structure pointer */
static long loop;                      /* general loping variable */

main(argc,argv)
int argc;
char *argv[];
{
/* check arguments */
  decibel = -1;
  if (argc > 1) {
    if (strcmp(argv[1],"-l") == 0) decibel = 0;
    else if (strcmp(argv[1],"-d") == 0) decibel = 1;
  }
  if ((argc != 4) || (decibel < 0)) {
    fprintf(stderr,"Usage: cdfdb -[ld] field scale_factor\n");
    exit(1);
  }
  scale = atof(argv[3]);
/* get header */
  gethdr(stdin,hbuff,HBMAX);
  if (getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"cdfdb: float format expected\n");
    exit(1);
  }
/* get bad data parameters */
  if (seekpar(hbuff,HBMAX,"bad",cline) == FAIL) bad = BAD;
  else bad = atof(cline);
  if (seekpar(hbuff,HBMAX,"badlim",cline) == FAIL) badlim = BADLIM;
  else badlim = atof(cline);
/* add comment */
  sprintf(cline,"%s: %s %s %s\n",argv[0],argv[1],argv[2],argv[3]);
  addcline(hbuff,HBMAX,cline);
/* put header */
  puthdr(stdout,hbuff,HBMAX);
/* allocate buffers and pointers */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  sbuff = getbuff(nsbuff);
  nvbuff = elemcnt(hbuff,HBMAX,'v');
  vbuff = getbuff(nvbuff);
  if ((modfld = getptr2(hbuff,HBMAX,vbuff,'v','r',argv[2],&fp)) == NULL) {
    fprintf(stderr,"cdfdb: can't find field %s\n",argv[2]);
    exit(1);
  }
/* transfer static slice */
  getslice(stdin,nsbuff,sbuff);
  putslice(stdout,nsbuff,sbuff);
/* loop on variable slices */
  while (getslice(stdin,nvbuff,vbuff) != EOF) {
/* make the desired modification to the specified field */
    for (loop = 0; loop < fp->tsize; loop++) {
/* do nothing if bad data */
      if (fabs(modfld[loop]) > badlim);
/* if decibel, convert linear value to decibel value */
      else if (decibel) {
        temp = fabs(modfld[loop]);
        if (temp < TINY) temp = TINY;
        modfld[loop] = 10.*(log10(temp) + scale);
      }
/* otherwise, convert decibel value to linear value */
      else {
        modfld[loop] = pow(10.,modfld[loop]/10. - scale);
      }
    }
/* write the variable slice */
    putslice(stdout,nvbuff,vbuff);
  }
  exit(0);
}
