/* cdfderiv -- Take the derivative of a specified field with respect
 * to a specified index field.
 */

#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"
#define TINY 1.e-30

static char *hbuff[HBMAX][LINE];  /* header buffer */
static char cline[LINE];          /* character line buffer */
static long nsbuff,nvbin,nvbout;  /* slice buffer sizes */
static float *sbuff,*vbuff;       /* slice buffer pointers */
static float *ifield;             /* pointer to specified index field */
static float *sfield;             /* pointer to specified field */
static float *dfield;             /* pointer to derivative field */
static long instance;             /* looping var over instances */
static long loop;                 /* the other looping variable */
static struct field *fp;          /* field structure temporary pointer */
static struct field *fps;         /* field structure pointer for specified
                                     field */
static float bad,badlim;          /* bad data values */
static long start,incr,size;      /* loop variables */
static float den;                 /* derivative temporaries */
static long i1,i2,ix1,ix2;        /* index temporaries */


main(argc,argv)
int argc;
char *argv[];
{
/* check arguments */
  if (argc != 4) {
    fprintf(stderr,"Usage: cdfderiv derived_field input_field dimension\n");
    exit(1);
  }
/* get header */
  gethdr(stdin,hbuff,HBMAX);
  if (getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"cdfderiv: float format expected\n");
    exit(1);
  }
/* get bad data values */
  if (seekpar(hbuff,HBMAX,"bad",cline) == OK) bad = atof(cline);
  else bad = BAD;
  if (seekpar(hbuff,HBMAX,"badlim",cline) == OK) badlim = atof(cline);
  else badlim = BADLIM;
/* compute input buffer lengths */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  nvbin = elemcnt(hbuff,HBMAX,'v');
/* add stuff to header, comment first */
  sprintf(cline,"%s: %s %s %s\n",argv[0],argv[1],argv[2],argv[3]);
  addcline(hbuff,HBMAX,cline);
/* derivative field next -- same dimensions as specified field */
  if ((fp = seekfld(hbuff,HBMAX,'v',argv[2])) == NULL) {
    fprintf(stderr,"cdfderiv: %s field not found in variable slice\n",argv[2]);
    exit(1);
  }
  addfld(hbuff,HBMAX,'v',argv[1],fp->smul,fp->sadd,fp->bytes,fp->dim,
       fp->dname1,fp->dsize1,
       fp->dname2,fp->dsize2,
       fp->dname3,fp->dsize3,
       fp->dname4,fp->dsize4);
/* allocate buffers */
  nvbout = elemcnt(hbuff,HBMAX,'v');
  sbuff = getbuff(nsbuff);
  vbuff = getbuff(nvbout);
/* assign field and structure pointers */
  sfield = getptr2(hbuff,HBMAX,vbuff,'v','d',argv[2],&fps);
  dfield = getptr(hbuff,HBMAX,vbuff,'v','d',argv[1]);
  if ((ifield = getptr2(hbuff,HBMAX,sbuff,'s','r',argv[3],&fp)) == NULL) {
    fprintf(stderr,"cdfderiv: %s field doesn't exist\n",argv[3]);
    exit(1);
  }
  if ((fp->dim != 1) || (strcmp(fp->fname,fp->dname1) != 0)) {
    fprintf(stderr,"cdfderiv: %s field is not an index field\n",argv[3]);
    exit(1);
  }
/* write header */
  puthdr(stdout,hbuff,HBMAX);
/* transfer static slice */
  getslice(stdin,nsbuff,sbuff);
  putslice(stdout,nsbuff,sbuff);
/* loop on variable slices */
  while (getslice(stdin,nvbin,vbuff) != EOF) {
/* loop on subspace instances */
    instance = 0;
    while (subspace1(fps,argv[3],instance,&start,&incr,&size) != FAIL) {
/* loop over points and do centered differences except on ends */
      for (loop = 0; loop < size; loop++) {
	if (loop == 0) {
	  i1 = start + incr*loop;
	  i2 = start + incr*(loop + 1);
	  ix1 = loop;
	  ix2 = loop + 1;
	}
	else if (loop == size - 1) {
	  i2 = start + incr*loop;
	  i1 = start + incr*(loop - 1);
	  ix2 = loop;
	  ix1 = loop - 1;
	}
	else {
	  i1 = start + incr*(loop - 1);
	  i2 = start + incr*(loop + 1);
	  ix1 = loop - 1;
	  ix2 = loop + 1;
	}
	den = ifield[ix2] - ifield[ix1];
	if (fabs(den) < TINY) divbyzero(instance);
	if (fabs(sfield[i1]) < badlim && fabs(sfield[i2]) < badlim) {
	  dfield[start + incr*loop] = (sfield[i2] - sfield[i1])/den;
	}
	else {
	  dfield[start + incr*loop] = bad;
	}
      }
/* increment instance */
      instance++;
    }
/* write variable slice */
    putslice(stdout,nvbout,vbuff);
  }
  exit(0);
}

divbyzero(instance)
int instance;
{
  fprintf(stderr,"cdfderiv: zero index field interval -- calc %d\n",instance);
  exit(1);
}
