/* cdfextr -- This program extracts selected entrys from the parameter,
 * static field, or variable field sections of a common data format
 * file.  Bigger than standard Candis headers can be read, but not written.
 *
 * new addition -- the -r option allows cdfextr to pass fields except the
 * named fields.
 */

#include <stdio.h>
#include "cdfhdr.h"
#define HBBIG 2*HBMAX

static char hbin[HBBIG][LINE],hbout[HBMAX][LINE];  /* header buffers */
static long nsbuffin,nsbuffout,nvbuffin,nvbuffout; /* slice buffer lengths */
static float *sbuffin,*sbuffout,*vbuffin,*vbuffout; /* slice buffer pointers */
static char pname[LINE],pval[LINE];             /* parameter stuff */
static float *vptrin[HBBIG],*vptrout[HBMAX];    /* variable field pointers */
static float *sptrin,*sptrout;                  /* static field pointers */
static long ssize,vsize[HBBIG];                 /* associated field lengths */
static int numfields;                           /* number of fields */
static int goop,loop;                           /* general looping variables */
static int pflag,sflag;                         /* parameter or static field */
static int vflag,i,j,found;
static struct field *fp;                        /* field pointer */
static char testnm[LINE],testval[LINE];

main(argc,argv)
int argc;
char *argv[];
{
/* check arguments */
  if (argc < 2) {
    fprintf(stderr,"Usage: cdfextr [-rps] entry1 entry2 ...\n");
    exit(1);
  }
  vflag = pflag = sflag = 0;
  /* check if the first char of the first argument is a dash */
  if(argv[1][0] == '-'){
    /* check for the options */
    if(argv[1][1] == 'r' || argv[1][2] == 'r') vflag = 1 ;
    if(argv[1][1] == 'p' || argv[1][2] == 'p') pflag = 1 ;
    if(argv[1][1] == 's' || argv[1][2] == 's') sflag = 1 ;
    if(pflag && sflag){
      fprintf(stderr,"%s : options -p and -s cannot coexist \n",argv[0]);
      exit(0);
    }
  }

/* get header */
  gethdr(stdin,hbin,HBBIG);
/* check for float format */
  if (getfmt(hbin,HBBIG) != 'f') {
    fprintf(stderr,"cdfextr: expecting float format\n");
    exit(1);
  }
/* make output header -- null header first */
  nullhdr(hbout,HBMAX,"float");
/* copy comments */
  copycmt(hbout,HBMAX,hbin,HBBIG);
/* if p flag, copy just those parameters of interest */
  if (pflag && !(vflag)) {
    loop = 2;
    while (loop < argc) {
      if (seekpar(hbin,HBBIG,argv[loop],pval) == FAIL) {
        fprintf(stderr,"cdfextr: can't find parameter %s\n",argv[loop]);
        exit(1);
      }
      addpar(hbout,HBMAX,argv[loop++],pval);
    }
  }
  /* check and see if the p and v flags are on */
  else if(pflag && vflag){
    found = i = 0; 
    while(getpar(hbin,HBBIG,i++,testnm,testval) != FAIL){
      found = 0;
      /* see if we match any of the  comand line arguments */
      for(j = 1+(pflag || vflag || sflag) ; j < argc ; j++ )
         if(strcmp(testnm,argv[j]) == 0 ){ 
           found = 1;
           break ;
         }
      /* otherwise add the perameter */
      if(!(found)) 
        addpar(hbout,HBMAX,testnm,testval);
    }
  } /* end of the p and v case */
  /* if not, copy whole mess */
  else copypar(hbout,HBMAX,hbin,HBBIG);
/*if s flag, copy static field entries of interest */
  if (sflag && !(vflag)) {
    loop = 2;
    while (loop < argc) {
      if ((fp = seekfld(hbin,HBBIG,'s',argv[loop])) == NULL) {
        fprintf(stderr,"cdfextr: can't find static field %s\n",argv[loop]);
        exit(1);
      }
      addfld(hbout,HBMAX,'s',argv[loop++],fp->smul,fp->sadd,fp->bytes,
           fp->dim,fp->dname1,fp->dsize1,fp->dname2,fp->dsize2,
           fp->dname3,fp->dsize3,fp->dname4,fp->dsize4);
    }
  }
  else if(sflag && vflag) {
     i = 0;
     while(fp = getfld(hbin,HBBIG,'s',i++)){
       found = 0;
       for(j = 1+(vflag || sflag || pflag); j < argc; j++)
          if(strcmp(fp->fname,argv[j]) == 0 ) {
            found = 1;
            break ;
          }
       if(!(found))addfld(hbout,HBMAX,'s',fp->fname,fp->smul,fp->sadd,fp->bytes,
         fp->dim,fp->dname1,fp->dsize1,fp->dname2,fp->dsize2,fp->dname3,
         fp->dsize3,fp->dname4,fp->dsize4);
     }
  } /* end of vflag and sflag if statement */
/* if not, all static fields get copied */
  else copyfld(hbout,HBMAX,hbin,HBBIG,'s');
/* if p or s flag, copy all variable fields */
  if(pflag || sflag) copyfld(hbout,HBMAX,hbin,HBBIG,'v');
/* if not, copy selected variable fields */
  else if(!(vflag)) {
    loop = 1;
    while (loop < argc) {
      if ((fp = seekfld(hbin,HBBIG,'v',argv[loop])) == NULL) {
        fprintf(stderr,"cdfextr: can't find variable field %s\n",argv[loop]);
        exit(1);
      }
      addfld(hbout,HBMAX,'v',argv[loop++],fp->smul,fp->sadd,fp->bytes,
           fp->dim,fp->dname1,fp->dsize1,fp->dname2,fp->dsize2,
           fp->dname3,fp->dsize3,fp->dname4,fp->dsize4);
    }
  }
  else  {
     found = i = 0;
     while(fp = getfld(hbin,HBBIG,'v',i++)){
       found = 0;
       for(j = 1+(vflag || sflag || pflag); j < argc; j++)
          if(strcmp(fp->fname,argv[j]) == 0 ) {
            found = 1;
            break ;
          }
       if(!found)addfld(hbout,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,
         fp->dim,fp->dname1,fp->dsize1,fp->dname2,fp->dsize2,fp->dname3,
         fp->dsize3,fp->dname4,fp->dsize4);
     }
  } /* end of the vflag case */

/* write the new header buffer */
  puthdr(stdout,hbout,HBMAX);
/* get space for static slice input and get static slice */
  nsbuffin = elemcnt(hbin,HBBIG,'s');
  sbuffin = getbuff(nsbuffin);
  getslice(stdin,nsbuffin,sbuffin);
/* if s flag, transfer selected fields in static slice */
  if (sflag) {
/* first, allocate static slice output buffer */
    nsbuffout = elemcnt(hbout,HBMAX,'s');
    sbuffout = getbuff(nsbuffout);
/* march through fields one by one */
    loop = 0;
    while ((fp = getfld(hbout,HBMAX,'s',loop++)) != NULL) {
/* st up pointers to input and output areas */
      sptrout = sbuffout + fp->ptr;
      ssize = fp->tsize;
      sptrin = getptr(hbin,HBBIG,sbuffin,'s','d',fp->fname);
/* transfer data */
      for (goop = 0; goop < ssize; goop++) sptrout[goop] = sptrin[goop];
    }
/* write the static slice */
    putslice(stdout,nsbuffout,sbuffout);
  }
/* otherwise just write the input slice */
  else putslice(stdout,nsbuffin,sbuffin);
/* allocate space for input variable slice */
  nvbuffin = elemcnt(hbin,HBBIG,'v');
  vbuffin = getbuff(nvbuffin);
/* if p or s flags set, just transfer all variable slices */
  if (pflag || sflag) {
    while (getslice(stdin,nvbuffin,vbuffin) != EOF) {
      putslice(stdout,nvbuffin,vbuffin);
    }
  }
/* otherwise, we have to transfer only selected fields */
  else {
/* first allocate space for output slice buffer */
    nvbuffout = elemcnt(hbout,HBMAX,'v');
    vbuffout = getbuff(nvbuffout);
/* next make a list of pointers and lengths */
    loop = 0;
    while ((fp = getfld(hbout,HBMAX,'v',loop)) != NULL) {
      vptrout[loop] = vbuffout + fp->ptr;
      vsize[loop] = fp->tsize;
      vptrin[loop++] = getptr(hbin,HBBIG,vbuffin,'v','d',fp->fname);
    }
    numfields = loop;
/* now loop on variable slices */
    while (getslice(stdin,nvbuffin,vbuffin) != EOF) {
/* loop finally on selected fields */
      for (loop = 0; loop < numfields; loop++) {
/* transfer field */
        for (goop = 0; goop < vsize[loop]; goop++) {
          vptrout[loop][goop] = vptrin[loop][goop];
        }
      }
/* write the new variable slice */
      putslice(stdout,nvbuffout,vbuffout);
    }
  }
  exit(0);
}
