/* cdfgnuplot.c -- Create an ASCII file suitable for input to the Gnuplot
 * plotting program.
 */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

static char hbuff[HBMAX][LINE];
static char pval[LINE];
static float *sbuff,*vbuff;
static long nsbuff,nvbuff;
static float *x,*y,*z;
static int plot;
static long i,ix,iy;
static char xtype,ytype;
static struct field *fp,fx,fy,fz;
static float badlim;

main(argc,argv)
int argc;
char *argv[];
{

/* check args */
  if (argc < 2 || argc > 3) {
    fprintf(stderr,
         "Usage: cdfgnuplot xfield yfield (2 1-D fields for plot)\n");
    fprintf(stderr,"                     -- or --\n");
    fprintf(stderr,"       cdfgnuplot field (1 2-D field for splot)\n");
    exit(1);
  }
  plot = (argc == 3);

/* get header */
    gethdr(stdin,hbuff,HBMAX);
  if (getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"cdfgnuplot: float format expected\n");
    exit(1);
  }

/* get bad data value */
  if (seekpar(hbuff,HBMAX,"badlim",pval) == OK) badlim = atof(pval);
  else badlim = BADLIM;

/* look for the fields */
  if (plot) {

/* plot format */
    xtype = 'v';
    if ((fp = seekfld(hbuff,HBMAX,'v',argv[1])) == NULL) {
      xtype = 's';
      if ((fp = seekfld(hbuff,HBMAX,'s',argv[1])) == NULL) {
        fprintf(stderr,"cdfgnuplot: can't find field %s\n",argv[1]);
        exit(1);
      }
    }
    fx = *fp;
    if (fx.dim != 1) {
      fprintf(stderr,"cdfgnuplot: field %s not 1-D\n",argv[1]);
      exit(1);
    }
    ytype = 'v';
    if ((fp = seekfld(hbuff,HBMAX,'v',argv[2])) == NULL) {
      ytype = 's';
      if ((fp = seekfld(hbuff,HBMAX,'s',argv[2])) == NULL) {
        fprintf(stderr,"cdfgnuplot: can't find field %s\n",argv[2]);
        exit(1);
      }
    }
    fy = *fp;
    if (fy.dim != 1) {
      fprintf(stderr,"cdfgnuplot: field %s not 1-D\n",argv[2]);
      exit(1);
    }
    if (fx.dsize1 != fy.dsize1 || strcmp(fx.dname1,fy.dname1)) {
      fprintf(stderr,"cdfgnuplot: fields %s and %s don't have same dimension\n",
           argv[1],argv[2]);
      exit(1);
    }
  }
  else {

/* splot format */
    if ((fp = seekfld(hbuff,HBMAX,'v',argv[1])) == NULL) {
      fprintf(stderr,"cdfgnuplot: can't find field %s\n",argv[1]);
      exit(1);
    }
    fz = *fp;
    if (fz.dim != 2) {
      fprintf(stderr,"cdfgnuplot: field %s not 2-D\n",argv[1]);
      exit(1);
    }
    if ((fp = seekfld(hbuff,HBMAX,'s',fz.dname1)) == NULL) {
      fprintf(stderr,"cdfgnuplot: can't find index field %s\n",fz.dname1);
      exit(1);
    }
    fx = *fp;
    if ((fp = seekfld(hbuff,HBMAX,'s',fz.dname2)) == NULL) {
      fprintf(stderr,"cdfgnuplot: can't find index field %s\n",fz.dname2);
      exit(1);
    }
    fy = *fp;
  }

/* get the slice buffers */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  nvbuff = elemcnt(hbuff,HBMAX,'v');
  sbuff = getbuff(nsbuff);
  vbuff = getbuff(nvbuff);
  if (getslice(stdin,nsbuff,sbuff) == EOF) {
    fprintf(stderr,"cdfgnuplot: EOF encountered while reading static slice\n");
    exit(1);
  }
  if (getslice(stdin,nvbuff,vbuff) == EOF) {
    fprintf(stderr,
         "cdfgnuplot: EOF encountered while reading variable slice\n");
    exit(1);
  }

/* get the pointers and go */
  if (plot) {

/* plot format */
    if (xtype == 'v') x = vbuff + fx.ptr;
    else x = sbuff + fx.ptr;
    if (ytype == 'v') y = vbuff + fy.ptr;
    else y = sbuff + fy.ptr;
    printf("# cdfgnuplot %s %s (1-D plot)\n",argv[1],argv[2]);
    for (i = 0; i < fx.tsize; i++) {
      if (fabs(x[i]) < badlim && fabs(y[i]) < badlim) {
        printf("%e    %e\n",x[i],y[i]);
      }
      else printf("\n");
    }
  }
  else {

/* splot format */
    z = vbuff + fz.ptr;
    x = sbuff + fx.ptr;
    y = sbuff + fy.ptr;
    printf("# cdfgnuplot %s (%s %s) (2-D plot)\n",argv[1],fx.fname,fy.fname);
    for (iy = 0; iy < fy.tsize; iy++) {
      for (ix = 0; ix < fx.tsize; ix++) {
        i = iy + fy.tsize*ix;
        if (fabs(z[i]) < badlim) {
          printf("%e    %e    %e\n",x[ix],y[iy],z[i]);
        }
        else printf("\n");
      }
      printf("\n");
    }
  }
}
