/* cdfinteg -- Take the integral of a specified field with respect
 * to a specified index field.
 */

#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

static char *hbuff[HBMAX][LINE];     	/* header buffer */
static char cline[LINE];               	/* character line buffer */
static long nsbuff,nvbin,nvbout;       	/* slice buffer sizes */
static float *sbuff,*vbuff;            	/* slice buffer pointers */
static float *ifield;                  	/* pointer to specified index field */
static float *sfield;                  	/* pointer to specified field */
static float *dfield;                  	/* pointer to integral field */
static long instance;                  	/* looping var over instances */
static long loop;                      	/* the other looping variable */
static struct field *fp;               	/* field structure temporary pointer */
static struct field *fps;              	/* field structure pointer for
					   specified field */
static float bad,badlim;               	/* bad data values */
static long start,incr,size;           	/* loop variables */
static int flag;                        /* up (+1) or down (-1) indicator */
static long index;                      /* field looping variable */
static double sum;                      /* integration temporary */


main(argc,argv)
int argc;
char *argv[];
{
/* check arguments */
  if (argc != 5) {
    fprintf(stderr,"Usage: cdfinteg -[tb] integrated_fld field dimension\n");
    exit(1);
  }

/* get header */
  gethdr(stdin,hbuff,HBMAX);
  if (getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"cdfinteg: float format expected\n");
    exit(1);
  }

/* get the flag */
  if (strcmp(argv[1],"-b") != 0 || strcmp(argv[1],"-t") != 0)
    flag = (strcmp(argv[1],"-b") == 0) ? 1 : -1;
  else {
    fprintf(stderr,"cdfinteg: flag type is not known\n");
    exit(1);
  }

/* get bad data values */
  bad = (seekpar(hbuff,HBMAX,"bad",cline) == OK)  ? bad = atof(cline) : BAD;
  badlim = (seekpar(hbuff,HBMAX,"badlim",cline) == OK) ? atof(cline):BADLIM;

/* compute input buffer lengths */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  nvbin = elemcnt(hbuff,HBMAX,'v');

/* add stuff to header, comment first */
  sprintf(cline,"cdfinteg: %s\n",argv[1]);
  addcline(hbuff,HBMAX,cline);
  sprintf(cline,"  %s %s %s\n",argv[2],argv[3],argv[4]);
  addcline(hbuff,HBMAX,cline);

/* integral field next -- same dimensions as specified field */
  if ((fp = seekfld(hbuff,HBMAX,'v',argv[3])) == NULL) {
    fprintf(stderr,"cdfinteg: %s field not found in variable slice\n",argv[3]);
    exit(1);
  }
  addfld(hbuff,HBMAX,'v',argv[2],fp->smul,fp->sadd,fp->bytes,fp->dim,
       fp->dname1,fp->dsize1,fp->dname2,fp->dsize2,
       fp->dname3,fp->dsize3,fp->dname4,fp->dsize4);

/* allocate buffers */
  nvbout = elemcnt(hbuff,HBMAX,'v');
  sbuff = getbuff(nsbuff);
  vbuff = getbuff(nvbout);

/* assign field and structure pointers */
  sfield = getptr2(hbuff,HBMAX,vbuff,'v','d',argv[3],&fps);
  dfield = getptr(hbuff,HBMAX,vbuff,'v','d',argv[2]);
  if ((ifield = getptr2(hbuff,HBMAX,sbuff,'s','r',argv[4],&fp)) == NULL) {
    fprintf(stderr,"cdfinteg: %s field doesn't exist\n",argv[4]);
    exit(1);
  }
  if ((fp->dim != 1) || (strcmp(fp->fname,fp->dname1) != 0)) {
    fprintf(stderr,"cdfinteg: %s field is not an index field\n",argv[4]);
    exit(1);
  }

/* write header */
  puthdr(stdout,hbuff,HBMAX);

/* transfer static slice */
  getslice(stdin,nsbuff,sbuff);
  putslice(stdout,nsbuff,sbuff);

/* loop on variable slices */
  while (getslice(stdin,nvbin,vbuff) != EOF) {

/* loop on subspace instances */
    instance = 0;
    while (subspace1(fps,argv[4],instance,&start,&incr,&size) != FAIL) {

/* up or down? */
/* up! */
      if (flag == 1) {
        for (loop = 0; loop < size; loop++) {
          index = start + incr*loop;
          if (loop == 0 || fabs(sfield[index]) > badlim ||
               fabs(sfield[index - incr]) > badlim) sum = 0.;
          else {
            sum += .5*(sfield[index] + sfield[index - incr])*(ifield[loop] -
                 ifield[loop - 1]);
          }
          if (fabs(sfield[index]) > badlim) dfield[index] = bad;
          else dfield[index] = sum;
        }
      }

/* down! */
      else {
        for (loop = size - 1; loop >= 0; loop--) {
          index = start + incr*loop;
          if (loop == size - 1 || fabs(sfield[index]) > badlim ||
               fabs(sfield[index + incr]) > badlim) sum = 0.;
          else {
            sum += .5*(sfield[index] + sfield[index + incr])*(ifield[loop] -
                 ifield[loop + 1]);
          }
          if (fabs(sfield[index]) > badlim) dfield[index] = bad;
          else dfield[index] = sum;
        }
      }

/* increment instance */
      instance++;
    }
/* write variable slice */
    putslice(stdout,nvbout,vbuff);
  }
  exit(0);
}
