/* cdfinterp -- Interpolate on a dimension from irregular to regular
 * intervals.
 */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

static char hbin[HBMAX][LINE],hbout[HBMAX][LINE];  /* header buffers */
static char cline[LINE],pline[LINE];               /* line buffer */
static float *sbin,*sbout,*vbin,*vbout;            /* slice buffers */
static long nsbin,nsbout,nvbin,nvbout;             /* slice buffer lengths */
static struct field *fp;                          /* field pointer temporary */
static char *dimname;                              /* name of dimension */
static char ddimname[LINE],dimname0[LINE];         /* index parameter names */
static int newsize;                               /* size of new index field */
static long loop,poop,goop;                        /* looping indices */
static struct field fsan[HBMAX],fsout[HBMAX]; /* discriptors for static flds */
static struct field fvin[HBMAX],fvout[HBMAX];    /* discriptors for var flds */
static int modflg[HBMAX];                       /* set if field needs interp */
static int nsflds,nvflds;                          /* numbers of each */
static float val0,dval;                            /* index parameter values */
static float *inptr,*outptr;                    /* pointers to interp fields */
static float *inx,*outx;                         /* pointers to index fields */
static long instance;                            /* loop index for subspace1 */
static long instart,inincr,insize;             /* input subspace1 parameters */
static long outstart,outincr,outsize;         /* output subspace1 parameters */
static float el0,el1,x0,x1,x;                      /* temporaries for interp */
static float bad,badlim;                           /* bad data values */

main (argc,argv)
int argc;
char *argv[];
{

/* check args */
  if (argc != 5) {
    fprintf(stderr,"Usage: cdfinterp dimension val0 dval nval\n");
    exit(1);
  }
  dimname = argv[1];
  val0 = atof(argv[2]);
  dval = atof(argv[3]);
  newsize = atoi(argv[4]);

/* get a header from standard input */
  gethdr(stdin,hbin,HBMAX);
  if (getfmt(hbin,HBMAX) != 'f') {
    fprintf(stderr,"cdfinterp: float format expected\n");
    exit(1);
  }

/* get bad data values */
  if (seekpar(hbin,HBMAX,"bad",cline) == OK) bad = atof(cline);
  else bad = BAD;
  if (seekpar(hbin,HBMAX,"badlim",cline) == OK) badlim = atof(cline);
  else badlim = BADLIM;

/* check for existence of index field of interest */
  if ((fp = seekfld(hbin,HBMAX,'s',dimname)) == NULL) {
    fprintf(stderr,"cdfinterp: can't find index field %s\n",dimname);
    exit(1);
  }
  if (fp->dim != 1 || strcmp(fp->fname,fp->dname1) != 0) {
    fprintf(stderr,"cdfinterp: %s not an index field\n",dimname);
    exit(1);
  }

/* make a new header */
  nullhdr(hbout,HBMAX,"float");

/* comments */
  copycmt(hbout,HBMAX,hbin,HBMAX);
  sprintf(cline,"cdfinterp: %s %s %s %s\n",argv[1],argv[2],argv[3],argv[4]);
  addcline(hbout,HBMAX,cline);

/* edit parameters */
  sprintf(ddimname,"d%s",dimname);
  sprintf(dimname0,"%s0",dimname);
  loop = 0;
  while (getpar(hbin,HBMAX,loop++,cline,pline) == OK) {
    if (strcmp(ddimname,cline) != 0 && strcmp(dimname0,cline) != 0) {
      addpar(hbout,HBMAX,cline,pline);
    }
  }
  addpar(hbout,HBMAX,dimname0,argv[2]);
  addpar(hbout,HBMAX,ddimname,argv[3]);

/* static fields */
  nsflds = 0;
  while ((fp = getfld(hbin,HBMAX,'s',nsflds)) != NULL) {
    fsan[nsflds++] = *fp;
    if (strcmp(dimname,fp->fname) == 0) {
      addfld(hbout,HBMAX,'s',dimname,fp->smul,fp->sadd,fp->bytes,1,
           dimname,newsize);
    }
    else {
      addfld(hbout,HBMAX,'s',fp->fname,fp->smul,fp->sadd,fp->bytes,fp->dim,
           fp->dname1,fp->dsize1,
           fp->dname2,fp->dsize2,
           fp->dname3,fp->dsize3,
           fp->dname4,fp->dsize4);
    }
  }

/* variable fields */
  nvflds = 0;
  while ((fp = getfld(hbin,HBMAX,'v',nvflds)) != NULL) {
    fvin[nvflds] = *fp;
    modflg[nvflds] = 0;
    if (fp->dim > 0 && strcmp(fp->dname1,dimname) == 0) {
      fp->dsize1 = newsize;
      modflg[nvflds]++;
    }
    if (fp->dim > 1 && strcmp(fp->dname2,dimname) == 0) {
      fp->dsize2 = newsize;
      modflg[nvflds]++;
    }
    if (fp->dim > 2 && strcmp(fp->dname3,dimname) == 0) {
      fp->dsize3 = newsize;
      modflg[nvflds]++;
    }
    if (fp->dim > 3 && strcmp(fp->dname4,dimname) == 0) {
      fp->dsize4 = newsize;
      modflg[nvflds]++;
    }
    addfld(hbout,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,fp->dim,
           fp->dname1,fp->dsize1,
           fp->dname2,fp->dsize2,
           fp->dname3,fp->dsize3,
           fp->dname4,fp->dsize4);
    nvflds++;
  }

/* fill output data base */
  loop = 0;
  while ((fp = getfld(hbout,HBMAX,'s',loop)) != NULL) fsout[loop++] = *fp;
  loop = 0;
  while ((fp = getfld(hbout,HBMAX,'v',loop)) != NULL) fvout[loop++] = *fp;

/* make slice buffers */
  nsbin = elemcnt(hbin,HBMAX,'s');
  sbin = getbuff(nsbin);
  nsbout = elemcnt(hbout,HBMAX,'s');
  sbout = getbuff(nsbout);
  nvbin = elemcnt(hbin,HBMAX,'v');
  vbin = getbuff(nvbin);
  nvbout = elemcnt(hbout,HBMAX,'v');
  vbout = getbuff(nvbout);

/* write new header to standard output */
  puthdr(stdout,hbout,HBMAX);

/* read static slice from standard input */
  getslice(stdin,nsbin,sbin);

/* fill output static slice */
  for (loop = 0; loop < nsflds; loop++) {
    if (strcmp(fsout[loop].fname,dimname) == 0) {  /* fill in modified fld */
      for (poop = 0; poop < fsout[loop].tsize; poop++) {
        sbout[fsout[loop].ptr + poop] = val0 + dval*poop;
      }
    }
    else {                          /* unmodified field */
      for (poop = 0; poop < fsan[loop].tsize; poop++) {
        sbout[fsout[loop].ptr + poop] = sbin[fsan[loop].ptr + poop];
      }
    }
  }

/* write new static slice */
  putslice(stdout,nsbout,sbout);

/* loop on variable slices */
  while (getslice(stdin,nvbin,vbin) != EOF) {

/* transfer the data, interpolating if necessary */
    for (loop = 0; loop < nvflds; loop++) {
      if (modflg[loop]) {           /* need to interpolate */

/* set up direct pointers for convenience */
        inptr = vbin + fvin[loop].ptr;
        outptr = vbout + fvout[loop].ptr;
        inx = getptr(hbin,HBMAX,sbin,'s','d',dimname);
        outx = getptr(hbout,HBMAX,sbout,'s','d',dimname);

/* loop on instance */
        instance = 0;
        while (subspace1(&fvin[loop],dimname,instance,
             &instart,&inincr,&insize) != FAIL) {

/* get subspace1 parameters for output */
          subspace1(&fvout[loop],dimname,instance++,&outstart,&outincr,
               &outsize);

/* do the interpolation -- loop on output elements */
          poop = 1;
          for (goop = 0; goop < outsize; goop++) {

/* move input pointer up to just past current output point */
            while (poop < insize - 1) {
              if ((inx[poop] > inx[poop - 1] && outx[goop] > inx[poop]) ||
                   (inx[poop] < inx[poop - 1] && outx[goop] < inx[poop])) {
                poop++;
              }
              else break;
            }

/* for each output element, see if it is spanned by input elements */
            if ((outx[goop] <= inx[poop] && outx[goop] >= inx[poop - 1]) ||
                 (outx[goop] <= inx[poop - 1] && outx[goop] >= inx[poop])) {

/* we are spanned -- do the interpolation */
              x0 = inx[poop - 1];
              x1 = inx[poop];
	      el0 = inptr[instart + inincr*(poop - 1)];
	      el1 = inptr[instart + inincr*poop];

/* check for bad data -- if so, output bad data */
	      if (fabs(el0) > badlim || fabs(el1) > badlim) {
		outptr[outstart + outincr*goop] = bad;
	      }
	      else {
		x = outx[goop];
		outptr[outstart + outincr*goop] =
		  (el0*(x - x1) + el1*(x0 - x))/(x0 - x1);
              }
            }

/* no span on this element -- set to bad data value */
            else {
              outptr[outstart + outincr*goop] = bad;
            }
          }
        }
      }
      else {                        /* simple transfer */
        for (poop = 0; poop < fvin[loop].tsize; poop++) {
          vbout[fvout[loop].ptr + poop] = vbin[fvin[loop].ptr + poop];
        }
      }
    }

/* write the new variable slice */
    putslice(stdout,nvbout,vbout);
  }
  exit(0);
}
