/* cdfisocut                                                              */
/* cdfisocut {-t or -b} dimname testvar testvalue                         */
/* cdfisocut  examines the variable field testvar and looks for the       */
/* value (testvalue) across the dimension given (dimname) and cuts the    */
/* field at this place; this is done for every instance of that dimension */
/* Then similar fields are cut at the same location as the testvar was    */
/* cut.  The -t or -b flag means to start at top and go down (-t) or      */
/* go from the bottom up (-b) in searching for testvalue.  An extra       */
/* variable field is added call dimname that contains the value of dimname*/
/* at the point where it was cut from the testvar, thus the new field     */
/* and testvar are the same size                                          */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

static char buffer[HBMAX][LINE],bufferw[HBMAX][LINE];
                                    /* buffers for header data */
static float *sbuffin,*sbuffout;    /* buffers that contain slice data */
static float bad,badlim;            /* values for bad and badlim */

static float lineprox1();
static float lineprox2();

main(argc,argv)
int argc;
char *argv[];
{
  int loop,loop1,pos,vpos;        /* used in looping */
  int way,code,flag1,pt1,pt2;     /* top down or bottom up? & code for table */
  int dimtest,table[2][HBMAX][7]; /* number of dimensions of the testvar */
  long nel,nelw;                  /* # of elements to get and to write */
  long inst,start,incr,size;      /* used with the subspace1 routine */
  char dname[LINE],testvar[LINE]; /* names from command line */
  char flag[3],pvalue[LINE];      /* flag from command line */
  char field,cl[LINE];            /* which field to be reduced */
  float testval,*cut,*varval;     /* value at which level to extract */
  float value1,value2;            /* used in the linear approx of data */
  float *din,*dout;               /* pointers for data in and out */
  struct field *fp;               /* pointer to the field structure */
  struct field *fb;               /* pointer to the field structure */

/* check command line arguments */
  if (argc != 5) {
    fprintf(stderr,"Usage: cdfisocut -t dname testvar testval\n");
    fprintf(stderr,"  or   cdfisocut -b dname testvar testval\n");
    exit(1);
  }
/* move stuff out of argument line */
  strncpy(flag,argv[1],2);
  flag[2]='\0';
  strcpy(dname,argv[2]);
  strcpy(testvar,argv[3]);
  testval=atof(argv[4]);
/* get header from stdin */
  gethdr(stdin,buffer,HBMAX);
/* check format */
  if (getfmt(buffer,HBMAX) != 'f') {
    fprintf(stderr,"cdfisocut: file must be float!\n");
    exit(1);
  }
/* check names given on command line */
  if ((fp=seekfld(buffer,HBMAX,'v',testvar))==NULL) {
    fprintf(stderr,"cdfisocut: %s is not a valid variable field\n",testvar);
    exit(1);
  }
  dimtest=fp->dim;
  fp=seekfld(buffer,HBMAX,'s',dname);
  if ((fp==NULL)||(fp->dim != 1)||(strcmp(fp->fname,fp->dname1) != 0)) {
    fprintf(stderr,"cdfisocut: %s is not a valid index variable\n",dname);
    exit(1);
  }
/* check flags and find which method to use */
/* (i.e. top down or bottom up) */
  if (strcmp(flag,"-b")==0)
    way = -1;
  else if (strcmp(flag,"-t")==0)
    way = 1;
  else {
    fprintf(stderr,"cdfisocut: %s flag doen't exist\n",flag);
    exit(1);
  }
/* make output header */
  nullhdr(bufferw,HBMAX,"float");
/* transfer comments */
  copycmt(bufferw,HBMAX,buffer,HBMAX);
/* add comment line */
  sprintf(cl,"cdfisocut: %s %s\n",argv[1],argv[2]);
  addcline(bufferw,HBMAX,cl);
  sprintf(cl,"   %s %s\n",argv[3],argv[4]);
  addcline(bufferw,HBMAX,cl);
/* transfer parameters */
  copypar(bufferw,HBMAX,buffer,HBMAX);
  if (seekpar(buffer,HBMAX,"bad",pvalue)==FAIL)
    bad=BAD;
  else
    bad=atof(pvalue);
  if (seekpar(buffer,HBMAX,"badlim",pvalue)==FAIL)
    badlim=BADLIM;
  else
    badlim=atof(pvalue);
/* put data concerning static and variable fields into table */
  field = 's';
  for (loop=0;loop<2;loop++) {
    if (loop==1) field = 'v';
    pos=0;
/* cycle thru fields */
    while ((fp=getfld(buffer,HBMAX,field,pos))!=NULL) {
/* set up data for table */
      table[loop][pos][0]=fp->dim;
      for (loop1=1;loop1<5;loop1++)
        table[loop][pos][loop1]=0;
      table[loop][pos][5]=fp->tsize;
      code=0;
/* place dsize into table */
      if (fp->dim >=1) table[loop][pos][1]=fp->dsize1;
      if (fp->dim >=2) table[loop][pos][2]=fp->dsize2;
      if (fp->dim >=3) table[loop][pos][3]=fp->dsize3;
      if (fp->dim >=4) table[loop][pos][4]=fp->dsize4;
/* determine the code that goes with each field */
      if (strcmp(dname,fp->fname)==0)
        code=5;
      else if (strcmp(dname,fp->dname1)==0) {
        if (fp->dim==dimtest) code=1;
        else
          code=5;
      }
      else if (strcmp(dname,fp->dname2)==0) {
        if (fp->dim==dimtest) code=2;
        else
          code=5;
      }
      else if (strcmp(dname,fp->dname3)==0) {
        if (fp->dim==dimtest) code=3;
        else
          code=5;
      }
      else if (strcmp(dname,fp->dname4)==0) {
        if (fp->dim==dimtest) code=4;
        else
          code=5;
      }
      if (strcmp(fp->fname,testvar)==0)
        code += 5;
      if (fp->dim==0)
        code=0;
      table[loop][pos][6]=code;
      strcpy(fp->dname1,"\0");
      strcpy(fp->dname2,"\0");
      strcpy(fp->dname3,"\0");
      strcpy(fp->dname4,"\0");
      ++pos;
    }
  }
/* transfer the fields to new header */
  field = 's';
  for (loop1=0;loop1<2;loop1) {
    if (loop1==1) field = 'v';
    pos=0;
/* loop thru each field */
    while ((fp=getfld(buffer,HBMAX,field,pos))!=NULL) {
      code=table[loop1][pos][6];
      if (code > 5) code -= 5;
/* if code = 1st dim then ... */
      if (code==1) {
        strcpy(fp->dname1,fp->dname2);
        strcpy(fp->dname2,fp->dname3);
        strcpy(fp->dname3,fp->dname4);
        fp->dsize1=fp->dsize2;
        fp->dsize2=fp->dsize3;
        fp->dsize3=fp->dsize4;
      }
/* if 2nd ... */
      else if (code==2) {
        strcpy(fp->dname2,fp->dname3);
        strcpy(fp->dname3,fp->dname4);
        fp->dsize2=fp->dsize3;
        fp->dsize3=fp->dsize4;
      }
/* if 3 ... */
      else if (code==3) {
        strcpy(fp->dname3,fp->dname4);
        fp->dsize3=fp->dsize4;
      }
/* if no match then send w/o change */
      if (code==0) {
        addfld(bufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,fp->bytes,
               fp->dim,fp->dname1,fp->dsize1,fp->dname2,fp->dsize2,
               fp->dname3,fp->dsize3,fp->dname4,fp->dsize4);
      }
/* else send changes (if code = 5 then drop the entry) */
      else if (code != 5) {
        if (fp->dim==1) addfld(bufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,
            fp->bytes,0);
        if (fp->dim==2) addfld(bufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,
            fp->bytes,1,fp->dname1,fp->dsize1);
        if (fp->dim==3) addfld(bufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,
            fp->bytes,2,fp->dname1,fp->dsize1,fp->dname2,fp->dsize2);
        if (fp->dim==4) addfld(bufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,
            fp->bytes,3,fp->dname1,fp->dsize1,fp->dname2,fp->dsize2,fp->dname3,
            fp->dsize3);
      }
      ++pos;
    }
    ++loop1;
  }
/* add new field to the output header */
  fp=seekfld(bufferw,HBMAX,'v',testvar);
  addfld(bufferw,HBMAX,'v',dname,fp->smul,fp->sadd,fp->bytes,fp->dim,fp->dname1,
     fp->dsize1,fp->dname2,fp->dsize2,fp->dname3,fp->dsize3);
/* send the header to stdout */
  puthdr(stdout,bufferw,HBMAX);
/* allocate memory for and get the static slice */
  nel=elemcnt(buffer,HBMAX,'s');
  sbuffin=getbuff(nel);
  nelw=elemcnt(bufferw,HBMAX,'s');
  sbuffout=getbuff(nelw);
  getslice(stdin,nel,sbuffin);
/* transfer the static slice and send to stdout */
  vpos=0;
  while ((fp=getfld(bufferw,HBMAX,'s',vpos++))!=NULL) {
    din=getptr(buffer,HBMAX,sbuffin,'s','d',fp->fname);
    dout=getptr(bufferw,HBMAX,sbuffout,'s','d',fp->fname);
    for (loop=0;loop<fp->tsize;loop++)
      dout[loop]=din[loop];
  }
  putslice(stdout,nelw,sbuffout);
/* transfer data from the index field dname to an array for latter use */
  din=getptr2(buffer,HBMAX,sbuffin,'s','d',dname,&fp);
  varval=getbuff(fp->tsize);
  for (loop=0;loop<fp->tsize;loop++)
    varval[loop]=din[loop];
/* allocate memory for the  variable slice */
  nel=elemcnt(buffer,HBMAX,'v');
  sbuffin=getbuff(nel);
  nelw=elemcnt(bufferw,HBMAX,'v');
  sbuffout=getbuff(nelw);
/* find testvar and cut it out and set up the array that contains at what  */
/* values of dname to cut out the data (then go back and update the static */
/* field accordingly and the 1st var slice, 2nd var slice etc */
  while (getslice(stdin,nel,sbuffin)!=EOF) {
    din=getptr2(buffer,HBMAX,sbuffin,'v','d',testvar,&fp);
    dout=getptr2(bufferw,HBMAX,sbuffout,'v','d',testvar,&fb);
    cut=getbuff(fb->tsize);
/* cycle thru the dimensions */
    for (inst=0;inst<fb->tsize;inst++) {
/* find which instance to take */
      if (subspace1(fp,dname,inst,&start,&incr,&size)==FAIL) {
        fprintf(stderr,"cdfisocut: error with field %s\n",fp->fname);
        exit(1);
      }
      dout[inst]=cut[inst]=bad;
/* cut the testfield first and obtain values for which to cut other fields */
/* if the -t flag was set the cycle from top to bottom */
      if (way==1) {
        for (loop=size-1;loop>0;loop--) {
          value2=din[start+loop*incr];
          value1=din[start+(loop-1)*incr];
          if (mid(value1,testval,value2) != 0) {
            dout[inst]=testval;
            cut[inst]=lineprox1(value1,testval,value2,
                                (float)loop-1,(float)loop);
            break;
          }
        }
      }
/* if the -b flag was set then cylce from bottom to top */
      else if (way == -1) {
        for (loop=0;loop<size-1;loop++) {
          value1=din[start+loop*incr];
          value2=din[start+(loop+1)*incr];
          if (mid(value1,testval,value2) != 0) {
            dout[inst]=testval;
            cut[inst]=lineprox1(value1,testval,value2,
                                (float)loop,(float)loop+1);
            break;
          }
        }
      }
    }
    vpos=0;
/* cycle thru table getting info about fields */
    while ((fp=getfld(buffer,HBMAX,'v',vpos))!=NULL) {
      flag1=table[1][vpos][6];
      if ((strcmp(fp->fname,dname)!=0)&&(flag1 != 5)) {
        din=getptr(buffer,HBMAX,sbuffin,'v','d',fp->fname);
        dout=getptr2(bufferw,HBMAX,sbuffout,'v','d',fp->fname,&fb);
/* if code is 0 then no change */
        if (flag1==0) {
          for (loop1=0;loop1<fp->tsize;loop1++)
            dout[loop1]=din[loop1];
        }
/* if field is to be cut then... */
        else if (flag1 < 5) {
/* cycle thru the dimensions */
          for (inst=0;inst<fb->tsize;inst++) {
            if (subspace1(fp,dname,inst,&start,&incr,&size)==FAIL) {
              fprintf(stderr,"cdfisocut: error with field %s\n",fp->fname);
              exit(1);
            }
            dout[inst]=bad;
            if (fabs(cut[inst])<badlim) {
              pt1 = cut[inst];
              pt2 = ((float)pt1 > cut[inst] ? pt1 - 1 : pt1 + 1);
              dout[inst]=lineprox2(din[start+pt1*incr],din[start+pt2*incr],
                   (float) pt1,(float) pt2,cut[inst]);
            }
          }
        }
      }
      ++vpos;
    }
    dout=getptr2(bufferw,HBMAX,sbuffout,'v','d',dname,&fb);
    for (loop=0;loop<fb->tsize;loop++) {
      if (fabs(cut[loop]) < badlim) {
        pt1 = cut[loop];
        pt2 = ((float)pt1 > cut[loop] ? pt1 - 1 : pt1 + 1);
        dout[loop]=lineprox2(varval[pt1],varval[pt2],(float)pt1,(float)pt2,
                          cut[loop]);
      }
      else dout[loop] = bad;
    }
    putslice(stdout,nelw,sbuffout);
  }
  exit(0);
}

/* linear approximation routine (using Newton's method) */
static float lineprox1(vala,valmid,valb,a,b)
float vala,valmid,valb,a,b;
{
  float linvalue;

  if ((fabs(vala)>badlim)||(fabs(valb)>badlim)||(valmid>badlim))
    return(bad);
  linvalue = (((b-a)*(valmid-vala))/(valb-vala))+a;
  return(linvalue);
}


/* linear approximation routine (using Newton's method)       */
/* for finding place */
static float lineprox2(va,vb,pa,pb,m)
float va,vb,pa,pb,m;
{
  float linvalue;

  if ((fabs(va)>badlim)||(fabs(vb)>badlim)||(m>badlim))
    return(bad);
  linvalue = (((m-pa)*(vb-va))/(pb-pa))+va;
  return(linvalue);
}

/* checks to see if valmid is betwen vala and valb */
/* (ie pt1 < ptmid < pt2 or pt1 > ptmid > pt2 */
mid(pt1,ptmid,pt2)
float pt1,ptmid,pt2;
{
  if ((pt1 > badlim)||(pt2 > badlim)||(ptmid > badlim))
    return(0);
  else if ((pt1 < ptmid)&&(ptmid < pt2))
    return(1);
  else if ((pt1 > ptmid)&&(ptmid > pt2))
    return(1);
  else
    return(0);
}
