/* cdfjday -- computes julian day for given year, month, day fields in */
/* either the variable or static slice, optionally beginning from a given */
/* reference date */

/* All date fields must be in the same slice and have the same */
/* dimensionality.  Data is written to the variable slice */
/* Field specs for jday match those for the day field */

/* Written by James Fox. */

#include <stdio.h>
#include <string.h>
#include "cdfhdr.h"

#define In(ix, iy, iz, iw) ((iw) + nz*((iz) + ny*((iy) + nx*(ix))))

long julian_date(); /* function which computes the julian day -- found in */
                    /* jtime.c */

static void dimcheck(), feed();

static struct field *fldptr; 
static char hbuff[HBMAX][LINE]; /* header buffer */
static char *year_fld, *month_fld, *day_fld, *jday_fld;
static char *ref_year, *ref_month, *ref_day;  /* pointers to command args */
static char slicetype;
static int yeardim, year_dsize1, year_dsize2, year_dsize3, year_dsize4;
static char year_dname1[LINE], year_dname2[LINE], year_dname3[LINE];
static char year_dname4[LINE];  /* dimensional parameters */ 
static float *sbuff, *slicebuff, *vbuff;  /* slicebuffers */
static float *year, *day, *month, *jday; /* pointers to relevant fields */
static long nsbuff, nvbuff, nelem, oelem;  /* element count parameters */

main(argc, argv)

     int argc;
     char *argv[];

{

  char cline[LINE];
  long y_ref, m_ref, d_ref;
  long stop;
  long p;

/* check arguments */

  if(argc != 5 && argc != 8){
    fprintf(stderr, "Usage: cdfjday year_fld month_fld day_fld jday_fld [ref_year ref_month ref_day]\n");
    exit(1);
  }

/* read the arguments into their storage variables */
  
  year_fld = argv[1];
  month_fld = argv[2];
  day_fld = argv[3];
  jday_fld = argv[4];
  
/* read optional arguments, if any */

  if(argc == 8){
    ref_year = argv[5];
    ref_month = argv[6];
    ref_day = argv[7];
  }

/* play with candis header */

  gethdr(stdin, hbuff, HBMAX);
  sprintf(cline, "cdfjday: %s %s %s %s\n", year_fld, month_fld, day_fld,
	  jday_fld);
  addcline(hbuff, HBMAX, cline);
  sprintf(cline, "reference: %s/%s/%s\n", ref_month, ref_day, ref_year);
  addcline(hbuff, HBMAX, cline);
  
/* check which slice the data is in */

  if((fldptr = seekfld(hbuff, HBMAX, 's', year_fld)) != NULL)
    slicetype = 's';
  
  else if((fldptr = seekfld(hbuff, HBMAX, 'v', year_fld)) != NULL)
    slicetype = 'v';
  
  else{
    fprintf(stderr, "cdfjday: could not find %s in s or v slice\n", 
	    year_fld);
    exit(1);
  }

  dimcheck();  /* check dimensions */

/* get an element count before the new field is added */

  oelem = elemcnt(hbuff, HBMAX, slicetype);

/* add jday field with appropriate dimensions */  

  if(yeardim == 4)
    addfld(hbuff, HBMAX, slicetype, jday_fld, fldptr->smul, fldptr->sadd, 
	   fldptr->bytes, yeardim, year_dname1, year_dsize1, year_dname2, 
	   year_dsize2, year_dname3, year_dsize3, year_dname4, year_dsize4);

  else if(yeardim == 3)
    addfld(hbuff, HBMAX, slicetype, jday_fld, fldptr->smul, fldptr->sadd, 
	   fldptr->bytes, yeardim, year_dname1, year_dsize1, year_dname2, 
	   year_dsize2, year_dname3, year_dsize3);

  else if(yeardim == 2)
    addfld(hbuff, HBMAX, slicetype, jday_fld, fldptr->smul, fldptr->sadd,
	   fldptr->bytes, yeardim, year_dname1, year_dsize1, year_dname2, 
	   year_dsize2);

  else if(yeardim == 1)
    addfld(hbuff, HBMAX, slicetype, jday_fld, fldptr->smul, fldptr->sadd, 
	   fldptr->bytes, yeardim, year_dname1, year_dsize1);

  else if(yeardim == 0)
    addfld(hbuff, HBMAX, slicetype, jday_fld, fldptr->smul, fldptr->sadd,
	   fldptr->bytes, yeardim);

  else{
    fprintf(stderr, "cdfjday: error adding %s to candis header\n", jday_fld);
    exit(1);
  }

/* done with header */

  puthdr(stdout, hbuff, HBMAX);

  if(slicetype == 'v'){

/* need to roll over the static slice now */
    
    nsbuff = elemcnt(hbuff, HBMAX, 's');
    sbuff = getbuff(nsbuff);
    getslice(stdin, nsbuff, sbuff);
    putslice(stdout, nsbuff, sbuff);
    stop = MAXSLICE;   /* used in main loop test below */
  }

  else
    stop = 1; /* if we are looking at the static slice it only needs to be */
              /* read once -- see main loop below */      
  
/* convert optional args to right type */
  
  if(argc == 8){
    y_ref = atol(ref_year);
    m_ref = atol(ref_month);
    d_ref = atol(ref_day);
  }

  else
    y_ref = m_ref = d_ref = -1;

/* allocate slice buffer with enough space for jday */

  nelem = elemcnt(hbuff, HBMAX, slicetype);
  slicebuff = getbuff(nelem);

/* main activity loop */

  while(stop != 0 && (p = getslice(stdin, oelem, slicebuff)) != EOF){
    
/* get pointers to the data */

    year = getptr(hbuff, HBMAX, slicebuff, slicetype, 'd', year_fld);
    month = getptr(hbuff, HBMAX, slicebuff, slicetype, 'd', month_fld);
    day = getptr(hbuff, HBMAX, slicebuff, slicetype, 'd', day_fld);
    jday = getptr(hbuff, HBMAX, slicebuff, slicetype, 'd', jday_fld);

/* enter the data */

    feed(y_ref, m_ref, d_ref); 
    putslice(stdout, nelem, slicebuff);
    stop--;
  }
  if(p != EOF && stop == 0){  /* it was static slice, output all variable */
                              /* slices */ 
      nvbuff = elemcnt(hbuff, HBMAX, 'v');
      vbuff = getbuff(nvbuff); 
      while((p = getslice(stdin, nvbuff, vbuff)) != EOF) 
        putslice(stdout, nvbuff, vbuff);
  }
}

static void feed(y, m, d)
long y, m, d;

{

  int i, j, k, l;
  int nx, ny, nz;  nz = 1;


  nx = year_dsize2;
  ny = year_dsize3;
  nz = year_dsize4;

  if(yeardim == 4)
    for(i = 0; i < year_dsize1; i++)
      for(j = 0; j < year_dsize2; j++)
	for(k = 0; k < year_dsize3; k++)
	  for(l = 0; l < year_dsize4; l++)
	    jday[In(i, j, k, l)] = 
	      julian_date((long) year[In(i, j, k, l)], 
			  (long) month[In(i, j, k, l)], 
			  (long) day[In(i, j, k, l)]) - julian_date(y, m, d); 


  if(yeardim == 3)
    for(i = 0; i < year_dsize1; i++)
      for(j = 0; j < year_dsize2; j++)
	for(k = 0; k < year_dsize3; k++)
	  jday[In(i, j, k, 0)] = 
	    julian_date((long) year[In(i, j, k, 0)], 
			(long) month[In(i, j, k, 0)], 
			(long) day[In(i, j, k, 0)]) - julian_date(y, m, d); 
  

  if(yeardim == 2)
    for(i = 0; i < year_dsize1; i++)
      for(j = 0; j < year_dsize2; j++)
	jday[In(i, j, 0, 0)] = 
	  julian_date((long) year[In(i, j, 0, 0)], 
		      (long) month[In(i, j, 0, 0)], 
		      (long) day[In(i, j, 0, 0)]) - julian_date(y, m, d); 

  if(yeardim == 1)
    for(i = 0; i < year_dsize1; i++)
      jday[i] = julian_date((long) year[i], (long) month[i], (long) day[i]) 
	- julian_date(y, m, d); 

  if(yeardim == 0)
    jday[0] = julian_date((long) year[0], (long) month[0], (long) day[0]) 
      - julian_date(y, m, d); 
}

	   
static void dimcheck(){   

/* gets dimensionality, dimension names, dimension sizes and checks for */ 
/*  consistency among all date fields */ 

   yeardim = fldptr->dim; 
  
   if(yeardim == 4){ 
     strcpy(year_dname1, fldptr->dname1); 
     strcpy(year_dname2, fldptr->dname2); 
     strcpy(year_dname3, fldptr->dname3); 
     strcpy(year_dname4, fldptr->dname4); 
     year_dsize1 = fldptr->dsize1; 
     year_dsize2 = fldptr->dsize2; 
     year_dsize3 = fldptr->dsize3; 
     year_dsize4 = fldptr->dsize4; 

     fldptr = seekfld(hbuff, HBMAX, slicetype, month_fld); 
     if(fldptr == NULL){ 
       fprintf(stderr, "cdfjday: could not find %s in %c slice\n",  
 	      month_fld, slicetype); 
       exit(1); 
     } 
    
     if(fldptr->dim != yeardim){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, month_fld); 
       exit(1); 
     } 

     if(strcmp(year_dname1, fldptr->dname1) != 0 ||  
        strcmp(year_dname2, fldptr->dname2) != 0 || 
        strcmp(year_dname3, fldptr->dname3) != 0 || 
        strcmp(year_dname4, fldptr->dname4) != 0 || 
        year_dsize1 != fldptr->dsize1 || year_dsize2 != fldptr->dsize2 ||  
        year_dsize3 != fldptr->dsize3 || year_dsize4 != fldptr->dsize4){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, month_fld); 
       exit(1); 
     } 
    
     fldptr = seekfld(hbuff, HBMAX, slicetype, day_fld); 
     if(fldptr == NULL){ 
       fprintf(stderr, "cdfjday: could not find %s in %c slice\n",  
 	      day_fld, slicetype); 
       exit(1); 
     } 

     if(fldptr->dim != yeardim){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, day_fld); 
       exit(1); 
     } 

     if(strcmp(year_dname1, fldptr->dname1) != 0 ||  
       strcmp(year_dname2, fldptr->dname2) != 0 || 
       strcmp(year_dname3, fldptr->dname3) != 0 || 
       strcmp(year_dname4, fldptr->dname4) != 0 || 
       year_dsize1 != fldptr->dsize1 || year_dsize2 != fldptr->dsize2 ||  
       year_dsize3 != fldptr->dsize3 || year_dsize4 != fldptr->dsize4){ 
      fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	     year_fld, day_fld); 
      exit(1); 
     } 
   } 

   else if(yeardim == 3){ 
     strcpy(year_dname1, fldptr->dname1); 
     strcpy(year_dname2, fldptr->dname2); 
     strcpy(year_dname3, fldptr->dname3); 
     year_dsize1 = fldptr->dsize1; 
     year_dsize2 = fldptr->dsize2; 
     year_dsize3 = fldptr->dsize3; 
     year_dsize4 = 1;

     fldptr = seekfld(hbuff, HBMAX, slicetype, month_fld); 
     if(fldptr == NULL){ 
       fprintf(stderr, "cdfjday: could not find %s in %c slice\n",  
 	      month_fld, slicetype); 
       exit(1); 
     } 
    
     if(fldptr->dim != yeardim){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, month_fld); 
       exit(1); 
     } 

     if(strcmp(year_dname1, fldptr->dname1) != 0 ||  
        strcmp(year_dname2, fldptr->dname2) != 0 || 
        strcmp(year_dname4, fldptr->dname4) != 0 || 
        year_dsize1 != fldptr->dsize1 || year_dsize2 != fldptr->dsize2 ||  
        year_dsize3 != fldptr->dsize3){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, month_fld); 
       exit(1); 
     } 
    
     fldptr = seekfld(hbuff, HBMAX, slicetype, day_fld); 
     if(fldptr == NULL){ 
       fprintf(stderr, "cdfjday: could not find %s in %c slice\n",  
 	      day_fld, slicetype); 
       exit(1); 
     } 

     if(fldptr->dim != yeardim){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, day_fld); 
       exit(1); 
     } 

     if(strcmp(year_dname1, fldptr->dname1) != 0 ||  
       strcmp(year_dname2, fldptr->dname2) != 0 || 
       strcmp(year_dname3, fldptr->dname3) != 0 || 
       year_dsize1 != fldptr->dsize1 || year_dsize2 != fldptr->dsize2 ||  
       year_dsize3 != fldptr->dsize3){ 
      fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	     year_fld, day_fld); 
      exit(1); 
     } 
   } 

   else if(yeardim == 2){ 
     strcpy(year_dname1, fldptr->dname1); 
     strcpy(year_dname2, fldptr->dname2); 
     year_dsize1 = fldptr->dsize1; 
     year_dsize2 = fldptr->dsize2; 
     year_dsize3 = 1;
     year_dsize4 = 1;

     fldptr = seekfld(hbuff, HBMAX, slicetype, month_fld); 
     if(fldptr == NULL){ 
       fprintf(stderr, "cdfjday: could not find %s in %c slice\n",  
 	      month_fld, slicetype); 
       exit(1); 
     } 
    
     if(fldptr->dim != yeardim){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, month_fld); 
       exit(1); 
     } 

     if(strcmp(year_dname1, fldptr->dname1) != 0 ||  
        strcmp(year_dname2, fldptr->dname2) != 0 || 
        year_dsize1 != fldptr->dsize1 || year_dsize2 != fldptr->dsize2){  
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, month_fld); 
       exit(1); 
     } 
    
     fldptr = seekfld(hbuff, HBMAX, slicetype, day_fld); 
     if(fldptr == NULL){ 
       fprintf(stderr, "cdfjday: could not find %s in %c slice\n",  
 	      day_fld, slicetype); 
       exit(1); 
     } 

     if(fldptr->dim != yeardim){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, day_fld); 
       exit(1); 
     } 

     if(strcmp(year_dname1, fldptr->dname1) != 0 ||  
       strcmp(year_dname2, fldptr->dname2) != 0 || 
       year_dsize1 != fldptr->dsize1 || year_dsize2 != fldptr->dsize2){  
      fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	     year_fld, day_fld); 
      exit(1); 
     } 
   } 

   else if(yeardim == 1){ 
     strcpy(year_dname1, fldptr->dname1); 
     year_dsize1 = fldptr->dsize1; 
     year_dsize2 = 1;
     year_dsize3 = 1;
     year_dsize4 = 1;

     fldptr = seekfld(hbuff, HBMAX, slicetype, month_fld); 
     if(fldptr == NULL){ 
       fprintf(stderr, "cdfjday: could not find %s in %c slice\n",  
 	      month_fld, slicetype); 
       exit(1); 
     } 
    
     if(fldptr->dim != yeardim){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, month_fld); 
       exit(1); 
     } 

     if(strcmp(year_dname1, fldptr->dname1) != 0 ||  
        year_dsize1 != fldptr->dsize1){  
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, month_fld); 
       exit(1); 
     } 
    
     fldptr = seekfld(hbuff, HBMAX, slicetype, day_fld); 
     if(fldptr == NULL){ 
       fprintf(stderr, "cdfjday: could not find %s in %c slice\n",  
 	      day_fld, slicetype); 
       exit(1); 
     } 

     if(fldptr->dim != yeardim){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, day_fld); 
       exit(1); 
     } 

     if(strcmp(year_dname1, fldptr->dname1) != 0 ||  
       year_dsize1 != fldptr->dsize1){ 
      fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	     year_fld, day_fld); 
      exit(1); 
     } 
   } 
  
   else if(yeardim == 0){ 
     fldptr = seekfld(hbuff, HBMAX, slicetype, month_fld); 
    
     if(fldptr == NULL){ 
       fprintf(stderr, "cdfjday: could not find %s in %c slice\n",  
 	      month_fld, slicetype); 
       exit(1); 
     } 
    
     if(fldptr->dim != yeardim){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n",  
 	      year_fld, month_fld); 
       exit(1); 
     } 
    
     fldptr = seekfld(hbuff, HBMAX, slicetype, day_fld); 
    
     if(fldptr == NULL){ 
       fprintf(stderr, "cdfjday: could not find %s in %c slice\n",  
 	      day_fld, slicetype); 
       exit(1); 
     } 

     if(fldptr->dim != yeardim){ 
       fprintf(stderr, "cdfjday: inconsistent dim for %s and %s\n", 
 	      year_fld, day_fld); 
       exit(1); 
    } 
  } 
}  
