/*  cdfmerge  --  This routine merges two or more cdf files into one. */
#include <math.h>
#include <stdio.h>
#include "cdfhdr.h"

#define  blast(s1,s2)   addpar(hbout,HBMAX,s1,s2)
#define  cmp(s1,s2)    (atof(s1) > atof(s2))


static struct arrai {
  float *bf;
};
static struct field *fd,*fdnew;
static char badnm[LINE],badlm[LINE],type,value[LINE],hbout[HBMAX][LINE],
     suffix[LINE],vname[LINE],hbin[HBMAX][LINE],string[LINE];
static FILE *fo;
static int lp,loop,lp1,temp;
static long *stanum,*varnum,vtotal,stotal,lp2;
static float *vbuff,*sbuff,*bds;
static struct arrai *sbig,*vbig;

main(argc,argv)
int argc;
char *argv[];


{
  float badval;
  temp = argc/2;

  /**  check to see if the number of arguments are match  **/
  if ((argc == 1) || (argc % 2 == 0))  {
    fprintf(stderr,"Usage: cdfmerge file1 suff1 file2 suff2 ... > fileout\n");
    exit(1);
  }

  /**  initialize the pointers and variables  **/
  stotal = (vtotal = 0);
  vbig = (struct arrai *)malloc(sizeof(struct arrai)*(temp));
  sbig = (struct arrai *)malloc(sizeof(struct arrai)*(temp));
  stanum = (long *)malloc(sizeof(long)*(temp));
  varnum = (long *)malloc(sizeof(long)*(temp));
  if ((vbig==NULL) || (sbig==NULL) || (stanum==NULL) || (varnum==NULL)) {
    fprintf(stderr,"cdfmerge: can't get enough memory\n");
    exit(1);
  }
  bds = getbuff(temp);
  for (loop = 0; loop < temp; loop++) bds[loop] = 0.;

  /**  create new header  **/
  nullhdr(hbout,HBMAX,"float");
  addcline(hbout,HBMAX,"cdfmerge:\n");
  for (lp = 0; lp < temp ; ++lp)   {
    strcpy(string," ");
    addcline(hbout,HBMAX,strcat(strcat(strcat(strcat(string,argv[lp*2+1]),
             " "),argv[lp*2+2]),"\n"));
  }

  for (loop = 0; loop < temp; ++loop)  {
    /** get suffix name **/
    strcpy(suffix,argv[(1+loop)*2]);
    /**  get the file(s)  **/
    if ((fo = fopen(argv[2*loop+1],"r")) == NULL) {
      fprintf(stderr,"cdfmerge: can't open %s\n",argv[2*loop+1]);
      exit(1);
    }
    gethdr(fo,hbin,HBMAX);
    /**  check for float format  **/
    if (getfmt(hbin,HBMAX) != 'f')     {
      fprintf(stderr,"cdfmerge: float format required\n");
      exit(1);
    }
    /**  check for variable "bad" and "badlim"  **/
    for (lp = 0 ; getpar(hbin,HBMAX,lp,vname,value) == OK ; ++lp)
      if (strcmp(vname,"bad") == 0)  {
        if (cmp(value,badnm))
          strcpy(badnm,value);
      }
      else if (strcmp(vname,"badlim") == 0)  {
        if (cmp(value,badlm))
          strcpy(badlm,value);
        bds[loop]= atof(value);
      }
      else if (vname[strlen(vname)-1] == '0') {
        vname[strlen(vname)-1] = '\0';
        blast(strcat(strcat(vname,argv[(loop+1)*2]),"0"),value);
      }
      else {
        blast(strcat(vname,argv[(loop+1)*2]),value);
      }

    /**  get the names from static and variable slices and copy 
         them into new buffer                                       **/
    type = 's';
    for (lp1 = 0 ; lp1 < 2 ; ++lp1)  {
      for (lp = 0;(fd = getfld(hbin,HBMAX,type,lp)) != NULL; ++lp) {
        switch(fd->dim)  {
        case 4 :
          strcat(fd->dname4,suffix);
        case 3 :
          strcat(fd->dname3,suffix);
        case 2 :
          strcat(fd->dname2,suffix);
        case 1 :
          strcat(fd->dname1,suffix);
        default :
          break;
        }
        /** check for name clashes **/
        strcat(fd->fname,suffix);
        while ((fdnew = seekfld(hbout,HBMAX,type,fd->fname)) != NULL) {
          /** only croak if sizes of clashing fields inconsistent **/
          if (fd->tsize != fdnew->tsize) {
            fprintf(stderr,"cdfmerge: inconsistent clashing fields named %s\n",
                 fd->fname);
            exit(1);
          }
          /** otherwise change the name **/
          else {
            if (strlen(fd->fname) >= LINE - 2) {
              fprintf(stderr,"cdfmerge: modified name %s too long\n",
                   fd->fname);
              exit(1);
            }
            strcat(fd->fname,".");
          }
        }
        addfld(hbout,HBMAX,type,fd->fname,fd->smul,
            fd->sadd,fd->bytes,fd->dim,fd->dname1,fd->dsize1,fd->dname2,
            fd->dsize2,fd->dname3,fd->dsize3,fd->dname4,fd->dsize4);
      }
      type = 'v';
    }

    /**  get the static and variable slices  **/
    stanum[loop] = elemcnt(hbin,HBMAX,'s');
    sbig[loop].bf = getbuff(stanum[loop]);
    getslice(fo,stanum[loop],sbig[loop].bf);
    stotal += stanum[loop];
    varnum[loop] = elemcnt(hbin,HBMAX,'v');
    vbig[loop].bf = getbuff(varnum[loop]);
    getslice(fo,varnum[loop],vbig[loop].bf);
    vtotal += varnum[loop];

    /**  close the file  **/
    fclose(fo);


  }

  /**  add "bad" and "badlim" parameters if any found in the input file  **/
  if (strcmp("",badnm) != 0)   {
    blast("bad",badnm);
    blast("badlim",badlm);
  }

  /**  put the header on standard output  **/
  puthdr(stdout,hbout,HBMAX);

  /**  combines the slices and put it on output file  **/
  sbuff = getbuff(stotal);
  vbuff = getbuff(vtotal);
  lp2 = 0;
  loop = 0;
  badval = atof(badnm);
  for (lp1 = 0 ; lp1 < temp ; ++lp1) {
    for (lp = 0; lp < stanum[lp1] ; ++lp)
      sbuff[loop++] = sbig[lp1].bf[lp];
    for (lp = 0; lp < varnum[lp1] ; ++lp)
      vbuff[lp2++]=(vbig[lp1].bf[lp]>bds[lp1] && bds[lp1] != 0.) ?
                   badval:vbig[lp1].bf[lp];
  }
  putslice(stdout,stotal,sbuff);
  putslice(stdout,vtotal,vbuff);
  exit(0);
}
