/* cdfrdim                                                            */
/* usage #1:  cdfrdim dim_name lower_limit upper_limit                */
/*   cdfrdim accepts the name of a dimension and will average that    */
/*   dimension across the given limits (lower and upper) and will     */
/*   reduce the field (i.e. if the field is four dimensional, a three */
/*   dimensional field will be created to hold the average, a 1-dim.  */
/*   field will create a scalar).  Cdfaver can handle up to 4d fields */
/* usage #2:  cdfrdim dim_name value                                  */
/*   this form of cdfrdim will extract a "plane" of data a cdf file   */
/*   by cutting the dim_name out at the given value                   */
/* cdfrdim uses stdin and stdout for all file handling.               */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

static char hbuffer[HBMAX][LINE],hbufferw[HBMAX][LINE];
                                                 /* buffers for header info */
static char cbuff[HBMAX][LINE];                  /* buffer for comments */
static char dimname[2][HBMAX][5][LINE];          /* table w/field dim_names */
static int table[2][HBMAX][6];                   /* table w/field dims */
static float *sbuff,*sbuffout;                   /* buffers for slice data */

main(argc,argv)
int argc;
char *argv[];
{
    char pname[LINE],value[LINE];   /* used in transfering param. data */
    char dname[LINE];               /* dim_name from command line argument */
    char cl[LINE];                  /* comment line */
    char field;                     /* which field is being reduced (header) */
    long nel,nelw;                  /* number of elements to read or write */
    long numread;                   /* number of lines read / status of read */
    long loop,loop1,loop2;          /* used for looping thru data */
    long fields[2];                 /* # of fields in static and var slices */
    long dim,ds1,ds2,ds3,ds4;       /* dim_size of each dimension and field */
    long x,y,z;                     /* used in cycling thru data */
    long flag;                      /* which dimension is being avg or cut */
    long numel;                     /* total number of elements in field */
    long stat;                      /* status of getslice */
    long bpos,bposout;              /* position within a buffer */
    long upper,lower;               /* limits on the reduction variable */
    long number;                    /* number of elements to average */
    float high,low;                 /* reduction index range */
    float dif;                      /* used in computing reduction var. */
    float total;                    /* sum of elements to average */
    float avg;                      /* avg = (total/number) */
    float bad,badlim;               /* bad data limits */
    struct field *fp;               /* pointer to field data */

/* check command line arguments */
    if ((argc < 3) || (argc > 4)) {
        fprintf(stderr,"Usage: cdfrdim dim_name low_limit high_limit\n");
        fprintf(stderr,"  or   cdfrdim dim_name_to_reduce value\n");
        exit(1);
    }
/* transfer stuff from command line */
    strcpy(dname,argv[1]);
    low = atof(argv[2]);
    if (argc == 3) high = low;
    else high = atof(argv[3]);
/* check order of low and high (i.e. make sure high is more than low) */
    if (high < low) {
        fprintf(stderr,"cdfrdim: high_limit is less than low limit\n");
        fprintf(stderr,"Usage: cdfrdim dim_name low_limit high_limit\n");
        fprintf(stderr,"  or   cdfrdim dim_name_to_reduce value\n");
        exit(1);
    }
/* get header from stdin */
    gethdr(stdin,hbuffer,HBMAX);
/* check format, if not float die! */
    if (getfmt(hbuffer,HBMAX) != 'f') {
        fprintf(stderr,"cdfrdim: wrong format type\n");
        exit(1);
    }
/* check validity of dim name */
    if ((fp=seekfld(hbuffer,HBMAX,'s',dname)) == NULL) {
        fprintf(stderr,"cdfrdim: %s is not a valid dimension name\n",dname);
        exit(1);
    }
    if ((fp->dim != 1) || (strcmp(fp->dname1,dname) !=0)) {
        fprintf(stderr,"cdfrdim: %s is not an index field\n",dname);
        exit(1);
    }
/* load in static slice */
    nel = elemcnt(hbuffer,HBMAX,'s');
    sbuff=(float*)malloc(sizeof(float)*nel);
    if (sbuff==NULL) {
        fprintf(stderr,"cdfrdim: not enough memory to hold slice data\n");
        exit(1);
    }
    getslice(stdin,nel,sbuff);
/* compute reduction variable for low */
    dif = 1.e30;
    lower = 0;
    for (loop=0;loop< fp->dsize1;loop++) {
        if (fabs(low - sbuff[fp->ptr + loop]) < dif) {
            dif = fabs(low - sbuff[fp->ptr + loop]);
            lower = loop;
        }
    }
/* for high */
    dif = 1.e30;
    upper = 0;
    for (loop=0;loop< fp->dsize1;loop++) {
        if (fabs(high -sbuff[fp->ptr + loop]) <dif) {
            dif = fabs(high - sbuff[fp->ptr + loop]);
            upper = loop;
        }
    }
/* put dimension data into tables (for static) */
    loop=0;
    while ((fp = getfld(hbuffer,HBMAX,'s',loop)) != NULL) {
        table[0][loop][0]=fp->dim;
        table[0][loop][5]=fp->tsize;
        if (fp->dim >= 1) {
            table[0][loop][1]=fp->dsize1;
            strcpy(dimname[0][loop][1],fp->dname1);
        }
        if (fp->dim >= 2) {
            table[0][loop][2]=fp->dsize2;
            strcpy(dimname[0][loop][2],fp->dname2);
        }
        if (fp->dim >= 3) {
            table[0][loop][3]=fp->dsize3;
            strcpy(dimname[0][loop][3],fp->dname3);
        }
        if (fp->dim >= 4) {
            table[0][loop][4]=fp->dsize4;
            strcpy(dimname[0][loop][4],fp->dname4);
        }
        ++loop;
    }
    fields[0]=loop;
/* same for variable */
    loop=0;
    while((fp = getfld(hbuffer,HBMAX,'v',loop)) != NULL) {
        table[1][loop][0]=fp->dim;
        table[1][loop][5]=fp->tsize;
        if (fp->dim >= 1) {
            table[1][loop][1]=fp->dsize1;
            strcpy(dimname[1][loop][1],fp->dname1);
        }
        if (fp->dim >= 2) {
            table[1][loop][2]=fp->dsize2;
            strcpy(dimname[1][loop][2],fp->dname2);
        }
        if (fp->dim >= 3) {
            table[1][loop][3]=fp->dsize3;
            strcpy(dimname[1][loop][3],fp->dname3);
        }
        if (fp->dim >= 4) {
            table[1][loop][4]=fp->dsize4;
            strcpy(dimname[1][loop][4],fp->dname4);
        }
        ++loop;
    }
    fields[1]=loop;
/* make output header */
    nullhdr(hbufferw,HBMAX,"float");
/* transfer comments to output buffer */
    numread = getcmt(hbuffer,HBMAX,cbuff);
    for (loop=0;loop<numread;++loop) {
        addcline(hbufferw,HBMAX,cbuff[loop]);
    }
/* transfer parameters and search for bad and badlim */
    bad = BAD;
    badlim = BADLIM;
    loop = 0;
    while ((numread = getpar(hbuffer,HBMAX,loop++,pname,value)) != FAIL) {
        addpar (hbufferw,HBMAX,pname,value);
        if (strcmp("bad",pname)==0) bad = atof(value);
        if (strcmp("badlim",pname)==0) badlim = atof(value);
    }
/* transfer static and variable fields making changes as needed */
    field = 's';
    for (loop2=0;loop2<2;loop2++) {
        if (loop2==1) field = 'v';
        loop=0;
/* loop thru each field */
        while ((fp=getfld(hbuffer,HBMAX,field,loop++)) != NULL) {
            flag=0;
/* which dimension to avg or reduce? */
            if ((fp->dim >= 1)&&(strcmp(dname,fp->dname1)==0)) flag=1;
            if ((fp->dim >= 2)&&(strcmp(dname,fp->dname2)==0)) flag=2;
            if ((fp->dim >= 3)&&(strcmp(dname,fp->dname3)==0)) flag=3;
            if ((fp->dim >= 4)&&(strcmp(dname,fp->dname4)==0)) flag=4;
/* if one then ... */
            if (flag==1) {
                strcpy(fp->dname1,fp->dname2);
                strcpy(fp->dname2,fp->dname3);
                strcpy(fp->dname3,fp->dname4);
                fp->dsize1=fp->dsize2;
                fp->dsize2=fp->dsize3;
                fp->dsize3=fp->dsize4;
            }
/* if two ... */
            if (flag==2) {
                strcpy(fp->dname2,fp->dname3);
                strcpy(fp->dname3,fp->dname4);
                fp->dsize2=fp->dsize3;
                fp->dsize3=fp->dsize4;
            }
/* if three ... */
            if (flag==3) {
                strcpy(fp->dname3,fp->dname4);
                fp->dsize3=fp->dsize4;
            }
/* if no match then send w/no change */
            if (flag==0) {
                addfld(hbufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,
                  fp->bytes,fp->dim,fp->dname1,fp->dsize1,fp->dname2,
                  fp->dsize2,fp->dname3,fp->dsize3,fp->dname4,fp->dsize4);
            }
/* else send with changes */
            else {
                if (fp->dim==1) addfld(hbufferw,HBMAX,field,fp->fname,fp->smul,
                  fp->sadd,fp->bytes,0);
                if (fp->dim==2) addfld(hbufferw,HBMAX,field,fp->fname,fp->smul,
                  fp->sadd,fp->bytes,1,fp->dname1,fp->dsize1);
                if (fp->dim==3) addfld(hbufferw,HBMAX,field,fp->fname,fp->smul,
                  fp->sadd,fp->bytes,2,fp->dname1,fp->dsize1,fp->dname2,
                  fp->dsize2);
                if (fp->dim==4) addfld(hbufferw,HBMAX,field,fp->fname,fp->smul,
                  fp->sadd,fp->bytes,3,fp->dname1,fp->dsize1,fp->dname2,
                  fp->dsize2,fp->dname3,fp->dsize3);
            }
        }
    }
/* add comment line */
    if (argc==3) sprintf(cl,"cdfrdim: %s %s\n",argv[1],argv[2]);
    else sprintf(cl,"cdfrdim: %s %s %s\n",argv[1],argv[2],argv[3]);
    addcline(hbufferw,HBMAX,cl);
/* send updated header to stdout */
    puthdr(stdout,hbufferw,HBMAX);
/* set up the loop that will tell if variable or static slice data is in */
/* memory.  loop1 < 0 is static, loop1 >= 0 is variable. */
    loop1= -1;
    field = 's';
/* set aside memory for slice data */
    nelw = elemcnt(hbufferw,HBMAX,field);
    sbuffout=(float*)malloc(sizeof(float)*nelw);
    if (sbuffout == NULL) {
        fprintf(stderr,"cdfrdim: not enough memory to hold slice data\n");
        exit(1);
    }
    stat = 1;
/* check the status of getslice */
    while(stat != EOF) {
        ++loop1;
/* loop thru fields */
        bpos=bposout=0;
        if (loop1 > 1) loop1 = 1;
        for (loop=0;loop<fields[loop1];loop++) {
/* determine limit to loop thru */
            ds1=ds2=ds3=ds4=1;
            dim = table[loop1][loop][0];
            numel = table[loop1][loop][5];
            if (dim >= 1) ds1 = table[loop1][loop][1];
            if (dim >= 2) ds2 = table[loop1][loop][2];
            if (dim >= 3) ds3 = table[loop1][loop][3];
            if (dim >= 4) ds4 = table[loop1][loop][4];
/* redetermine which dimension to average or reduce */
            flag=0;
            if ((dim>=1)&&(strcmp(dname,dimname[loop1][loop][1])==0)) flag=1;
            if ((dim>=2)&&(strcmp(dname,dimname[loop1][loop][2])==0)) flag=2;
            if ((dim>=3)&&(strcmp(dname,dimname[loop1][loop][3])==0)) flag=3;
            if ((dim>=4)&&(strcmp(dname,dimname[loop1][loop][4])==0)) flag=4;
/* if none to average or reduce then send to buffer */
            if ((flag==0)||(dim==0)) {
                for (loop2=0;loop2<numel;loop2++) {
                    sbuffout[bposout++]=sbuff[bpos++];
                }
            }
/* if flag = 4 find avg of 4th dim */
            if (flag==4) {
/* cycle thru data */
              for (y=0;y<numel;y += ds4) {
                number=0;
                total=0.;
                for (x=lower;x<=upper;++x) {
/* find good data */
                  if (fabs(sbuff[bpos+x+y]) < badlim) {
                    number++;
                    total += sbuff[bpos+x+y];
                  }
                }
/* find average */
                if (number == 0) avg = bad;
                else avg = (total/number);
/* put in buffer */
                sbuffout[bposout++] = avg;
              }
              bpos += numel;
            }
/* if flag = 3 find avg of 3rd dim */
            else if (flag==3) {
              for (z=0;z<numel;z+=(ds3*ds4)) {
                for (y=0;y<ds4;y++) {
                  number=0;
                  total=0.;
                  for (x=lower*ds4;x<=upper*ds4;x+=ds4) {
fprintf(stderr,"bpos,x,y,z,sbuff,fabs(sbuff): %d %d %d %d %g %g\n",bpos,x,y,z,sbuff[bpos+x+y+z],fabs(sbuff[bpos+x+y+z]));
                    if (fabs(sbuff[bpos+x+y+z]) < badlim) {
                      number++;
                      total += sbuff[bpos+x+y+z];
                    }
                  }
                  if (number == 0) avg = bad;
                  else avg = (total/number);
                  sbuffout[bposout++] = avg;
                }
              }
              bpos += numel;
            }
/* if flag = 2 find avg of 2nd dim */
            else if (flag==2) {
              for(z=0;z<numel;z+=(ds2*ds3*ds4)) {
                for(y=0;y<(ds3*ds4);y++) {
                  number=0;
                  total=0.;
                  for(x=lower*ds3*ds4;x<=upper*ds3*ds4;x+=(ds3*ds4)) {
                    if (fabs(sbuff[bpos+x+y+z]) < badlim) {
                      number++;
                      total += sbuff[bpos+x+y+z];
                    }
                  }
                  if (number == 0) avg = bad;
                  else avg = (total/number);
                  sbuffout[bposout++] = avg;
                }
              }
              bpos += numel;
            }
/* if flag = 1 find avg of 1st dim */
            else if (flag==1) {
              for (y=0;y<(ds2*ds3*ds4);y++) {
                number=0;
                total=0.;
                for (x=lower*ds2*ds3*ds4;x<=upper*ds2*ds3*ds4;x+=ds2*ds3*ds4) {
                  if (fabs(sbuff[bpos+x+y]) < badlim) {
                    number++;
                    total += sbuff[bpos+x+y];
                  }
                }
                if (number == 0) avg = bad;
                else avg = (total/number);
                sbuffout[bposout++] = avg;
              }
              bpos += numel;
            }
        }
        putslice(stdout,nelw,sbuffout);
/* if loop1 is equal to 0 then change buffers to be variable not static data */
        if (loop1 == 0) {
            field = 'v';
/* free static buffers */
            free(sbuff);
            free(sbuffout);
            nel = elemcnt(hbuffer,HBMAX,field);
            nelw = elemcnt(hbufferw,HBMAX,field);
/* set up memory for variable data */
            sbuff = (float*)malloc(sizeof(float)*nel);
            sbuffout = (float*)malloc(sizeof(float)*nelw);
            if ((sbuff == NULL)||(sbuffout == NULL)) {
                fprintf(stderr,"cdfrdim: not enough memory to hold data\n");
                exit(1);
            }
        }
        stat = getslice(stdin,nel,sbuff);
    }
    exit(0);
}
