/* cdfrlab -- convert a Candis file to rlab format */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

static char hbuff[HBMAX][LINE],pname[LINE],pvalue[LINE];
static float *sbuff,*vbuff;
static long nsbuff,nvbuff;
static struct field *fp;

main(argc,argv)
int argc;
char *argv[];
{
  long loop;
  float value;

/* check command line */
  if (argc != 1) {
    fprintf(stderr,"Usage: cdfrlab < candisfile > rlabfile\n");
    exit(1);
  }

/* get header */
  gethdr(stdin,hbuff,HBMAX);
  if (getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"cdfrlab: float format expected\n");
    exit(1);
  }

/* compute input buffer lengths and make buffers */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  nvbuff = elemcnt(hbuff,HBMAX,'v');
  sbuff = getbuff(nsbuff);
  vbuff = getbuff(nvbuff);

/* get static and first variable slice (other variable slices ignored) */
  getslice(stdin,nsbuff,sbuff);
  getslice(stdin,nvbuff,vbuff);

/* loop through parameters */
  loop = 0;
  while (getpar(hbuff,HBMAX,loop++,pname,pvalue) == OK) {
    value = atof(pvalue);
    rlabwrite(pname,0,1,1,&value);
  }

/* loop through static slice */
  loop = 0;
  while ((fp = getfld(hbuff,HBMAX,'s',loop++)) != NULL) {
    rlabwrite(fp->fname,fp->dim,fp->dsize1,fp->dsize2,sbuff + fp->ptr);
  }

/* loop through first variable slice */
  loop = 0;
  while ((fp = getfld(hbuff,HBMAX,'v',loop++)) != NULL) {
    rlabwrite(fp->fname,fp->dim,fp->dsize1,fp->dsize2,vbuff + fp->ptr);
  }

/* finished */
  exit(0);
}

/* rlabwrite -- write the data to rlab format */
rlabwrite(name,ndims,nrow,ncol,data)
char *name;
int ndims,nrow,ncol;
float *data;
{
  int scol,ecol,col,row;

/* skip entirely if number of dimensions exceeds 2 */
  if (ndims > 2) return;

/* fix number of columns if number of dimensions less than 2 */
  if (ndims < 2) ncol = 1;

/* fix number of rows if number of dimensions less than 1 */
  if (ndims < 1) nrow = 1;

/* write the header line */
  printf("# matrix : %s no_of_rows: %d no_of_cols: %d REAL\n",name,nrow,ncol);

/* loop over columns 5 at a time */
  scol = 0;
  while (scol < ncol) {

/* find ending column */
    ecol = scol + 5;
    if (ecol > ncol) ecol = ncol;

/* write subheader */
    printf("# matrix columns %d thru %d\n",scol + 1, ecol);

/* loop over rows */
    for (row = 0; row < nrow; row++) {

/* subloop over columns */
      for (col = scol; col < ecol; col++) {

/* write the element */
	printf(" %g",data[col + ncol*row]);
      }

/* make a newline to get into next row */
      printf("\n");
    }

/* increment column group */
    scol += 5;
  }
}
