/* cdfuncat: cdf program to split some named index field so that each     *
 * instance corresponds to one slice in the un catted file. Several       *
 * restrictions apply to the file and field to be split: first the file   *
 * must have only one slice, the field must be present, and preferably    *
 * the field will occour in all variable fields ( if not variable fields  * 
 * not depending on that field will be moved into the static slice ).     *
 */

#include <stdio.h>
#include "cdfhdr.h"

#define PRNAME argv[0]
#define MAXFLDS (HBMAX - 20)
#define MAXNM 30

static void min_dimsz1();

main(argc,argv)
int argc;
char **argv;
{
  int i,j,k;
  void min_dimsz1();
  char indxname[30];
  char hbuff[HBMAX][LINE],newhbuff[HBMAX][LINE];
  char cline[LINE];
  float *dataptr;
  long stat_size,var_size;
  float *stat_buf,*var_buf;
  long stat_loop,var_loop,totwritten;
  float *stat_ptrs[MAXFLDS],*var_ptrs[MAXFLDS]; 
  long stat_sizes[MAXFLDS],var_sizes[MAXFLDS];
  struct field *fb;
  long var_loop1_size[MAXFLDS],var_loop2_size[MAXFLDS];
  long new_stat_count, new_var_count ; 
  int num_slices;
  float *var_ptr;
  char pname[MAXNM],pval[MAXNM],dfield[MAXNM],field0[MAXNM];

  /* check for the correct useage */
  if(argc != 2 || isatty(stdin)) {
    fprintf(stderr,"Usage: %s indexfield\n",PRNAME);
    exit(1);
  }
  /* copy the argument over */
  strcpy(indxname,argv[1]);

  /* load in the old header */
  gethdr(stdin,hbuff,HBMAX);
  
  /* find out the amount of space that will be needed and allocate buffers */
  stat_size = elemcnt(hbuff,HBMAX,'s');
  var_size  = elemcnt(hbuff,HBMAX,'v');

  stat_buf = getbuff(stat_size);
  var_buf  = getbuff(var_size );

  /* load in the slices -- swap bytes if needed since write doesn't use
   * putslice
   */
  if(getslice(stdin,stat_size,stat_buf) == EOF) {
    fprintf(stderr,"%s: unable to load static slice\n",PRNAME);
    exit(1);
  }
  if (FLOATSWAP) floatswap(stat_size,stat_buf);
  
  if(getslice(stdin,var_size,var_buf) == EOF) {
    fprintf(stderr,"%s: unable to load variable slice\n",PRNAME);
    exit(1);
  }
  if (FLOATSWAP) floatswap(var_size,var_buf);

  /* now check to see if that was the last variable slice : if not then die *
   * we will exit if this is the case so it will not matter if we trash     *
   * var_buf                                                                */ 

  if(getslice(stdin,var_size,var_buf) != EOF) {
    fprintf(stderr,"%s: file contains more than one variable slice \n",PRNAME);
    exit(1);
  }
 
  /* just check to see that the field exists before we waste any more time */
  if(seekfld(hbuff,HBMAX,'s',indxname) == NULL) {
    fprintf(stderr,"%s: specified index field",PRNAME);
    fprintf(stderr," %s does not exist\n",indxname ); 
    exit(1);
  }

  /* well, if it existed start making up new header */
  nullhdr(newhbuff,HBMAX,"float");
  copycmt(newhbuff,HBMAX,hbuff,HBMAX);

  /* copy all perameters unrelated to the field to be uncatted */
  i = 0;
  strcpy(dfield,"d"); 
  strcat(dfield,indxname);
  strcpy(field0,indxname);
  strcat(field0,"0");
  while(getpar(hbuff,HBMAX,i++,pname,pval) != FAIL)
    if(strcmp(dfield,pname) && strcmp(field0,pname))
      addpar(newhbuff,HBMAX,pname,pval); 

  /* say that we were here */
  sprintf(cline,"%s %s\n",PRNAME,indxname);
  addcline(newhbuff,HBMAX,cline);
 
  /* now start working on static slice part of header */
  /* copy over all fields except the target field     */
  stat_loop = i = 0;
  while((fb = getfld(hbuff,HBMAX,'s',i)) != NULL){
    if(strcmp(fb->fname,indxname) != 0){
      addfld(newhbuff,HBMAX,'s',fb->fname,fb->smul,fb->sadd,fb->bytes,
             fb->dim,fb->dname1,fb->dsize1,fb->dname2,fb->dsize2,
             fb->dname3,fb->dsize3,fb->dname4,fb->dsize4);
      min_dimsz1(fb);
      stat_ptrs[stat_loop]  = fb->ptr + stat_buf;
      stat_sizes[stat_loop] = fb->tsize;
      stat_loop++;
    }
    /* increment count */
    i++;
  }

  /* recover info about target field */
  fb = seekfld(hbuff,HBMAX,'s',indxname); 
  /* get data pointer to target field */
  dataptr = fb->ptr + stat_buf;
  num_slices = fb->dsize1;

  /* just in case, check that this field has only 1 dimension */
  if ( fb->dim != 1 ) {
    fprintf(stderr,"%s : field %s has",PRNAME,indxname);
    fprintf(stderr," more than one dimension \n");
    exit(1);
  }

  /* add the target field as the first variable field */
  addfld(newhbuff,HBMAX,'v',indxname,fb->smul,fb->sadd,fb->bytes,0);

  /* now pass the variable fields through, or if they don't depend on *
   * then target field have them moved to the static slice            */
  var_loop = j = 0;
  while(( fb = getfld(hbuff,HBMAX,'v',j)) != NULL){
    /* find out which dimension (if any ) is the target field */
    /* then add the field to where it belongs */

    if(strcmp(fb->dname1,indxname)== 0 && fb->dim >= 1) { 
      addfld(newhbuff,HBMAX,'v',fb->fname,fb->smul,fb->sadd,fb->bytes,
             fb->dim-1,fb->dname2,fb->dsize2,fb->dname3,fb->dsize3,
             fb->dname4,fb->dsize4); 
      /* set minimum dimension size to 1 */
      min_dimsz1(fb);
      var_ptrs[var_loop] = fb->ptr+var_buf; 
      var_sizes[var_loop] = fb->tsize/num_slices;
      var_loop1_size[var_loop] = fb->dsize2*fb->dsize3*fb->dsize4;
      var_loop2_size[var_loop] = fb->tsize; 
      var_loop++;
    }

    else if(strcmp(fb->dname2,indxname)==0 &&fb->dim >= 2)  {
      addfld(newhbuff,HBMAX,'v',fb->fname,fb->smul,fb->sadd,fb->bytes,
             fb->dim-1,fb->dname1,fb->dsize1,fb->dname3,fb->dsize3,
             fb->dname4,fb->dsize4); 
      min_dimsz1(fb);
      var_ptrs[var_loop] = fb->ptr+var_buf; 
      var_sizes[var_loop] = fb->tsize/num_slices;
      var_loop1_size[var_loop] = fb->dsize3*fb->dsize4;
      var_loop2_size[var_loop] = fb->dsize2*fb->dsize3*fb->dsize4; 

      var_loop++;
    }

    else if(strcmp(fb->dname3,indxname)==0&& fb->dim >= 3 )  {
      addfld(newhbuff,HBMAX,'v',fb->fname,fb->smul,fb->sadd,fb->bytes,
             fb->dim-1,fb->dname1,fb->dsize1,fb->dname2,fb->dsize2,
             fb->dname4,fb->dsize4); 
      min_dimsz1(fb);
      var_ptrs[var_loop] = fb->ptr+var_buf; 
      var_sizes[var_loop] = fb->tsize/num_slices;
      var_loop1_size[var_loop] = fb->dsize4;
      var_loop2_size[var_loop] = fb->dsize3*fb->dsize4; 
      var_loop++;
    }

    else if(strcmp(fb->dname4,indxname)==0 && fb->dim >=4){
      addfld(newhbuff,HBMAX,'v',fb->fname,fb->smul,fb->sadd,fb->bytes,
             fb->dim-1,fb->dname1,fb->dsize1,fb->dname2,fb->dsize2,
             fb->dname3,fb->dsize3); 
      min_dimsz1(fb);
      var_ptrs[var_loop] = fb->ptr+var_buf; 
      var_sizes[var_loop] = fb->tsize/num_slices;
      var_loop1_size[var_loop] = 1;
      var_loop2_size[var_loop] = fb->dsize4; 
      var_loop++;
    }
    /* otherwise field must not depend on the named dimensions     *
     * move it to the static slice                                 */
    else {
      addfld(newhbuff,HBMAX,'s',fb->fname,fb->smul,fb->sadd,fb->bytes,
             fb->dim,fb->dname1,fb->dsize1,fb->dname2,fb->dsize2,
             fb->dname3,fb->dsize3,fb->dname4,fb->dsize4); 
       min_dimsz1(fb);
       stat_ptrs[stat_loop] = fb->ptr + var_buf;
       stat_sizes[stat_loop] = fb->tsize;
       if(stat_sizes[stat_loop] == 0 ) stat_sizes[stat_loop] = 1;
       stat_loop++;
    }
 
     /* increment the field position counter */
     j++ ;
  }

  /* that should do it for the header  */
  puthdr(stdout,newhbuff,HBMAX);
  
  /* get new element counts */
  new_stat_count = elemcnt(newhbuff,HBMAX,'s');
  new_var_count  = elemcnt(newhbuff,HBMAX,'v');

  /* write the new static element count to the file */
  fprintf(stdout,"%8ld",new_stat_count);

  /* loop over the static fields writing out data */
  for(k = 0; k< stat_loop; k++) {
    if(fwrite(stat_ptrs[k],sizeof(float),stat_sizes[k],stdout) !=
       stat_sizes[k]){
      fprintf(stderr,"%s: failed fwrite \n",PRNAME);
      exit(1);
    }
  }

  /* loop over the slices */
  for(k = 0; k< num_slices ; k++){
    
    /* write out the element count for each slice */
    fprintf(stdout,"%8ld",new_var_count);
    /* reset counter of numbers written */
    totwritten = 0;

    /* write out the value of the target field */
    totwritten += fwrite(dataptr+k,sizeof(float),1,stdout);

    /* loop over the data arrays */
    for( i = 0; i < var_loop; i++){
      
      /* write out the data for each array */
      for(j = 0; j< var_sizes[i]/var_loop1_size[i] ; j++){
        var_ptr = var_ptrs[i]+var_loop2_size[i]*j + var_loop1_size[i]*k;
        totwritten += fwrite(var_ptr,sizeof(float),var_loop1_size[i],stdout) ;
      }

    } /* end of loop over arrays */
      /* check that we wrote the # of elements that we intended to write */
      if(totwritten != new_var_count){
        fprintf(stderr,"%s: failed fwrite \n",PRNAME);
        exit(1);
      }
  }   /* end of loop over slices */

  /* close output and exit */
  fclose(stdout);
  exit(0);
}

/* quick function to alter zero dimension sizes to one so that the  *
 * total size can be calculated as the product of dimension sizes   */
static void min_dimsz1(fb)
struct field *fb;
{
  if(fb->dim < 1 ) fb->dsize1 =1;
  if(fb->dim < 2 ) fb->dsize2 =1;
  if(fb->dim < 3 ) fb->dsize3 =1;
  if(fb->dim < 4 ) fb->dsize4 =1;

  return ;
}
