/* cdfvlim -- removes unreasonable data values from any common data format */
/*	       file and replaces them with the "bad" data. */

#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

/* max number of fields in the slice to be limited */
#define SM 20

static char *hbuff[HBMAX][LINE];		/* header buffer */
static float *sbuff,*vbuff;			/* slice buffers */
static char cline[LINE];			/* comment line buffer */
static long nsbuff,nvbuff;			/* slice buffer lengths */
static float bad,badupper[SM],badlower[SM];     /* bad data values */
static float *data[SM];			        /* data field */
static long i,c;				/* loop index */
static long datal[SM];				/* data field length */
static struct field *fp,*datafp;		/* field structure pointer */

main(argc,argv)
int argc;
char *argv[];
{

/* check arguments */
  if (argc%3 != 1 || argc == 1) {
    fprintf(stderr,"Usage: cdfvlim field1_name lower_limit upper_limit ...\n");
    exit(1);
  }

/* limit number of arguments */
  if (argc/3 >= SM) {
    fprintf(stderr,"cdfvlim: max number of limited fields = %d\n",SM);
    exit(1);
  }

/* get header from input file */
  gethdr(stdin,hbuff,HBMAX);
  if(getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"cdfvlim: float format required\n");
    exit(1);
  }

/* assign bad data values */
  if(seekpar(hbuff,HBMAX,"bad",cline)==OK) bad=atof(cline);
    else bad=BAD;

/* allocate buffers */
  nsbuff=elemcnt(hbuff,HBMAX,'s');
  nvbuff=elemcnt(hbuff,HBMAX,'v');
  sbuff=getbuff(nsbuff);
  vbuff=getbuff(nvbuff);

/* write output header with added comment */
  sprintf(cline,"%s:\n",argv[0]);
  addcline(hbuff,HBMAX,cline);
  for(c=1;c<argc;c=c+3) {
    sprintf(cline,"  %s %s %s\n",argv[c],argv[c+1],argv[c+2]);
    addcline(hbuff,HBMAX,cline);
  }
  puthdr(stdout,hbuff,HBMAX);

/* transfer static slice */
  getslice(stdin,nsbuff,sbuff);
  putslice(stdout,nsbuff,sbuff);

/* check for existence of data field */
  for(c=1;c<argc;c=c+3) {
    if((fp=seekfld(hbuff,HBMAX,'v',argv[c]))==NULL) {
      fprintf(stderr,"cdfvlim: data field %s not found\n",argv[c]);
      exit(1);
    }
    datal[c] = fp->tsize;

/* assign pointer to data field */
    data[c] = getptr2(hbuff,HBMAX,vbuff,'v','d',argv[c],&datafp);
    badupper[c] = atof(argv[c+2]);
    badlower[c] = atof(argv[c+1]);
  }

/* loop for each field being checked */
  while(getslice(stdin,nvbuff,vbuff) != EOF) {
    for(c=1;c<argc;c=c+3) {

/* loop over all data pointers */
      for(i=0;i<datal[c];i++) {
	if((data[c][i] > badupper[c]) || (data[c][i] < badlower[c])) {
	  data[c][i] = bad;
	}
      }
    }
    putslice(stdout,nvbuff,vbuff);
  }
  exit(0);
}
