/* hrdrawair.c -- Translate Dave Jorgensen's ascii file of P-3 slow
 * tape to Candis format.
 */

#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"
#include "rawget.h"

#define DAY 86400.
#define DTOR 57.2958

/* Candis stuff */
static char hbuf[HBMAX][LINE];
static char cline[LINE];
static float *sbuff,*vbuff;
static long nsbuff,nvbuff;
static float *vars[NVARS];
static float *lon,*lat,*wx,*wy,*time,*gsx,*gsy,*da,*ra;

/* input stuff */
static char *names[] = {"ac","mo","dy","yr","hr","mn","sc",
  "nav","la1","lo1","la2","lo2","la4","lo4","gx1","gy1",
  "gx2","gy2","gx4","gy4","hd","ra1","ra2","ra3","pc","rl","ps","ta","td",
  "gs","tk","ws","wd","rd","rs","ru","wz","wgs","lw","ts"};
static struct p3slowblock *dat;
struct p3slowblock *rawget();

main()
{
  int i;
  float dayfactor,otime;

/* make a header */
  nullhdr(hbuf,HBMAX,"float");

/* comments */
  addcline(hbuf,HBMAX,"hrdrawair:\n");

/* static fields */
  for (i = 0; i < 7; i++) {
    addfld(hbuf,HBMAX,'s',names[i],1.,0.,'s',0);
  }

/* variable fields from input */
  for (i = 7; i < NVARS; i++) {
    addfld(hbuf,HBMAX,'v',names[i],10000.,0.,'l',0);
  }

/* derived variable fields */
  addfld(hbuf,HBMAX,'v',"wx",100.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"wy",100.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"tptime",1.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"lon",10000.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"lat",10000.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"da",100.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"gsx",10.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"gsy",10.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"ra",1.,0.,'l',0);

/* allocate space */
  nsbuff = elemcnt(hbuf,HBMAX,'s');
  sbuff = getbuff(nsbuff);
  nvbuff = elemcnt(hbuf,HBMAX,'v');
  vbuff = getbuff(nvbuff);

/* assign pointers */
  for (i = 0; i < 7; i++) {
    vars[i] = getptr(hbuf,HBMAX,sbuff,'s','d',names[i]);
  }
  for (i = 7; i < NVARS; i++) {
    vars[i] = getptr(hbuf,HBMAX,vbuff,'v','d',names[i]);
  }
  wx = getptr(hbuf,HBMAX,vbuff,'v','d',"wx");
  wy = getptr(hbuf,HBMAX,vbuff,'v','d',"wy");
  time = getptr(hbuf,HBMAX,vbuff,'v','d',"tptime");
  lon = getptr(hbuf,HBMAX,vbuff,'v','d',"lon");
  lat = getptr(hbuf,HBMAX,vbuff,'v','d',"lat");
  da = getptr(hbuf,HBMAX,vbuff,'v','d',"da");
  gsx = getptr(hbuf,HBMAX,vbuff,'v','d',"gsx");
  gsy = getptr(hbuf,HBMAX,vbuff,'v','d',"gsy");
  ra = getptr(hbuf,HBMAX,vbuff,'v','d',"ra");

/* get first record */
  if ((dat = rawget()) == (struct p3slowblock*)NULL) {
    fprintf(stderr,"hrdrawair: oops! no input data\n");
    exit(1);
  }

/* take care of static slice -- just aircraft number */
  *vars[0] = dat->ac;
  *vars[1] = dat->mo;
  *vars[2] = dat->dy;
  *vars[3] = dat->yr;
  *vars[4] = dat->hr;
  *vars[5] = dat->mn;
  *vars[6] = dat->sc;
  *time = dat->hr*3600. + dat->mn*60. + dat->sc;

/* write header and static slice */
  puthdr(stdout,hbuf,HBMAX);
  putslice(stdout,nsbuff,sbuff);

/* loop on 1 sec records, one per variable slice */
  dayfactor = 0.;
  while ((dat = rawget()) != (struct p3slowblock*)NULL) {
    *vars[7] = dat->nav;
    *vars[8] = dat->la1;
    *vars[9] = dat->lo1;
    *vars[10] = dat->la2;
    *vars[11] = dat->lo2;
    *vars[12] = dat->la4;
    *vars[13] = dat->lo4;
    *vars[14] = dat->gx1;
    *vars[15] = dat->gy1;
    *vars[16] = dat->gx2;
    *vars[17] = dat->gy2;
    *vars[18] = dat->gx4;
    *vars[19] = dat->gy4;
    *vars[20] = dat->hd;
    *vars[21] = dat->ra1;
    *vars[22] = dat->ra2;
    *vars[23] = dat->ra3;
    *vars[24] = dat->pc;
    *vars[25] = dat->rl;
    *vars[26] = dat->ps;
    *vars[27] = dat->ta;
    *vars[28] = dat->td;
    *vars[29] = dat->gs;
    *vars[30] = dat->tk;
    *vars[31] = dat->ws;
    *vars[32] = dat->wd;
    *vars[33] = dat->rd;
    *vars[34] = dat->rs;
    *vars[35] = dat->ru;
    *vars[36] = dat->uw;
    *vars[37] = dat->ui;
    *vars[38] = dat->lw;
    *vars[39] = dat->tas;

/* compute time */
    otime = *time;
    *time = dat->hr*3600. + dat->mn*60. + dat->sc + dayfactor;
    if (*time < otime) {
      *time += DAY;
      dayfactor += DAY;
    }

/* compute drift angle */
    *da = dat->hd - dat->tk;
    if (*da > 180.) *da -= 360.;
    if (*da < -180.) *da += 360.;

/* compute x and y wind components */
    *wx = -dat->ws*sin(dat->wd/DTOR);
    *wy = -dat->ws*cos(dat->wd/DTOR);

/* select appropriate ground speed */
    if (dat->nav == 1.) {
      *gsx = dat->gx1;
      *gsy = dat->gy1;
    }
    else if (dat->nav == 2.) {
      *gsx = dat->gx2;
      *gsy = dat->gy2;
    }
    else {
      fprintf(stderr,"hrdrawrad: nav %f not recognized\n",dat->nav);
      exit(1);
    }

/* select appropriate radar altimeter */
    *ra = dat->ra1;

/* get lon and lat from GPS for now */
    *lon = dat->lo4;
    *lat = dat->la4;

/* write variable slice */
    putslice(stdout,nvbuff,vbuff);
  }
}
