/* hrdrawair2.c -- Translate Dave Jorgensen's ascii file of P-3 slow
 * tape to Candis format.
 */

#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"
#include "rawget2.h"

#define DAY 86400.
#define DTOR 57.2958  /* degrees to radians */

/* Candis stuff */
static char hbuf[HBMAX][LINE];
static char cline[LINE];
static float *sbuff,*vbuff;
static long nsbuff,nvbuff;
static float *vars[NVARS];
static float *lon,*lat,*wx,*wy,*time,*gsx,*gsy,*da;

/* input stuff */
static char *names[] = {
  "ac","yr","mo","dy",
  "ps","ta","rd","rs","tas","tk","hd","pc","rl","lw","td","ru",
  "ga","pa","av1","av2","wgs","was","wz"};
static struct p3slowblock *dat;
struct p3slowblock *rawget2();

main(argc,argv)
int argc;
char *argv[];
{
  int i;
  float dayfactor,otime;

  if(argc > 1) {
    fprintf(stderr,"%s: Too many arguments!\n%s: Usage:\n%s:   %s\n",
        argv[0],argv[0],argv[0],argv[0]);
    exit(1);
  }

  /* make a header */
  nullhdr(hbuf,HBMAX,"float");

  /* comments */
  addcline(hbuf,HBMAX,"hrdrawair2:\n");

  /* static fields */
  for (i = 0; i < 4; i++) {
    addfld(hbuf,HBMAX,'s',names[i],1.,0.,'s',0);
  }

  /* variable fields from input */
  for (i = 4; i < NVARS; i++) {
    addfld(hbuf,HBMAX,'v',names[i],10000.,0.,'l',0);
  }

  /* derived variable fields */
  addfld(hbuf,HBMAX,'v',"wx",100.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"wy",100.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"tptime",1.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"lon",10000.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"lat",10000.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"gsx",10.,0.,'l',0);
  addfld(hbuf,HBMAX,'v',"gsy",10.,0.,'l',0);

  /* allocate space */
  nsbuff = elemcnt(hbuf,HBMAX,'s');
  sbuff = getbuff(nsbuff);
  nvbuff = elemcnt(hbuf,HBMAX,'v');
  vbuff = getbuff(nvbuff);

  /* assign pointers */
  for (i = 0; i < 4; i++) {
    vars[i] = getptr(hbuf,HBMAX,sbuff,'s','d',names[i]);
  }
  for (i = 4; i < NVARS; i++) {
    vars[i] = getptr(hbuf,HBMAX,vbuff,'v','d',names[i]);
  }
  wx = getptr(hbuf,HBMAX,vbuff,'v','d',"wx");
  wy = getptr(hbuf,HBMAX,vbuff,'v','d',"wy");
  time = getptr(hbuf,HBMAX,vbuff,'v','d',"tptime");
  lon = getptr(hbuf,HBMAX,vbuff,'v','d',"lon");
  lat = getptr(hbuf,HBMAX,vbuff,'v','d',"lat");
  gsx = getptr(hbuf,HBMAX,vbuff,'v','d',"gsx");
  gsy = getptr(hbuf,HBMAX,vbuff,'v','d',"gsy");

  /* get first record */
  if ((dat = rawget2()) == (struct p3slowblock*)NULL) {
    fprintf(stderr,"%s: oops! no input data\n",argv[0]);
    exit(1);
  }

  /* take care of static slice -- just aircraft number and date */
  *vars[0] = dat->ac;
  *vars[1] = dat->yr;
  *vars[2] = dat->mo;
  *vars[3] = dat->dy;

  /* write header and static slice */
  puthdr(stdout,hbuf,HBMAX);
  putslice(stdout,nsbuff,sbuff);

  /* loop on 1 sec records, one per variable slice */
  dayfactor = 0.;
  while ((dat = rawget2()) != (struct p3slowblock*)NULL) {
    *vars[4] = dat->ps;
    *vars[5] = dat->ta;
    *vars[6] = dat->rd;
    *vars[7] = dat->rs;
    *vars[8] = dat->tas;
    *vars[9] = dat->tk;
    *vars[10] = dat->hd;
    *vars[11] = dat->pc;
    *vars[12] = dat->rl;
    *vars[13] = dat->lw;
    *vars[14] = dat->td;
    *vars[15] = dat->ru;
    *vars[16] = dat->ga;
    *vars[17] = dat->pa;
    *vars[18] = dat->av1;
    *vars[19] = dat->av2;
    *vars[20] = dat->wgs;
    *vars[21] = dat->was;
    *vars[22] = dat->wz;

    /* compute time */
    otime = *time;
    *time = dat->hr*3600. + dat->mn*60. + dat->sc + dayfactor;
    if (*time < otime) {
      *time += DAY;
      dayfactor += DAY;
    }

    /* compute x and y wind components */
    *wx = -dat->ws*sin(dat->wd/DTOR);
    *wy = -dat->ws*cos(dat->wd/DTOR);

    /* compute x and y ground speed components */
    *gsx = dat->gs*sin(dat->tk/DTOR);
    *gsy = dat->gs*cos(dat->tk/DTOR);

    /* get lon and lat */
    *lon = dat->lod+dat->lom/60;
    *lat = dat->lad+dat->lam/60;

    /* write variable slice */
    putslice(stdout,nvbuff,vbuff);
  }
}
