/* rawget2 -- Get a 1 sec record from P-3 slow tape data file created by
 * Dave Jorgensen for quick look purposes.  Returns a null pointer at
 * end-of-file.
 */

#include <stdio.h>
#include "rawget2.h"

static int bsize,count = 0;
static struct p3slowblock dat;
static int i = 0;

char inputline[140];

struct p3slowblock *rawget2()
{

  /* Extract heading information if first line */
  if (count == 0) {
    count++;
    if(fgets(inputline,140,stdin) == NULL)
      return((struct p3slowblock*)NULL);
    sscanf(inputline,"%2d%2d%2d %2d",&dat.yr,&dat.mo,&dat.dy,&dat.ac);
    return(&dat);
  }

  /* get the variables */

  /* Return a null structure pointer if EOF */
  if(fgets(inputline,140,stdin) == NULL)
    return((struct p3slowblock*)NULL);


  if (strlen(inputline)!=137) {
    fprintf(stderr,"rawget2: input must have 136-char lines after header!\n");
    exit(1);
  }

  /* prepare input string for sscanf (" 20 -30.2 " -> "020-030.20", etc.) */
  for(i = 140;i >= 0;i--)
  {
    if(i>0 && inputline[i] == '-' && inputline[i-1] == ' ')
    {
      inputline[i]='0';
      inputline[i-1]='-';
    }
    else if(inputline[i] == ' ')
      inputline[i] = '0';
  }

  /* fill structure */
  sscanf(inputline,
      "%2d%2d%2d%4f%5f%5f%5f%6f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f%5f"
      ,&dat.hr,&dat.mn,&dat.sc,&dat.lad,&dat.lam,&dat.lod,&dat.lom,&dat.ps,
      &dat.ta,&dat.rd,&dat.rs,&dat.gs,&dat.tas,&dat.tk,&dat.hd,&dat.pc,&dat.rl,
      &dat.lw,&dat.td,&dat.ru,&dat.ga,&dat.pa,&dat.ws,&dat.wd,&dat.av1,
      &dat.av2,&dat.wgs,&dat.was,&dat.wz);

  /* increment line count */
  count++;

  /* return pointer to the data structure */
  return(&dat);
}
