/* fzin -- accept input from rs232 port, send to a sequence of files */
/* if not BSD, System V assumed */
/* note that SunOS 4.0 has System V termio, unlike SunOS 3.* */

#include <stdio.h>
#include <signal.h>
#include <fcntl.h>
#include "cdfhdr.h"

#if BSD
#include <sgtty.h>
struct sgttyb com;

#elseif LINUX
#include <termios.h>
#include <unistd.h>
struct termios com;

#else
#include <termio.h>
struct termio com;
#endif BSD

#include <errno.h>
#define MAX 500                      /* size of input buffer */
#define TIMEOUT 1                    /* Sys V timeout in .1 sec units */
#define  MINCHARS 100                /* Sys V burst package */
#define XON "\021"
#define XOFF "\023"
#define BUFFERS 500                  /* number of buffer files */
int cline;
int speed;
int zero = 0;
int actual,wactual;
char c[MAX];
char filename[80];
char openfile[80];
char command[80];
int interval;
int current_file;
int file_desc;
int switch_flag;
int reset_alarm();

/* start main routine */
main(argc,argv)
int argc;
char *argv[];
{
/* check arguments */
  if (argc < 5) {
    printf("Usage:  fzin linename baudrate file_prefix interval\n");
    exit(1);
  }
  interval = atoi(argv[4]);
  sprintf(openfile,"open%s",argv[3]);
  current_file = 0;
  switch_flag = 1;
  file_desc = -1;
/* get baudrate */
  speed = atoi(argv[2]);
  if (speed == 300) speed = B300;
  else if (speed == 1200) speed = B1200;
  else if (speed == 2400) speed = B2400;
  else if (speed == 4800) speed = B4800;
  else if (speed == 9600) speed = B9600;
  else if (speed == 19200) speed = B19200;
  else {
    printf("%d: bad speed\n",speed);
    exit(1);
  }
/* open comline */
  cline = open(argv[1],2);
  if (cline == -1) {
    fprintf(stderr,"fzin: can't open %s\n",argv[1]);
    exit(1);
  }
/* flush buffer */
#if BSD
  ioctl(cline,TIOCFLUSH,&zero);
#elseif LINUX
  tcflush(cline,TCIOFLUSH);
#else
  ioctl(cline,TCFLSH,0);
#endif BSD
/* set up comline */
#if BSD
  gtty(cline,&com);
  com.sg_ispeed = speed;
  com.sg_ospeed = speed;
  com.sg_flags = RAW | TANDEM;
  stty(cline,&com);
#elseif LINUX
  tcgetattr(cline,&com);
  com.c_iflag = IXOFF | IGNBRK;
  com.c_oflag = 0;
  com.c_cflag = speed | CS8 | CREAD | CLOCAL;
  com.c_lflag = 0;
  tcsetattr(cline,TCSANOW,&com);
#else
  ioctl(cline,TCGETA,&com);
  com.c_iflag = IXOFF | IGNBRK;
  com.c_oflag = 0;
  com.c_cflag = speed | CS8 | CREAD | CLOCAL;
  com.c_lflag = 0;
  com.c_cc[4] = MINCHARS;
  com.c_cc[5] = TIMEOUT;
  ioctl(cline,TCSETA,&com);
#endif BSD
/* set up alarm */
  signal(SIGALRM,reset_alarm);
  alarm(interval);
/* loop forever, transferring characters from comline to tty */
  for (;;) {
/* switch output files if time */
    if (switch_flag) {
      switch_flag = 0;
      if (file_desc >= 0) {
        close(file_desc);
        sprintf(command,"mv %s %s",openfile,filename);
        system(command);
      }
      sprintf(filename,"%s%03d",argv[3],current_file++);
      if (current_file >= BUFFERS) current_file = 0;
      if ((file_desc = open(openfile,O_WRONLY | O_CREAT,0644)) < 0) {
        fprintf(stderr,"fzin: can't create file %s\n",filename);
        exit(5);
      } 
    }
/* do the read from the comline */
    write(cline,XON,1);                 /* turn on com line input */
    actual = read(cline,c,MAX);         /* read a chunk */
    write(cline,XOFF,1);                /* turn off com line input */
    if (actual < 0) {
      fprintf(stderr,"fzin: error %d %d on com line read\n",errno,actual);
      exit(2);
    }
    if ((actual > 0) && ((wactual = write(file_desc,c,actual)) != actual)) {
      fprintf(stderr,"fzin: error %d %d %d on write to %s\n",
           errno,actual,wactual,filename);
      exit(3);
    }
  }
}

/* alarm signal handler */
reset_alarm(sig,code,scp,addr)
int sig,code;
struct sigcontext *scp;
char *addr;
{
/* set alarm flag */
  switch_flag = 1;
/* reset alarm clock */
  alarm(interval);
}
