/* rtape -- Routine to read arbitrary tape on UNIX.
 * The data are sent to the standard output.
 */
#include <stdio.h>
#define MAX 200000                 /* max size of tape record */
#define READONLY 0
#define EMAX 20                    /* default max number of errors */
#define USAGE "Usage: rtape [-bz] [-f fc1:fc2] [-e maxerrors] tape_device\n"

int tape;
char buff[MAX];
long nrecords,nbytes,ecount;
int fc1,fc2,junk,delfc;
char command[80];

/* getopt stuff */
extern char *optarg;
extern int optind;
int c;
 
main(argc,argv)
int argc;
char *argv[];
{
  int emax,filecount,fulltape,blockflag,zeroquit;
  char devname[80];

/* process arguments */
  blockflag = 1;
  fulltape = 0;
  zeroquit = 1;
  emax = EMAX;
  while ((c = getopt(argc,argv,"bf:ze:")) != -1) {
    if (c == 'b') {
      blockflag = 0;
    }
    else if (c == 'f') {
      fulltape = 1;
      if (sscanf(optarg,"%d:%d:%d",&fc1,&fc2,&junk) != 2) {
        fprintf(stderr,"rtape: format of file count argument '%s' incorrect\n",
             optarg);
        exit(1);
      }
      if (fc1 < 1 || fc2 < fc1) {
        fprintf(stderr,"rtape: must have fc1 > 0 and fc2 >= fc1\n");
        exit(1);
      }
      delfc = fc2 - fc1 + 1;
    }
    else if (c == 'e') {
      emax = atoi(optarg);
    }
    else if (c == 'z') {
      zeroquit = 0;
    }
    else {
      fprintf(stderr,USAGE);
      exit(1);
    }
  }
  if (optind != argc - 1) {
    fprintf(stderr,USAGE);
    exit(1);
  }

/* move device name into variable */
  sprintf(devname,"%s",argv[optind]);

/* forward space tape if needed */
  if (fulltape && fc1 > 1) {
    sprintf(command,"mt -f %s fsf %d",devname,fc1 - 1);
    system(command);
  }

/* open tape for read */
  if ((tape = open(devname,READONLY,0)) < 0) {
    fprintf(stderr,"Can't open tape device %s for read\n",devname);
    exit(0);
  }

/* loop on files */
  filecount = 0;
  while (fulltape || filecount == 0) {
    filecount++;
    nrecords = ecount = 0;

/* quit if beyond end file count */
    if (fulltape && filecount > delfc) break;

/* read tape record until one of zero length found */
    while ((nbytes = read(tape,buff,MAX)) != 0) {

/* write stuff if no error (i. e., non-negative nbytes) */
      if (nbytes > 0) {
        if (blockflag == 1) fprintf(stdout,"%8d",nbytes);
        fwrite(buff,sizeof(*buff),nbytes,stdout);
        fflush(stdout);
        nrecords++;
      }
      else {

/* error processing */
        fprintf(stderr,"tape error, record skipped\n");
        ecount++;
        if (ecount > emax) {
          fprintf(stderr,"rtape: error count exceeds %d in this file\n",emax);
          break;
        }
      }
    }

/* optional exit condition -- zero record length file, i. e., two tape marks */
    if (nrecords == 0) {
      fprintf(stderr,"rtape: zero records in this file\n");
      if (zeroquit) break;
    }

/* if fulltape, write a zero length header at this point */
    if (fulltape && blockflag) fprintf(stdout,"%8d",0);
  }

/* close tape */
  close(tape);

/* quit */
  exit(0);
}
