/* wtape -- Routine to write tape on Berkeley UNIX from rtape format.
 * Data are taken from the standard input.
 * The maximum allowable number of bytes per tape record is MAX.
 * Set device argument according to mtio(4) in the Berkeley UNIX manual
 * to get the proper tape density (/dev/rmt4 for 800 bpi,
 * /dev/rmt12 for 1600 bpi.) and device.
 */
#include <stdio.h>
#define MAX 200000
#define WRITEONLY 1
int tape,eof,i;
char buff[MAX],head[9];
long int nbytes,er;
 
main(argc,argv)
int argc;
char *argv[];
{
/* check arguments */
  if (argc != 2) {
    fprintf(stderr,"Usage: wtape /dev/raw_tape_device\n");
    exit(1);
  }
/* open tape for write */
  if ((tape = open(argv[1],WRITEONLY,0)) < 0) {
    fprintf(stderr,"Can't open tape device %s for write\n",argv[1]);
    exit(1);
  }
/* read disk and write to tape */
  for (;;) {
/* look at rtape ascii header to find number of bytes in record */
    eof = 0;
    for (i = 0; i < 8; i++) {
      if ((head[i] = getchar()) == EOF) {
        eof = 1;
        break;
      }
    }
    head[8] = '\0';
    nbytes = atoi(head);
    if (eof) break;
/* write a record only if length is greater than zero */
    if (nbytes > 0) {
      if (fread(buff,sizeof(*buff),nbytes,stdin) <= 0) {
        fprintf(stderr,"phase error on disk file\n");
        exit(1);
      }
      if ((er = write(tape,buff,nbytes)) <= 0) {
        fprintf(stderr,"error in tape write\n");
        exit(1);
      }
    }
  }
/* close tape */
  close(tape);
/* finish up */
/*
  fprintf(stderr,"done\n");
*/
  exit(0);
}
