/* zin -- accept input from rs232 port, send to stdout */
/* provision is made for error file */
/* if not BSD or LINUX, System V assumed */
/* note that SunOS 4.0 has System V termio, unlike SunOS 3.* */

#include <stdio.h>
#include "cdfhdr.h"

#if BSD
#include <sgtty.h>
struct sgttyb com;

#elseif LINUX
#include <termios.h>
#include <unistd.h>
struct termios com;

#else
#include <termio.h>
struct termio com;
#endif BSD

#include <errno.h>
#define MAX 500                      /* size of input buffer */
#define TIMEOUT 5                   /* Sys V timeout in .1 sec units */
#define  MINCHARS 500                /* Sys V burst package */
#define XON "\021"
#define XOFF "\023"
int cline;
int speed;
int zero = 0;
int actual,wactual;
char c[MAX];
FILE *efile;
int reset_alarm();

/* start main routine */
main(argc,argv)
int argc;
char *argv[];
{
/* check arguments */
  if ((argc < 3) || (argc > 4)) {
    printf("Usage:  zin linename baudrate [error_file]\n");
    exit(1);
  }
/* make error file arrangements */
  if (argc == 4) {
    if ((efile = fopen(argv[3],"a")) == NULL) efile = stderr;
  }
  else efile = stderr;
/* get baudrate */
  speed = atoi(argv[2]);
  if (speed == 300) speed = B300;
  else if (speed == 1200) speed = B1200;
  else if (speed == 2400) speed = B2400;
  else if (speed == 4800) speed = B4800;
  else if (speed == 9600) speed = B9600;
  else if (speed == 19200) speed = B19200;
  else {
    printf("%d: bad speed\n",speed);
    exit(1);
  }
/* open comline */
  cline = open(argv[1],2);
  if (cline == -1) {
    fprintf(efile,"zin: can't open %s\n",argv[1]);
    exit(1);
  }
/* flush buffer */
#if BSD
  ioctl(cline,TIOCFLUSH,&zero);
#elseif LINUX
  tcflush(cline,TCIOFLUSH);
#else
  ioctl(cline,TCFLSH,0);
#endif BSD
/* set up comline */
#if BSD
  gtty(cline,&com);
  com.sg_ispeed = speed;
  com.sg_ospeed = speed;
  com.sg_flags = RAW | TANDEM;
  stty(cline,&com);
#elseif LINUX
  tcgetattr(cline,&com);
  com.c_iflag = IXOFF | IGNBRK;
  com.c_oflag = 0;
  com.c_cflag = speed | CS8 | CREAD | CLOCAL;
  com.c_lflag = 0;
  tcsetattr(cline,TCSANOW,&com);
#else
  ioctl(cline,TCGETA,&com);
  com.c_iflag = IXOFF | IGNBRK;
  com.c_oflag = 0;
  com.c_cflag = speed | CS8 | CREAD | CLOCAL;
  com.c_lflag = 0;
  com.c_cc[4] = MINCHARS;
  com.c_cc[5] = TIMEOUT;
  ioctl(cline,TCSETA,&com);
#endif BSD
/* loop forever, transferring characters from comline to tty */
  for (;;) {
    write(cline,XON,1);                 /* turn on com line input */
    actual = read(cline,c,MAX);         /* read a chunk */
    write(cline,XOFF,1);                /* turn off com line input */
    if (actual < 0) {
      fprintf(efile,"zin: error %d %d on com line read\n",errno,actual);
      exit(2);
    }
    if ((actual > 0) && ((wactual = write(1,c,actual)) != actual)) {
      fprintf(efile,"zin: error %d %d %d on stdout write\n",
           errno,actual,wactual);
      exit(3);
    }
  }
}
