/* navpac -- This set of routines does navigation of latitude and
 * longitude lines and plots them on the image.
 */
#include <stdio.h>
#include <math.h>
#include "mcidas.h"
#define EARTH_RADIUS 6370.
#define DEG_PER_RAD 57.2958

int dumbnav();
double gst();
float trunc1();

/* drawlatlon -- This routine draws latitude and longitude lines on
 * image using line and text routines supplied by dispimage.
 */
drawlatlon(scratchpad,navheader)
int scratchpad;
struct goes *navheader;
{
  float lon,lat,inc,hinc,lonlatint;
  int x1,y1,x2,y2;
  int needtext,xoffset,yoffset;
  char text[40];
  int ifast,fast;
  float lon11,lat11,lat12,lon12,lat21,lon21,lat22,lon22,lon3,lat3;
  float lonmin,lonmax,latmin,latmax;

/* initialize for slow (complete) comp in dumbnav first time */
  fast = 0;
  ifast = 0;

/* set latitude-longitude line intervals and size of tics */
  if (navheader->xres == 1) {
    lonlatint = 2.;
    inc = .25;
  }
  else if (navheader->xres == 2) {
    lonlatint = 2.;
    inc = .5;
  }
  else if (navheader->xres == 4) {
    lonlatint = 5.;
    inc = 1.;
  }
  else {
    lonlatint = 10.;
    inc = 2.;
  }
  hinc = .5*inc;

/* set the text offset */
  xoffset = -10;
  yoffset = 10;

/* find reasonable latitude and longitude limits */
  invnav(navheader,0,0,&lon12,&lat12,&fast,&ifast);
  invnav(navheader,navheader->xsize,0,&lon22,&lat22,&fast,&ifast);
  invnav(navheader,0,navheader->ysize,&lon11,&lat11,&fast,&ifast);
  invnav(navheader,navheader->xsize,navheader->ysize,
       &lon21,&lat21,&fast,&ifast);
  invnav(navheader,navheader->xsize/2,0,&lon3,&lat3,&fast,&ifast);
  lonmin = lon11 < lon12 ? lon11 : lon12;
  lonmax = lon21 > lon22 ? lon21 : lon22;
  latmin = lat11 < lat21 ? lat11 : lat21;
  latmax = lat12 > lat22 ? lat12 : lat22;
  latmax = latmax > lat3 ? latmax : lat3;
  lonmin = trunc1(lonmin,lonlatint);
  lonmax = trunc1(lonmax,lonlatint) + lonlatint;
  latmin = trunc1(latmin,lonlatint);
  latmax = trunc1(latmax,lonlatint) + lonlatint;
  if (lonmin < -180.) lonmin = 180.;
  if (lonmax > 180.) lonmax = 180.;
  if (latmin < -90.) latmin = -90.;
  if (latmax > 90.) latmax = 90.;

/* problem if image spans dateline */
  if (lonmin > lonmax) {
    lonmin = -180.;
    lonmax = 180.;
  }

/* draw constant longitude lines */
  for (lon = lonmin; lon < lonmax; lon += lonlatint) {
    needtext = 1;
    for (lat = latmin; lat < latmax; lat += inc) {
      if (dumbnav(navheader,lon,lat,&x1,&y1,&fast) < 0) continue;
      if (dumbnav(navheader,lon,lat + hinc,&x2,&y2,&fast) < 0) continue;
      drawline(scratchpad,x1,y1,x2,y2);

/* print longitude numbers */
      if (needtext) {
        if (y2 > .85*navheader->ysize) {
          sprintf(text,"%.0f",lon);
          drawtext(scratchpad,x2 + xoffset,y2 + yoffset - 20,text);
          needtext = 0;
        }
      }
    }
  }

/* draw constant latitude lines */
  for (lat = latmin; lat < latmax; lat += lonlatint) {
    needtext = 1;
    for (lon = lonmin; lon < lonmax; lon += inc) {
      if (dumbnav(navheader,lon,lat,&x1,&y1,&fast) < 0.) continue;
      if (dumbnav(navheader,lon + hinc,lat,&x2,&y2,&fast) < 0.) continue;
      drawline(scratchpad,x1,y1,x2,y2);

/* print latitude numbers */
      if (needtext) {
        if (x2 > .03*navheader->xsize) {
          sprintf(text,"%.0f",lat);
          drawtext(scratchpad,x2 + xoffset,y2 + yoffset,text);
          needtext = 0;
        }
      }
    }
  }
}

/* trunc1 -- do mod operation */
float trunc1(a,m)
float a,m;
{
  a = (int)((a + 1000.*m)/m);
  a = (a - 1000.)*m;
  return(a);
}
