/* invnav -- invert the navigation problem -- given an image position,
 * find the latitude and longitude.  Uses dumbnav iteratively.
 * 0 is returned on success, -1 on failure.
 */

#include <stdio.h>
#include <math.h>
#include "mcidas.h"

#define DLON 5
#define DLAT 5
#define NLON 73
#define NLAT 37

static int xt[NLON][NLAT],yt[NLON][NLAT];     /* trial grid */
static int onimage[NLON][NLAT];        /* 0 if point on image, -1 otherwise */

int invnav(navheader,x,y,flon,flat,fast,ifast)
struct goes *navheader;                /* contains nav information */
int x,y;                               /* image coordinates of point */
float *flon,*flat;                     /* latitude and longitude */
int *fast;                             /* fast flag for dumbnav */
int *ifast;                            /* fast flag for invnav */
{
  int lon,lat,lon1,lat1,lon2,lat2;     /* lat-lon indices */
  int alon,alat;                       /* nearest grid point */
  int blon,blat;                       /* spanning grid point */
  long sdist,newsdist;                 /* test distances */
  float x1,x2,x3,y1,y2,y3,a1,a2,a3,dd,da,db,dc;  /* temporaries */
  int xr,yr;                           /* temporaries */

/* fill in trial grid if ifast flag not set */
  if (!(*ifast)) {
    for (lat = 0; lat < NLAT; lat++) {
      for (lon = 0; lon < NLON; lon++) {
        onimage[lon][lat] = dumbnav(navheader,(float)(lon*DLON - 180),
             (float)(lat*DLAT - 90),&xt[lon][lat],&yt[lon][lat],fast);
      }
    }
    *ifast = 1;
  }

/* first, find nearest point */
  sdist = 1000000;
  lon1 = lat1 = 1000;
  for (lat = 0; lat < NLAT; lat++) {
    for (lon = 0; lon < NLON; lon++) {
      if (onimage[lon][lat] > -2 &&
           (newsdist = abs(x - xt[lon][lat]) + abs(y - yt[lon][lat])) < sdist) {
        lon1 = lon;
        lat1 = lat;
        sdist = newsdist;
      }
    }
  }

/* next, find bracketing point */
  alon = lon1*DLON - 180;
  alat = lat1*DLAT - 90;
  if (x - xt[lon1][lat1] > 0) blon = alon + DLON;
  else blon = alon - DLON;
  if (blon > 180) {
    blon -= 360;
    alon -= 360;
  }
  if (blon < -180) {
    blon += 360;
    alon += 360;
  }
  if (y - yt[lon1][lat1] > 0) blat = alat - DLAT;
  else blat = alat + DLAT;
  lon2 = (blon + 180)/DLON;
  lat2 = (blat + 90)/DLAT;

/* this should never happen */
  if (alon < -180 || alon > 180) {
    fprintf(stderr,"invnav panic: alon = %d\n",alon);
    exit(1);
  }

/* put in rough default values if off view */
  *flon = alon;
  *flat = alat;

/* give up if anything in square is invisible or out of range */
  if (blat < -90 || blat > 90) return(-1);
  if (blon < -180 || blon > 180) return(-1);
  if (onimage[lon1][lat2] == -2 || onimage[lon2][lat1] == -2) return(-1);

/* finally, linearly interpolate */
  x1 = xt[lon1][lat1];
  x2 = xt[lon1][lat2];
  x3 = xt[lon2][lat1];
  y1 = yt[lon1][lat1];
  y2 = yt[lon1][lat2];
  y3 = yt[lon2][lat1];
  a1 = alon;
  a2 = alon;
  a3 = blon;
  dd = x2*y3 + x1*y2 + x3*y1 - x2*y1 - x3*y2 - x1*y3;
  da = a1*x2*y3 + a3*x1*y2 + a2*x3*y1 - a3*x2*y1 - a2*x1*y3 - a1*x3*y2;
  db = a2*y3 + a1*y2 + a3*y1 - a2*y1 - a3*y2 - a1*y3;
  dc = x2*a3 + x1*a2 + x3*a1 - x2*a1 - x3*a2 - x1*a3;
  *flon = (da + db*x + dc*y)/dd;
  a1 = alat;
  a2 = blat;
  a3 = alat;
  da = a1*x2*y3 + a3*x1*y2 + a2*x3*y1 - a3*x2*y1 - a2*x1*y3 - a1*x3*y2;
  db = a2*y3 + a1*y2 + a3*y1 - a2*y1 - a3*y2 - a1*y3;
  dc = x2*a3 + x1*a2 + x3*a1 - x2*a1 - x3*a2 - x1*a3;
  *flat = (da + db*x + dc*y)/dd;

/* successful return */
  return(0);
}
