/* getimage.c -- Candis version of subroutine that reads an image
 * from a file produced by mcselect.
 */

#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"
#include "mcidas.h"

static char hbuff[HBMAX][LINE];
static char cline[LINE],navtype[LINE];
static long nsbuff,nvbuff;
static float *sbuff,*vbuff;
static char *image;
static char *imagebuff;
static struct field *fp;
static FILE *imagein;

char *getimage(filename,index,gptr,nxsize,nysize,nlon,nlat)
char *filename;               /* name of file containing image */
int index;                    /* position in sequence */
struct goes *gptr;            /* pointer to info structure */
int nxsize,nysize;            /* new image size */
float nlon,nlat;              /* new longitude and latitude */
{
  int nx,ny,ix,iy,jx,jy;
  int nxcoor,nycoor;
  int newx,newy,fast;
  long xoffset,yoffset;
  
  /* open image file */
  if ((imagein = fopen(filename,"r")) == NULL) {
    fprintf(stderr,"getimage: can't open file %s\n",filename);
    exit(1);
  }
  
  /* get header */
  gethdr(imagein,hbuff,HBMAX);
  if (getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"getimage: float format expected\n");
    exit(1);
  }
  
  /* allocate buffer space */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  sbuff = getbuff(nsbuff);
  nvbuff = elemcnt(hbuff,HBMAX,'v');
  vbuff = getbuff(nvbuff);
  
  /* get pointer to image */
  image = (char*)getptr2(hbuff,HBMAX,vbuff,'v','d',"image",&fp);
  nx = fp->dsize2*sizeof(float);
  ny = fp->dsize1;
  
  /* get slices */
  getslice(imagein,nsbuff,sbuff);
  getslice(imagein,nvbuff,vbuff);
  
  /* get the image type */
  if (seekpar(hbuff,HBMAX,"navtype",navtype) != OK) {
    fprintf(stderr,"mcdisp: no navtype parameter\n");
    exit(1);
  }
  strcpy(gptr->navtype,navtype);
  
  /* get x and y sizes, resolutions, and coordinates */
  gptr->xsize = (long)*getptr(hbuff,HBMAX,sbuff,'s','d',"xsize");
  gptr->ysize = (long)*getptr(hbuff,HBMAX,sbuff,'s','d',"ysize");
  if (gptr->xsize != nx || gptr->ysize != ny) {
    fprintf(stderr,"mcdisp: (nx,ny) != (xsize,ysize)\n");
    exit(1);
  }
  gptr->xres = (long)*getptr(hbuff,HBMAX,sbuff,'s','d',"xres");
  gptr->yres = (long)*getptr(hbuff,HBMAX,sbuff,'s','d',"yres");
  gptr->xcoor = (long)*getptr(hbuff,HBMAX,sbuff,'s','d',"xcoor");
  gptr->ycoor = (long)*getptr(hbuff,HBMAX,sbuff,'s','d',"ycoor");
  
  /* make sure that nxsize is integral multiple of 8 and non-negative */
  if (nxsize < 0) nxsize = 0;
  if (nysize < 0) nysize = 0;
  nxsize = nxsize/8;
  nxsize = nxsize*8;
  
  /* get time */
  gptr->year = (int)*getptr(hbuff,HBMAX,vbuff,'v','d',"year");
  gptr->day = (int)*getptr(hbuff,HBMAX,vbuff,'v','d',"day");
  gptr->hour = (int)*getptr(hbuff,HBMAX,vbuff,'v','d',"hour");
  gptr->minute = (int)*getptr(hbuff,HBMAX,vbuff,'v','d',"minute");
  gptr->second = (int)*getptr(hbuff,HBMAX,vbuff,'v','d',"second");

  /* get bandmap */
  if (seekpar(hbuff,HBMAX,"bandmap",cline) == FAIL) failpar("bandmap");
  gptr->bandmap = atoi(cline);
  
  /* get parameters needed for navigation */
  if (strcmp(navtype,"GOES") == 0) {
    if (seekpar(hbuff,HBMAX,"epochd",cline) == FAIL) failpar("epochd");
    gptr->epochd = atoi(cline);
    if (seekpar(hbuff,HBMAX,"epocht",cline) == FAIL) failpar("epocht");
    gptr->epocht = atoi(cline);
    if (seekpar(hbuff,HBMAX,"semima",cline) == FAIL) failpar("semima");
    gptr->semima = atof(cline);
    if (seekpar(hbuff,HBMAX,"eccen",cline) == FAIL) failpar("eccen");
    gptr->eccen = atof(cline);
    if (seekpar(hbuff,HBMAX,"orbinc",cline) == FAIL) failpar("orbinc");
    gptr->orbinc = atof(cline);
    if (seekpar(hbuff,HBMAX,"meana",cline) == FAIL) failpar("meana");
    gptr->meana = atof(cline);
    if (seekpar(hbuff,HBMAX,"perigee",cline) == FAIL) failpar("perigee");
    gptr->perigee = atof(cline);
    if (seekpar(hbuff,HBMAX,"asnode",cline) == FAIL) failpar("asnode");
    gptr->asnode = atof(cline);
    if (seekpar(hbuff,HBMAX,"declin",cline) == FAIL) failpar("declin");
    gptr->declin = atof(cline);
    if (seekpar(hbuff,HBMAX,"rascen",cline) == FAIL) failpar("rascen");
    gptr->rascen = atof(cline);
    if (seekpar(hbuff,HBMAX,"deglin",cline) == FAIL) failpar("deglin");
    gptr->deglin = atof(cline);
    if (seekpar(hbuff,HBMAX,"degele",cline) == FAIL) failpar("degele");
    gptr->degele = atof(cline);
    if (seekpar(hbuff,HBMAX,"lintot",cline) == FAIL) failpar("lintot");
    gptr->lintot = atoi(cline);
    if (seekpar(hbuff,HBMAX,"eletot",cline) == FAIL) failpar("eletot");
    gptr->eletot = atoi(cline);
    if (seekpar(hbuff,HBMAX,"pitch",cline) == FAIL) failpar("pitch");
    gptr->pitch = atof(cline);
    if (seekpar(hbuff,HBMAX,"yaw",cline) == FAIL) failpar("yaw");
    gptr->yaw = atof(cline);
    if (seekpar(hbuff,HBMAX,"roll",cline) == FAIL) failpar("roll");
    gptr->roll = atof(cline);
  }
  else if (strcmp(navtype,"GVAR") == 0) {
    if (seekpar(hbuff,HBMAX,"pitch",cline) == FAIL) failpar("pitch");
    gptr->pitch = atof(cline);
    if (seekpar(hbuff,HBMAX,"yaw",cline) == FAIL) failpar("yaw");
    gptr->yaw = atof(cline);
    if (seekpar(hbuff,HBMAX,"roll",cline) == FAIL) failpar("roll");
    gptr->roll = atof(cline);
    if (seekpar(hbuff,HBMAX,"lam",cline) == FAIL) failpar("lam");
    gptr->lam = atof(cline);
    if (seekpar(hbuff,HBMAX,"dr",cline) == FAIL) failpar("dr");
    gptr->dr = atof(cline);
    if (seekpar(hbuff,HBMAX,"phi",cline) == FAIL) failpar("phi");
    gptr->phi = atof(cline);
    if (seekpar(hbuff,HBMAX,"psi",cline) == FAIL) failpar("psi");
    gptr->psi = atof(cline);
    if (seekpar(hbuff,HBMAX,"rma",cline) == FAIL) failpar("rma");
    gptr->rma = atof(cline);
    if (seekpar(hbuff,HBMAX,"pma",cline) == FAIL) failpar("pma");
    gptr->pma = atof(cline);
    if (seekpar(hbuff,HBMAX,"elevbnds",cline) == FAIL) failpar("elevbnds");
    gptr->elevbnds = atof(cline);
    if (seekpar(hbuff,HBMAX,"elevdln",cline) == FAIL) failpar("elevdln");
    gptr->elevdln = atof(cline);
    if (seekpar(hbuff,HBMAX,"scanbnds",cline) == FAIL) failpar("scanbnds");
    gptr->scanbnds = atof(cline);
    if (seekpar(hbuff,HBMAX,"scanpix",cline) == FAIL) failpar("scanpix");
    gptr->scanpix = atof(cline);
    if (seekpar(hbuff,HBMAX,"lineoffset",cline) == FAIL) failpar("lineoffset");
    gptr->lineoffset = atof(cline);
    if (seekpar(hbuff,HBMAX,"scanoffset",cline) == FAIL) failpar("scanoffset");
    gptr->scanoffset = atof(cline);
  }
  else {
    fprintf(stderr,"getimage: unknown navtype %s\n",navtype);
    exit(1);
  }

/* find new xcoor, ycoor if needed */
  if (nxsize != 0 && nysize != 0) {
    fast = 0;
    if (dumbnav(gptr,nlon,nlat,&newx,&newy,&fast) < 0) {
      fprintf(stderr,"getimage: lon-lat %f %f out of picture\n",nlon,nlat);
      exit(1);
    }
    newx = newx/4;
    newx = newx*4;
    nxcoor = gptr->xcoor + (newx - nxsize/2)*gptr->xres;
    nycoor = gptr->ycoor + (newy - nysize/2)*gptr->yres;
  }

/* put in default values for new image coordinates if necessary */
  else {
    nxsize = gptr->xsize;
    nysize = gptr->ysize;
    nxcoor = gptr->xcoor;
    nycoor = gptr->ycoor;
  }

/* we must transfer on word boundary */
  xoffset = (nxcoor - gptr->xcoor)/gptr->xres;
  xoffset = xoffset/4;
  xoffset = xoffset*4;
  nxcoor = gptr->xcoor + xoffset*gptr->xres;
  yoffset = (nycoor - gptr->ycoor)/gptr->yres;

/* allocate image buffer */
  imagebuff = (char*)getbuff(nxsize*nysize/sizeof(float) + 1);

/* transfer image data to buffer after swapping bytes if necessary */
  if (FLOATSWAP) floatswap(gptr->xsize*gptr->ysize/sizeof(float),
			   (float*)image);
  for (iy = 0; iy < nysize; iy++) {
    for (ix = 0; ix < nxsize; ix++) {
      jx = xoffset + ix;
      jy = yoffset + iy;
      if (jx >= 0 && jx < gptr->xsize && jy >= 0 && jy < gptr->ysize) {
        imagebuff[ix + nxsize*iy] = image[jx + gptr->xsize*jy];
      }
      else {
        imagebuff[ix + nxsize*iy] = 0;
      }
    }
  }

/* fix up goes header */
  gptr->xcoor = nxcoor;
  gptr->ycoor = nycoor;
  gptr->xsize = nxsize;
  gptr->ysize = nysize;

/* close image file and free buffers */
  free(vbuff);
  free(sbuff);
  fclose(imagein);

/* return stuff to main program */
  return(imagebuff);
}

/* failure routine */
failpar(text)
char *text;
{
  fprintf(stderr,"getimage: failed to find required parameter %s\n",text);
  exit(1);
}
