/* getpacket -- This subroutine reads a mcidas data stream from the
 * standard input, returning a pointer to each mcidas data packet
 * in turn.  On end of file, returns NULL.
 */

#include <stdio.h>
#include <math.h>
#include "mcidas.h"

static struct mcpacket packet;     /* this buffers the received packet */

struct mcpacket *getpacket()
{
  unsigned char buffer;            /* input buffer */
  int loop;                        /* looping variable */

/* loop on trials */
  while (1) {

/* get a candidate packet header */
    while (1) {
      if (mcread(&buffer,1) != 1) return(NULL);
      if ((buffer & 0xf0) == 0xf0) break;
    }
    packet.fxub = buffer;
    packet.thread = (buffer >> 2) & 3;

/* get routing code */
    if (mcread(&buffer,1) != 1) return(NULL);
    packet.r_code = buffer;

/* get length bytes */
    if (mcread(&buffer,1) != 1) return(NULL);
    packet.length1 = buffer;
    if (mcread(&buffer,1) != 1) return(NULL);
    packet.length2 = buffer;

/* calculate length */
    packet.length = 256*packet.length1 + packet.length2 - 5;
    if (packet.length < 0 || packet.length > DATAMAX) {
/*
      fprintf(stderr,"getpacket: improper packet length %d\n",packet.length);
*/
      continue;
    }

/* get data */
    if (mcread(packet.data,packet.length) != packet.length) return(NULL);

/* null terminate it */
    packet.data[packet.length] = '\0';

/* get marker */
    if (mcread(&buffer,1) != 1) return(NULL);
    packet.marker = buffer;

/* get end character */
/*
    if (mcread(&buffer,1) != 1) return(NULL);
    if (buffer != 0x0d) {
      fprintf(stderr,"getpacket: end of packet character %x != CR\n",buffer);
      continue;
    }
*/

/* return packet pointer */
    return(&packet);
  }
}

/* mcread -- this routine reads code from pc-mcidas stream and takes
 * care of escaped characters.  returns number of characters read.
 */

mcread(data,elements)
unsigned char *data;               /* pointer to data storage area */
int elements;                      /* number of requested data elements */
{
  int nread;
  char buff;
  nread = 0;
  while (nread < elements) {
    if (fread(&buff,sizeof(char),1,stdin) == 0) {
      return(nread);
    }
    if (buff == 0x1b) {
      if (fread(&buff,sizeof(char),1,stdin) == 0) {
        return(nread);
      }
      buff = buff & 0x9f;
    }
    data[nread++] = buff;
  }
  return(nread);
}
