/* getstream.c -- This routine converts the packet data structure of
 * pc-mcidas to a stream structure, where an arbitrary number of bytes
 * may be obtained irrespective of the packet structure.
 * Getstream(pointer,number) reads "number" bytes into the location
 * pointed to by "pointer".  Resetstream() forces a new packet to be
 * read on the next call to getstream.  Resetstream must be called
 * before the first call to getstream or any of the other
 * routines.  Getshort gets 2 bytes from the stream and interprets
 * it as a short integer.  Getword gets 4 bytes from
 * the stream and interprets it as a long integer.  Getsync gets
 * 8 bytes from the stream and returns 0 if they are 0x00ff00ff00ff00ff.
 * Getword assumes that a long is 4 bytes.
 * This stuff won't work on Crays.  Getskip(number) skips the indicated
 * number of bytes in the stream.
 */

#include <stdio.h>
#include <math.h>
#include "mcidas.h"

static int packet_pos;              /* current position in data */
static struct mcpacket *packet;     /* pointer to current data packet */
struct mcpacket *getpacket();

getstream(pointer,number)
char *pointer;
int number;
{
  int loop;

/* loop on points required */
  for (loop = 0; loop < number; loop++) {

/* get a new packet if required */
    if (packet_pos >= packet->length) {
      while (1) {
        if ((packet = getpacket()) == NULL) {
          fprintf(stderr,"getstream: unexpected eof\n");
          exit(1);
        }
        if (packet->thread == THREAD) break;
        else {
          fprintf(stderr,"getstream: packet with thread %d ignored\n",
               packet->thread);
        }
      }
      packet_pos = 0;
    }

/* transfer a byte and increment position in packet */
    pointer[loop] = packet->data[packet_pos++];
  }
  return(0);
}

resetstream()
{

/* read a new packet and reset packet position to zero */
  while (1) {
    if ((packet = getpacket()) == NULL) {
      fprintf(stderr,"getstream: unexpected eof\n");
      exit(1);
    }
    if (packet->thread == THREAD) break;
    else {
      fprintf(stderr,"getstream: packet with thread %d ignored\n",
           packet->thread);
    }
  }
  packet_pos = 0;
  return(0);
}

short getshort()
{
  unsigned char buff[2];
  short word;

/* convert 2 bytes into a short integer */
  getstream(buff,2);

/* this should work for both VAXs and others */
  word = buff[1] + 256*buff[0];
  return(word);
}

long getword()
{
  unsigned char buff[4];
  long word;

/* convert 4 bytes into a long word */
/* this routine assumes that a long is 4 bytes */
  getstream(buff,4);

/* this should work for both VAXs and others */
  word = buff[3] + 256*buff[2] + 65536*buff[1] + 16777216*buff[0];
  return(word);
}

getsync()
{
  unsigned char buff[8];
  int goop;
  int trycount;

/* get the next 8 bytes and test for sync pattern */
  trycount = 0;
  if (getstream(buff,8) == -1) return(-1);
  while (1) {
    if (buff[0] == 0 && buff[1] == 0xff && buff[2] == 0 && buff[3] == 0xff &&
         buff[4] == 0 && buff[5] == 0xff && buff[6] == 0 &&
         buff[7] == 0xff) return(trycount);
    else {

/* we didn't get a sync -- shift, add a byte, and retest */
      trycount++;
      if (trycount >= TRYMAX) return(trycount);
      for (goop = 1; goop < 8; goop++) buff[goop - 1] = buff[goop];
      if (getstream(buff + 7,1) == -1) return(-1);
    }
  }
}

getskip(number)
int number;
{
  int loop;

/* loop on points required */
  for (loop = 0; loop < number; loop++) {

/* get a new packet if required */
    if (packet_pos >= packet->length) {
      if ((packet = getpacket()) == NULL) {
        fprintf(stderr,"getstream: unexpected eof\n");
        exit(1);
      }
      packet_pos = 0;
    }

/* increment packet postion */
    packet_pos++;
  }
}
