/* mcmap -- this program converts files produced by the mcidas system
 * out of pixel format and into the standard candis format the pixel values 
 * are converted either to temperature or brightness scales by the effective 
 * opposite of the functions ibrite and itemp in procimage. the field is also
 * converted to a grid in latitude and longitude (this is the main trick).
 * this is accomplished primarily by using the dumbnav routine found in
 * navpac and looping on latitude and longitude. The number of points in the 
 * proscessed field is dependent on the setting of the extreme latitude
 * and longitudes as well as the dlong and dlat values (all float). Choosing
 * a small value of dlat or dlong will cause the program to become incredibly 
 * slow so keep dlong and dlat relatively large. 
 */

#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"
#include "mcidas.h"

#define SANELAT1  0.00 
#define SANELAT2  70.00
#define SANELON1 -180.00
#define SANELON2  0.0

#define NAME argv[0]
#define BADR 1000
#define BADVAL 999.999

struct field *fb;                /* field info pointer                     */
char newname[LINE];              /* name of the converted field            */
char fname[LINE];                /* name of the old field (image)          */
char navtype[LINE];
float latlow,lathigh;            /* latitude extremes                      */
float longlow,longhigh;          /* longitude extremes                     */
float dlat,dlong;                /* change in lat or long per pixel        */
char type;                       /* type of image 'i','w' or 'v'           */
char hbuff[HBMAX][LINE];         /* buffer for the old header              */
char newbuff[HBMAX][LINE];       /* buffer for the new header              */
int bit,pak,nssze;               /* pixel size,number per long             */
long selmnt, velmnt;             /* slice element counts                   */
long snewsize;                   /* sixe of static field after changes     */
float xsize,ysize;               /* size of the image                      */
unsigned long MASK[] = {
  0,1,3,0,15,0,0,0,255,0,0,0,0,0,0,0,64535,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,4164895295};

main(argc,argv)
int argc;                        /* there should be 8 arguments            */
char *argv[];
{

  char cline[LINE],value[LINE] ;  /* comment line , parameter check       */
  float *sptr,*vptr;              /* pointers to buffers                  */
  long newfsize,oldsize;          /* old and new element counts           */
  char prec;                      /* precision 's','l','p'                */
  long nnum,tgtofst;              /* element sizes for putslice2          */
  char thedim[LINE],samedim[LINE];/* name of 2nd dim of target field      */
  char d1[LINE],d2[LINE];         /* dimension names of old field         */
  char fmt ;                      /* format check char should be 'f'      */
  int loop,par;                   /* obvious,check perameter              */
  long bnum,cnvsze,cnt;           /* counters for slice sub-buffers       */
  float *vnewptr,*snewptr;        /* pointers to slice sub-buffers        */
  float *slastptr,*lastptr;       /* pointers to ends of buffers          */
  float *temptr;                  /* temporary pointer for sampling data  */
  long nfsze,bgnsze,sfirst;       /* sub-buffer counters                  */
  long newfsze,sendsze,lastsze;   /* more of the same                     */
  float smul,sadd;                /* multiplier and additive constant     */
  int slice1 = 1;                 /* flag for first iteration             */
  char otherdim[LINE];            /* the first dimension                  */
  float *xsizeptr,*ysizeptr;      /* size in pixels (x,y) of the image    */
  struct goes *navheader;         /* storage for navigation info          */
  struct goes nav;                /* the actual storage of nav info       */
  long latsize,longsize;          /* number of points in lat or long      */
  float *lon,*lat;                /* storage space for static fields      */
  long putslice2();               /* altered putslice function            */


  /* check for correct number of arguments (8) */
  if (argc != 8){
    fprintf(stderr,"Usage:");
    fprintf(stderr," %s lonlow dlon lonhigh latlow dlat lathigh",NAME);
    fprintf(stderr," newname  \n");
    exit(1);
  }

  /* set up the lat and long extremes  and deltas */
  latlow   = atof(argv[4]);
  dlat     = atof(argv[5]);
  lathigh  = atof(argv[6]);
  longlow  = atof(argv[1]);
  dlong    = atof(argv[2]);
  longhigh = atof(argv[3]);

  /* get the name of the new field */
  strcpy(newname,argv[7]);
  /* set fname to image as always */
  strcpy(fname,"image");

  /* get the header  */
  gethdr(stdin,hbuff,HBMAX);

  /* check that format is float */
  if(( fmt = getfmt(hbuff,HBMAX)) != 'f'){
    fprintf(stderr,"%s: format not float \n",NAME);
    exit(1);
  }

  /* check that the field exists */
  if(( fb = seekfld(hbuff,HBMAX,'v',fname)) == NULL){
    fprintf(stderr,"%s: field %s nonexistent \n",NAME,fname);
    exit(1);
  }

  /* get bit and pak */
  bit = fb->smul;
  pak = fb->sadd;

  /* get the first dimension */
  strcpy(thedim,"lon");
  strcpy(d1,fb->dname1);

  /* and the second*/
  strcpy(otherdim,"lat");
  strcpy(d2,fb->dname2);

  /* check that the field is in the right format for conversion */
  if(fb->bytes != 'p'){
    fprintf(stderr,"%s: field %s not pixel \n",NAME,fname);
    exit(1);
  }

  /* check that pixel size is correct otherwise navpac will get confused */
  if(bit !=8) {
    fprintf(stderr,"%s: pixel size != 8\n",NAME);
    exit(1);
  }
  /* set pointer navset to point to nav */
  navheader = &nav;
  /* get the element counts */
  snewsize = selmnt = elemcnt(hbuff,HBMAX,'s');
  velmnt = elemcnt(hbuff,HBMAX,'v');

  /* and now the buffers for the old data */
  sptr =  getbuff(selmnt);
  vptr =  getbuff(velmnt);


  /* get the new lat,lon, and total counts */
  longsize = (int)((longhigh - longlow)/dlong + .5) ;
  latsize  = (int)((lathigh  - latlow )/dlat + .5)  ;
  if(longsize <0 ){
    dlong *= -1;
    longsize *= -1;
  }
  if(latsize <0){
    dlat  *= -1;
    latsize *= -1;
  }
  newfsize = latsize*longsize;

  /* check that sizes are reasonable or dumbnav will crash */
  if(longlow<SANELON1 || longlow>SANELON2 || longhigh>SANELON2 || 
      longhigh<SANELON1){
    fprintf(stderr,"%s: longitude values out of range \n",NAME);
    exit(1);
  }
  if(latlow<SANELAT1 || latlow>SANELAT2 || lathigh>SANELAT2 ||
      lathigh<SANELAT1){
    fprintf(stderr,"%s: latitude out of range \n",NAME);
    exit(1);
  }
  /* get sizes of the slice sub buffers */
  bgnsze = fb->ptr;
  lastsze = velmnt - fb->ptr -fb->tsize;
  lastptr = vptr + fb->ptr +fb->tsize;

  /* now allocate buffers for the new static fields (lat,lon) */
  if((lon = (float*)malloc(sizeof(float)*longsize)) == NULL){
    fprintf(stderr,"%s: can't allocate enough memory\n",NAME);
    exit(1);
  }
  if((lat = (float*)malloc(sizeof(float)*latsize)) == NULL){
    fprintf(stderr,"%s: can't allocate enough memory\n",NAME);
    exit(1);
  }

  /* get the buffer for the image field */
  if((vnewptr = (float*)malloc(sizeof(float)*latsize*longsize)) == NULL){
    fprintf(stderr,"%s: can't allocate enough memory\n",NAME);
    exit(1);
  }

  /* copy the header over to the new buffer */
  nullhdr(newbuff,HBMAX,"float");
  copycmt(newbuff,HBMAX,hbuff,HBMAX);
  copypar(newbuff,HBMAX,hbuff,HBMAX);

  /* copy the static fields over, the affected field will be appended later */
  loop = 0;
  while((fb = getfld(hbuff,HBMAX,'s',loop)) != NULL){
    if(strcmp(d1,fb->fname) != 0 && strcmp(d2,fb->fname)!= 0)
      addfld(newbuff,HBMAX,'s',fb->fname,fb->smul,fb->sadd,fb->bytes,fb->dim,
          fb->dname1,fb->dsize1,fb->dname2,fb->dsize2,fb->dname3,
          fb->dsize3,fb->dname4,fb->dsize4);
    else
      snewsize -= fb->tsize;
    loop++;
  }
  /* set up pointers to x,y size fields */
  if(!(fb = seekfld(hbuff,HBMAX,'s',"xsize"))){
     fprintf(stderr,"%s: can't find xsize field \n",NAME);
     exit(1);
  }
  xsizeptr = (sptr+fb->ptr);
  if(!(fb = seekfld(hbuff,HBMAX,'s',"ysize"))){
    fprintf(stderr,"%s: can't find ysize field\n",NAME);
    exit(1);
  }
 
  ysizeptr = (sptr +fb->ptr);
  
  /*  say we were here */
  sprintf(cline,"%s:  %s %s %s %s %s %s %s \n",NAME,argv[1],argv[2] 
      ,argv[3],argv[4],argv[5],argv[6],newname);
  addcline(newbuff,HBMAX,cline);

  /* add the dimensions (lat,lon)  */
  addfld(newbuff,HBMAX,'s',thedim,1.0,0.0,'s',
      1,thedim,longsize);
  addfld(newbuff,HBMAX,'s',otherdim,1.0,0.0,'s',
      1,otherdim,latsize );

  /* move variable fields over */
  loop = 0;
  while(( fb = getfld(hbuff,HBMAX,'v',loop)) != NULL){
    loop++;
    if(strcmp(fname,fb->fname) != 0)
      addfld(newbuff,HBMAX,'v',fb->fname,fb->smul,fb->sadd,fb->bytes,
          fb->dim,fb->dname1,fb->dsize1,fb->dname2,fb->dsize2,
          fb->dname3,fb->dsize3,fb->dname4,fb->dsize4);
    else{
      addfld(newbuff,HBMAX,'v',newname,1.0,0.0,'s',2,"lon",longsize,
          "lat",latsize);
    }
  }

  /*   time to work with the slices */
  while((cnt = getslice(stdin,selmnt,sptr)) != EOF){
  /* pick up xsize,ysize values */
    xsize = xsizeptr[0];
    ysize = ysizeptr[0];
    if((cnt = getslice(stdin,velmnt,vptr)) != EOF){
      if(slice1 == 1){
        /* add index perameters */
        sprintf(cline,"%s0",thedim);
        sprintf(value,"%f",longlow);
        addpar(newbuff,HBMAX,cline,value);
        sprintf(value,"%f",dlong);
        sprintf(cline,"d%s",thedim);
        addpar(newbuff,HBMAX,cline,value);

        /* now for the first dimension */
        sprintf(cline,"%s0",otherdim);
        sprintf(value,"%f",latlow);
        addpar(newbuff,HBMAX,cline,value);
        sprintf(cline,"d%s",otherdim);
        sprintf(value,"%f",dlat);
        addpar(newbuff,HBMAX,cline,value);

        /* set up navigation header */
        navset(&nav,sptr,vptr,&type);

        /* get rid of the header */
        puthdr(stdout,newbuff,HBMAX);

        /* needed perameters added return to conversions */
      } /* end of operations preformed only on first slice */

      /* set up static fields */
      sconv(lat,lon);

      /* get rid of static slice */
      putslice2(stdout,snewsize,sptr,longsize,lon,latsize,lat);

      /* now for the variable slices */
      vconv(vptr,latsize,longsize,vnewptr,&nav);

      /* get rid of the  slice */
      putslice2(stdout,bgnsze,vptr, latsize*longsize,vnewptr,lastsze,lastptr);
      slice1 = 0;

    }
  }                 /* end of for(;;) */
  /* keep sun happy */
  exit(0);
}                 /* of program */


/* sconv -- the easy part -- the new dimensions will be found by 
 * adding dlat or dlong to the low values of lat or long  
 * the modified dimension will the be placed in the buffer (lat or long)
 * and left for putslice2 to  throw out */

sconv(lat,lon)

float *lat,*lon;   /* index fields for the altered image field */
{
  int  loop2;
  float loop;

  loop2 = 0;
  /* insert latitude data into field */
  for(loop = latlow; loop <lathigh; loop += dlat){
    lat[loop2] = loop;
    loop2++;
  }

  /* now for longitude */
  loop2 = 0;
  for(loop = longlow; loop <= longhigh; loop +=dlong){
    lon[loop2] = loop;
    loop2 ++;
  }

  /* that should be all we need */
}


/* vconv - the hard part -- this should convert the variable slice to 
 * float format. the conversion from float format is accomplished by
 * taking the long elements pointed out by dumbnav that the pixels are
 * packed in. the longs are right shifted by (loop%bit)*bit bits on each 
 * pass and mask is set to remove all but the rightmost bit bits. the pixels
 * now in int format are converted into a temperature value dependent on 
 * type and placed in the buffer at newptr. They are now in float format 
 * the old buffer is assumed to have dimensions XSIZE(x)XYSIZE(y) in unpacked 
 * form.  
 */


vconv(vptr,latsize,longsize,newptr,nav)

float *vptr,*newptr  ;               /* buffers for new and old data        */
long latsize,longsize;               /* latitude and longitude counts       */
struct goes *nav;                    /* storage for nav. info               */

{
  int loop,loop2;                     /* generic loop counters              */
  long oldcount;                      /* count of packed data               */
  long *lngptr;                       /* pointer to image fld in old buffer */
  float latloop, longloop;            /* loops on latitude and longitude    */
  int fast = 0;                       /* flag that dumbnav has run once     */
  int x,y;                            /* locations of point in old buffer   */
  int llama;                          /* temp. storage for unpacked pixel   */
  float ttemp1,ttemp2;                /* test temps for ir images           */

  /* locate the image in the variable slice buffer */
  fb = seekfld(hbuff,HBMAX,'v',fname);

  /* set up pointer to image and convert to long */
  lngptr = (long*)(vptr + fb->ptr);

  /* swap bytes if necessary */
  if (FLOATSWAP) floatswap(fb->tsize,(float*)lngptr);

  /* set longloop and latloop to lowest values of lat and lon */
  longloop = longlow;

  /* loop on longitude */
  for(loop = 0; loop <longsize; loop++){

    /* reset latitude */
    latloop = latlow;

    /* now loop on latitude */
    for(loop2 = 0; loop2<latsize; loop2++){
      dumbnav(nav,longloop,latloop,&x,&y,&fast);

      /* check that we can see the point in question */
      if(x<0 || y<0 || y>ysize || x>xsize) newptr[loop*latsize + loop2] = BADR;
      else {

        /* convert from pixel fmt. to int */
        llama = (lngptr[y*(int)(xsize/4) + x/pak]>>(bit*(x%pak))) &255;

        /* convert to temp or brightness as appropriate */
        if(type == 'v') newptr[loop*latsize + loop2] = llama/64.;
        else {
          ttemp1 = 418. - llama;
          ttemp2 = 330. - llama/2.;
          newptr[loop*latsize + loop2] = ttemp1 < ttemp2 ? ttemp1 : ttemp2;
        }
      }  /* end of else ... */

      /* increment latloop */
      latloop += dlat;

    }  /* end of inner for (lat) */

    /* increment longloop */
    longloop += dlong;

  } /* end of outer for (long) */

}     /* finished */



/* putslice2 --  change to putslice that allows mulitple buffers to be
 * treated as one  slice buffer when they are put on stream
 * Putslice2 accepts two buffers of floats, an open file pointer,
 * and the element
 * count, writes the number in 8 byte decimal ascii, then writes the
 * floats using fwrite.  Returns the number of floats written. Abuff is 
 * attached to the end of sbuff so that additions and modifications can be
 * made to a file without endless duplication of massive slice buffers
 */

long putslice2(stream,num,sbuff,anum,abuff,bnum,bbuff)
FILE *stream; /* the data file pointer */
long num,anum,bnum; /* the number of elements to write */
float sbuff[],abuff[],bbuff[];  /* the buffer of floats */
{
  long nwritten;  /* number of elements written */
  char elct[9];
  if(num+anum+bnum < 0 || num +anum+bnum> MAXSLICE) {
    fprintf(stderr,"putslice2: weird element count = %ld\n",num+anum+bnum);
    exit(1);
  }
  sprintf(elct,"%8ld",(num+anum+bnum));
  fputs(elct,stream);
  if ((num+anum+bnum) == 0) return(0);

  /* swap float byte order if necessary */
  if (FLOATSWAP) {
    floatswap(num,sbuff);
    floatswap(anum,abuff);
    floatswap(bnum,bbuff);
  }
  nwritten = fwrite(sbuff,sizeof(float),num,stream);
  nwritten += fwrite(abuff,sizeof(float),anum,stream);
  nwritten += fwrite(bbuff,sizeof(float),bnum,stream);
  if(nwritten != (num+anum+bnum)) {
    fprintf(stderr,"putslice2: botched fwrite\n");
    exit(1);
  }
  return(nwritten);
}
/* navset - use the candis perameter section of the header to get the
 * info. needed later by dumbnav 
 */
navset(navheader,sbuff,vbuff,type)

struct goes *navheader;      /* pointer to navigation info */
float *sbuff;                /* pointer to the static fields in buffer */
float *vbuff;                /* pointer to variable field buffer */
char *type;                  /* type of image to be set from header */
{
  char value[LINE];            /* this is where the values will be 
                                  * stored for a while as strings */
  int ok;                      /* flag for checking if seekpar was succesful*/
  struct field *fb;            /* usual header info pointer */

  /* get navtype */
  if (seekpar(hbuff,HBMAX,"navtype",navtype) != OK) {
    fprintf(stderr,"mcmap: no navtype parameter\n");
    exit(1);
  }
  strcpy(navheader->navtype,navtype);

  /* get type out of parameter bandmap -- works for both GOES and GVAR */
  if(( ok = seekpar(hbuff,HBMAX,"bandmap",value)) == OK){
    ok = atoi(value);
    if(ok == 1) *type = 'v';
    else *type = 'i';
  }
  else {
    fprintf(stderr,"mcmap: image type not found\n" );
    exit(1);
  }

  /* get parts out of paramenters section of header */
  if (strcmp(navtype,"GOES") == 0) {
    if((ok = seekpar(hbuff,HBMAX,"epochd",value)) ==OK)
      navheader->epochd = (long)atoi(value);
    if((ok = seekpar(hbuff,HBMAX,"epocht",value)) == OK)
      navheader->epocht = atoi(value);
    if((ok = seekpar(hbuff,HBMAX,"asnode",value)) == OK)
      navheader->asnode = atof(value);
    if((ok = seekpar(hbuff,HBMAX,"perigee",value)) == OK)
      navheader->perigee = atof(value);
    if((ok =seekpar(hbuff,HBMAX,"meana",value)) == OK)
      navheader->meana   = atof(value);
    if((ok =seekpar(hbuff,HBMAX,"orbinc",value)) == OK)
      navheader->orbinc = atof(value);
    if((ok = seekpar(hbuff,HBMAX,"eccen",value)) == OK)
      navheader->eccen = atof(value);
    if((ok = seekpar(hbuff,HBMAX,"semima",value)) == OK)
      navheader->semima = atof(value);
    if((ok = seekpar(hbuff,HBMAX,"rascen",value)) == OK)
      navheader->rascen = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"declin",value)) == OK)
      navheader->declin = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"pitch",value)) == OK)
      navheader->pitch = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"yaw",value)) == OK)
      navheader->yaw = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"roll",value)) == OK)
      navheader->roll = atof(value);
    if((ok = seekpar(hbuff,HBMAX,"lintot",value)) == OK)
      navheader->lintot = atoi(value);
    if((ok = seekpar(hbuff,HBMAX,"deglin",value)) == OK)
      navheader->deglin = atof(value);
    if((ok = seekpar(hbuff,HBMAX,"eletot",value)) == OK)
      navheader->eletot = atoi(value);
    if((ok = seekpar(hbuff,HBMAX,"degele",value)) == OK)
      navheader->degele = atof(value);
    if((ok = seekpar(hbuff,HBMAX,"piclin",value)) == OK)
      navheader->piclin = atol(value);
    if((ok = seekpar(hbuff,HBMAX,"spinp",value)) == OK)
      navheader->spinp = atof(value);
  }
  else if (strcmp(navtype,"GVAR") == 0) {
    if(( ok = seekpar(hbuff,HBMAX,"pitch",value)) == OK)
      navheader->pitch = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"yaw",value)) == OK)
      navheader->yaw = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"roll",value)) == OK)
      navheader->roll = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"lam",value)) == OK)
      navheader->lam = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"dr",value)) == OK)
      navheader->dr = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"phi",value)) == OK)
      navheader->phi = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"psi",value)) == OK)
      navheader->psi = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"rma",value)) == OK)
      navheader->rma = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"pma",value)) == OK)
      navheader->pma = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"elevbnds",value)) == OK)
      navheader->elevbnds = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"elevdln",value)) == OK)
      navheader->elevdln = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"scanbnds",value)) == OK)
      navheader->scanbnds = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"scanpix",value)) == OK)
      navheader->scanpix = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"lineoffset",value)) == OK)
      navheader->lineoffset = atof(value);
    if(( ok = seekpar(hbuff,HBMAX,"scanoffset",value)) == OK)
      navheader->scanoffset = atof(value);
  }
  else {
    fprintf(stderr,"mcmap: %s is unknown nav type\n");
    exit(1);
  }

  /* now for the things that have to be recovered from static fields */

  if((fb = seekfld(hbuff,HBMAX,'s',"xcoor")) != NULL)
    navheader->xcoor = sbuff[fb->ptr];
  if((fb = seekfld(hbuff,HBMAX,'s',"ycoor")) != NULL)
    navheader->ycoor = sbuff[fb->ptr];
  if((fb = seekfld(hbuff,HBMAX,'s',"xsize")) != NULL)
    navheader->xsize = sbuff[fb->ptr];
  if((fb = seekfld(hbuff,HBMAX,'s',"ysize")) != NULL)
    navheader->ysize = sbuff[fb->ptr];
  if((fb = seekfld(hbuff,HBMAX,'s',"xres")) != NULL)
    navheader->xres = sbuff[fb->ptr];
  if((fb = seekfld(hbuff,HBMAX,'s',"yres")) != NULL)
    navheader->yres = sbuff[fb->ptr];

  /* now for things that must be taken from variable fields */

  if((fb = seekfld(hbuff,HBMAX,'v',"day")) != NULL )
    navheader->day = vbuff[fb->ptr];
  if((fb = seekfld(hbuff,HBMAX,'v',"year")) != NULL)
    navheader->year = vbuff[fb->ptr ];
  if((fb = seekfld(hbuff,HBMAX,'v',"hour")) != NULL)
    navheader->hour = vbuff[fb->ptr];
  if((fb = seekfld(hbuff,HBMAX,'v',"minute")) != NULL)
    navheader->minute = vbuff[fb->ptr];
  if((fb = seekfld(hbuff,HBMAX,'v',"second")) != NULL)
    navheader->second = vbuff[fb->ptr];

  /* that should be all we need */

} /* go home */
