/* procimage -- Remap image and add lines and text. */

#include <stdio.h>
#include "mcidas.h"

/* external stuff */
extern int visible,watervapor,infrared;
extern float bcut,tcut,wcut;
extern float tdither,bdither,wdither;

/* internal stuff */
static char *buff = NULL;          /* pointer to storage for image */
static char cline[200];            /* for label */
int itemp();                       /* routine that gets pix value from temp */
int ibrite();                      /* routine to get pix value from bright */
float temp();                      /* get temperature from pix val */
float brite();                     /* get brightness from pix val */
static char *trim();               /* trim off leading pathname */
static int ncolors;                /* number of colors available */
static int cvals[256];             /* color pixel values */
static int cval;                   /* temporary -- cvals index */

procimage(index,image,gptr,filename,image_depth)
int index;                          /* position of image in sequence */
unsigned char *image;               /* pointer to image */
struct goes *gptr;                  /* pointer to info structure */
char *filename;                     /* name of input file for this image */
int image_depth;                    /* requested depth of output image */
{
  int position,modpos,monopos,monobuff,monocut,ix,iy,nx,ny,value,c1,c2;
  int dth1,dth2,dth3,dth4,xtest,ytest,dtest;
  int i;

/* trim path off filename */
  filename = trim(filename);

/* check for too many images */
  if (index >= MAXIMAGES - 1) {
    fprintf(stderr,"procimage: too many images\n");
    exit(1);
  }

/* get image size */
  nx = gptr->xsize;
  ny = gptr->ysize;

/* get image type */
  if (strcmp(gptr->navtype,"GOES") == 0) {
    visible = gptr->bandmap == 1;
    watervapor = gptr->bandmap == 200;
    infrared = gptr->bandmap == 80;
  }
  else if (strcmp(gptr->navtype,"GVAR") == 0) {
    visible = gptr->bandmap == 1;
    watervapor = gptr->bandmap == 4;
    infrared = gptr->bandmap == 8;
  }
  else {
    fprintf(stderr,"procimage: what kind of image is %s?\n",gptr->navtype);
    exit(1);
  }
  if (visible + watervapor + infrared == 0) {
    fprintf(stderr,"procimage: %d is unknown bandmap\n",gptr->bandmap);
    exit(1);
  }

/* monochrome images */
  if (image_depth == 1) {

/* allocate space if needed */
    if (buff == NULL) {
      if ((buff = (char*)malloc(nx*ny/8)) == NULL) {
        fprintf(stderr,"procimage: can't get enough memory\n");
        exit(1);
      }
    }

/* decide on monocut depending on type of image and set up dithering */
    if (visible) {
      monocut = ibrite(bcut);
      dth1 = ibrite(bcut + bdither) - ibrite(bcut);
      dth2 = ibrite(bcut - bdither) - ibrite(bcut);
      dth3 = ibrite(bcut + bdither/3.) - ibrite(bcut);
      dth4 = ibrite(bcut - bdither/3.) - ibrite(bcut);
    }
    else if (watervapor) {
      monocut = itemp(wcut);
      dth1 = itemp(wcut + wdither) - itemp(wcut);
      dth2 = itemp(wcut - wdither) - itemp(wcut);
      dth3 = itemp(wcut + wdither/3.) - itemp(wcut);
      dth4 = itemp(wcut - wdither/3.) - itemp(wcut);
    }
    else if (infrared) {
      monocut = itemp(tcut);
      dth1 = itemp(tcut + tdither) - itemp(tcut);
      dth2 = itemp(tcut - tdither) - itemp(tcut);
      dth3 = itemp(tcut + tdither/3.) - itemp(tcut);
      dth4 = itemp(tcut - tdither/3.) - itemp(tcut);
    }
    else {
      fprintf(stderr,"procimage: band %d not known\n",gptr->bandmap);
      exit(1);
    }

/* map pixels from image to mono buffer, doing dithering */
    position = 0;
    modpos = 8;
    monopos = 0;
    monobuff = 0;
    for (iy = 0; iy < ny; iy++) {
      ytest = iy % 2;
      for (ix = 0; ix < nx; ix++) {
        xtest = ix % 2;
        value = image[position++];

/* add dithering value depending on location in 2x2 set of pixels */
        dtest = xtest + 2*ytest;
        switch (dtest) {
          case 0 :
            value += dth1;
            break;
          case 1 :
            value += dth2;
            break;
          case 2 :
            value += dth3;
            break;
          case 3 :
            value += dth4;
            break;
        }

/* do black-white decision for each pixel */
        if (visible) {
          value = value < monocut ? 1 : 0;
        }
        else {
          value = value > monocut ? 1 : 0;
        }

/* pack resulting bit into bitmap */
        modpos--;
        monobuff = monobuff | (value << modpos);
        if (modpos == 0 || ix == nx - 1) {
          modpos = 8;
          buff[monopos++] = monobuff;
          monobuff = 0;
        }
      }
    }
  }

/* color option */
  else if (image_depth == 8) {

/* get color map from hardware (2 <= ncolors <= 256) */
    ncolors = getcolors(cvals);

/* allocate space */
    if (buff == NULL) {
      if ((buff = (char*)malloc(nx*ny)) == NULL) {
        fprintf(stderr,"procimage: can't get enough memory\n");
        exit(1);
      }
    }

/* map pixels from image to buffer */
    if (visible) {
      position = 0;
      for (iy = 0; iy < ny; iy++) {
        for (ix = 0; ix < nx; ix++) {
          cval = ncolors*(-bcut + bdither + brite(image[position]))/(2.*bdither);
          if (cval < 0) cval = 0;
          if (cval > ncolors - 1) cval = ncolors - 1;
          buff[position++] = cvals[cval];
        }
      }
    }
    else if (infrared) {
      position = 0;
      for (iy = 0; iy < ny; iy++) {
        for (ix = 0; ix < nx; ix++) {
          cval = ncolors*(-tcut + tdither + temp(image[position]))/(2.*tdither);
          if (cval < 0) cval = 0;
          if (cval > ncolors - 1) cval = ncolors - 1;
          buff[position++] = cvals[cval];
        }
      }
    }
    else if (watervapor) {
      position = 0;
      for (iy = 0; iy < ny; iy++) {
        for (ix = 0; ix < nx; ix++) {
          cval = ncolors*(-wcut + wdither + temp(image[position]))/(2.*wdither);
          if (cval < 0) cval = 0;
          if (cval > ncolors - 1) cval = ncolors - 1;
          buff[position++] = cvals[cval];
        }
      }
    }
  }

/* bad pixel size */
  else {
    fprintf(stderr,"procimage: pixel size %d not supported\n",image_depth);
    exit(1);
  }

/* send image to hardware */
  sendimage(index,nx,ny,image_depth,buff);

/* draw longitude and latitude lines on hardware */
  drawlatlon(index,gptr);

/* draw file name and range values on image */
  if (visible) {
    sprintf(cline,"%s -- Brightness: %.2f to %.2f",filename,bcut - bdither,
         bcut + bdither);
  }
  else if (infrared) {
    sprintf(cline,"%s -- Temperature: %.0f to %.0f",filename,tcut - tdither,
         tcut + tdither);
  }
  else if (watervapor) {
    sprintf(cline,"%s -- Temperature: %.0f to %.0f",filename,wcut - wdither,
         wcut + wdither);
  }
  drawtext(index,10,25,cline);

/* finish things up */
  finishimage(index,nx,ny);
}

/* itemp -- Compute pixel value for specified temperature.
 * This info obtained from Bob Green, Unidata.
 */
int itemp(tval)
float tval;
{
  int c1,c2;
  c1 = 2.*(330. - tval);
  c2 = 418. - tval;
  return(c1 < c2 ? c1 : c2);
}

float temp(pixval)
int pixval;
{
  float t1,t2;
  t1 = 418 - pixval;
  t2 = 330. - pixval/2.;
  return(t1 < t2 ? t1 : t2);
}

/* ibrite -- Compute pixel value for specified brightness (0-1) */
int ibrite(bval)
float bval;
{
  return((int)(64.*bval));
}

float brite(pixval)
int pixval;
{
  return(pixval/64.);
}

/* trim -- trim off leading pathname */
static char *trim(name)
char *name;
{
  char *i;
  i = name + strlen(name);
  while (i > name && *(i - 1) != '/') i--;
  return(i);
}
