/* putimage.c -- This routine creates a file and writes an image
 * to it.  The file name is derived from date, time, satellite and
 * type of image.
 * This version writes a Candis file, with the image in pixel format.
 */

#include <stdio.h>
#include <math.h>
#include <strings.h>
#include "cdfhdr.h"
#include "mcidas.h"

static char hbuff[HBMAX][LINE];
static char cline[LINE];
static long nsbuff = 0,nvbuff = 0;
static long maxnsbuff = 0,maxnvbuff = 0;
static float *sbuff = NULL,*vbuff = NULL;
static float *year,*day,*hour,*minute,*second;
static float *xcoor,*ycoor,*zcoor;
static float *xsize,*ysize;
static float *xres,*yres,*zres;
static float *x,*y;
static char *image;
static FILE *outfile;
static char filename[100];

putimage(header,imagebuffer)
struct goes *header;                   /* pointer to image header info */
char *imagebuffer;                     /* pointer to the image */
{
  int ix,iy,nx,ny;

/* create file name */
  sprintf(filename,"i-%d.%x-%d.%03d.%02d.%02d-%d.%d.%d.%d",header->satid,
       header->bandmap,header->year,header->day,header->hour,header->minute,
       header->xcoor,header->xres,header->ycoor,header->yres);

/* open file for write */
  if ((outfile = fopen(filename,"w")) == NULL) {
    fprintf(stderr,"putimage: can't open file %s\n",filename);
    exit(1);
  }

/* create a Candis header */
  nullhdr(hbuff,HBMAX,"float");
  sprintf(cline,"mcselect: %s\n",filename);
  addcline(hbuff,HBMAX,cline);

/* add parameters */
  sprintf(cline,"%d",header->satid);
  addpar(hbuff,HBMAX,"satid",cline);
  sprintf(cline,"%d",header->nbyte);
  addpar(hbuff,HBMAX,"nbyte",cline);
  sprintf(cline,"%x",header->bandmap);
  addpar(hbuff,HBMAX,"bandmap",cline);
  addpar(hbuff,HBMAX,"navtype",header->navtype);
  addpar(hbuff,HBMAX,"source",header->source);
  addpar(hbuff,HBMAX,"calib",header->calib);
  if (strcmp(header->navtype,"GOES") == 0) {
    sprintf(cline,"%d",header->epochd);
    addpar(hbuff,HBMAX,"epochd",cline);
    sprintf(cline,"%d",header->epocht);
    addpar(hbuff,HBMAX,"epocht",cline);
    sprintf(cline,"%g",header->semima);
    addpar(hbuff,HBMAX,"semima",cline);
    sprintf(cline,"%g",header->eccen);
    addpar(hbuff,HBMAX,"eccen",cline);
    sprintf(cline,"%g",header->orbinc);
    addpar(hbuff,HBMAX,"orbinc",cline);
    sprintf(cline,"%g",header->meana);
    addpar(hbuff,HBMAX,"meana",cline);
    sprintf(cline,"%g",header->perigee);
    addpar(hbuff,HBMAX,"perigee",cline);
    sprintf(cline,"%g",header->asnode);
    addpar(hbuff,HBMAX,"asnode",cline);
    sprintf(cline,"%g",header->declin);
    addpar(hbuff,HBMAX,"declin",cline);
    sprintf(cline,"%g",header->rascen);
    addpar(hbuff,HBMAX,"rascen",cline);
    sprintf(cline,"%d",header->piclin);
    addpar(hbuff,HBMAX,"piclin",cline);
    sprintf(cline,"%g",header->spinp);
    addpar(hbuff,HBMAX,"spinp",cline);
    sprintf(cline,"%g",header->deglin);
    addpar(hbuff,HBMAX,"deglin",cline);
    sprintf(cline,"%d",header->lintot);
    addpar(hbuff,HBMAX,"lintot",cline);
    sprintf(cline,"%d",header->scanners);
    addpar(hbuff,HBMAX,"scanners",cline);
    sprintf(cline,"%g",header->degele);
    addpar(hbuff,HBMAX,"degele",cline);
    sprintf(cline,"%d",header->eletot);
    addpar(hbuff,HBMAX,"eletot",cline);
    sprintf(cline,"%g",header->pitch);
    addpar(hbuff,HBMAX,"pitch",cline);
    sprintf(cline,"%g",header->yaw);
    addpar(hbuff,HBMAX,"yaw",cline);
    sprintf(cline,"%g",header->roll);
    addpar(hbuff,HBMAX,"roll",cline);
    sprintf(cline,"%d",header->iajust);
    addpar(hbuff,HBMAX,"iajust",cline);
    sprintf(cline,"%g",header->iajtim);
    addpar(hbuff,HBMAX,"iajtim",cline);
    sprintf(cline,"%g",header->iseang);
    addpar(hbuff,HBMAX,"iseang",cline);
    sprintf(cline,"%d",header->scanline1);
    addpar(hbuff,HBMAX,"scanline1",cline);
    sprintf(cline,"%g",header->timesl1);
    addpar(hbuff,HBMAX,"timesl1",cline);
    sprintf(cline,"%d",header->betacount1);
    addpar(hbuff,HBMAX,"betacount1",cline);
    sprintf(cline,"%d",header->scanline2);
    addpar(hbuff,HBMAX,"scanline2",cline);
    sprintf(cline,"%g",header->timesl2);
    addpar(hbuff,HBMAX,"timesl2",cline);
    sprintf(cline,"%d",header->betacount2);
    addpar(hbuff,HBMAX,"betacount2",cline);
    sprintf(cline,"%g",header->gamma);
    addpar(hbuff,HBMAX,"gamma",cline);
    sprintf(cline,"%g",header->gammadot);
    addpar(hbuff,HBMAX,"gammadot",cline);
  }
  else if (strcmp(header->navtype,"GVAR") == 0) {
    sprintf(cline,"%g",header->pitch);
    addpar(hbuff,HBMAX,"pitch",cline);
    sprintf(cline,"%g",header->yaw);
    addpar(hbuff,HBMAX,"yaw",cline);
    sprintf(cline,"%g",header->roll);
    addpar(hbuff,HBMAX,"roll",cline);
    sprintf(cline,"%g",header->lam);
    addpar(hbuff,HBMAX,"lam",cline);
    sprintf(cline,"%g",header->dr);
    addpar(hbuff,HBMAX,"dr",cline);
    sprintf(cline,"%g",header->phi);
    addpar(hbuff,HBMAX,"phi",cline);
    sprintf(cline,"%g",header->psi);
    addpar(hbuff,HBMAX,"psi",cline);
    sprintf(cline,"%g",header->rma);
    addpar(hbuff,HBMAX,"rma",cline);
    sprintf(cline,"%g",header->pma);
    addpar(hbuff,HBMAX,"pma",cline);
    sprintf(cline,"%g",header->elevbnds);
    addpar(hbuff,HBMAX,"elevbnds",cline);
    sprintf(cline,"%g",header->elevdln);
    addpar(hbuff,HBMAX,"elevdln",cline);
    sprintf(cline,"%g",header->scanbnds);
    addpar(hbuff,HBMAX,"scanbnds",cline);
    sprintf(cline,"%g",header->scanpix);
    addpar(hbuff,HBMAX,"scanpix",cline);
    sprintf(cline,"%g",header->lineoffset);
    addpar(hbuff,HBMAX,"lineoffset",cline);
    sprintf(cline,"%g",header->scanoffset);
    addpar(hbuff,HBMAX,"scanoffset",cline);
  }
  else {
    fprintf(stderr,"putimage: %s is unrecognized nav type\n",header->navtype);
    exit(1);
  }

/* add index fields and other static fields */
  addfld(hbuff,HBMAX,'s',"xcoor",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'s',"ycoor",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'s',"zcoor",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'s',"xsize",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'s',"ysize",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'s',"xres",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'s',"yres",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'s',"zres",1.,0.,'s',0);
  nx = header->xsize/32;
  if (nx*32 < header->xsize) nx++;
  nx *= 8;
  ny = header->ysize;
  addfld(hbuff,HBMAX,'s',"y",1.,0.,'s',1,"y",ny);
  addfld(hbuff,HBMAX,'s',"x",1.,0.,'s',1,"x",nx);

/* add variable fields */
  addfld(hbuff,HBMAX,'v',"year",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"day",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"hour",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"minute",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"second",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"image",8.,4.,'p',2,"y",ny,"x",nx);

/* reallocate slice buffers if needed */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  if (nsbuff > maxnsbuff) {
    maxnsbuff = nsbuff;
    sbuff = getbuff(nsbuff);
  }
  nvbuff = elemcnt(hbuff,HBMAX,'v');
  if (nvbuff > maxnvbuff) {
    maxnvbuff = nvbuff;
    vbuff = getbuff(nvbuff);
  }

/* allocate pointers */
  xcoor = getptr(hbuff,HBMAX,sbuff,'s','d',"xcoor");
  ycoor = getptr(hbuff,HBMAX,sbuff,'s','d',"ycoor");
  zcoor = getptr(hbuff,HBMAX,sbuff,'s','d',"zcoor");
  xsize = getptr(hbuff,HBMAX,sbuff,'s','d',"xsize");
  ysize = getptr(hbuff,HBMAX,sbuff,'s','d',"ysize");
  xres = getptr(hbuff,HBMAX,sbuff,'s','d',"xres");
  yres = getptr(hbuff,HBMAX,sbuff,'s','d',"yres");
  zres = getptr(hbuff,HBMAX,sbuff,'s','d',"zres");
  y = getptr(hbuff,HBMAX,sbuff,'s','d',"y");
  x = getptr(hbuff,HBMAX,sbuff,'s','d',"x");
  year = getptr(hbuff,HBMAX,vbuff,'v','d',"year");
  day = getptr(hbuff,HBMAX,vbuff,'v','d',"day");
  hour = getptr(hbuff,HBMAX,vbuff,'v','d',"hour");
  minute = getptr(hbuff,HBMAX,vbuff,'v','d',"minute");
  second = getptr(hbuff,HBMAX,vbuff,'v','d',"second");
  image = (char*)getptr(hbuff,HBMAX,vbuff,'v','d',"image");

/* fill index fields */
  for (ix = 0; ix < nx; ix++) x[ix] = 4*ix;
  for (iy = 0; iy < ny; iy++) y[iy] = iy;

/* transfer scalar data */
  *xcoor = header->xcoor;
  *ycoor = header->ycoor;
  *zcoor = header->zcoor;
  *xsize = 4*nx;
  *ysize = header->ysize;
  *xres = header->xres;
  *yres = header->yres;
  *zres = header->zres;
  *year = header->year;
  *day = header->day;
  *hour = header->hour;
  *minute = header->minute;
  *second = header->second;

/* transfer image -- pad with zeros if xsize not even multiple of 4 */
  for (ix = 0; ix < nx*ny*4; ix++) image[ix] = 0;
  for (ix = 0; ix < header->xsize; ix++) {
    for (iy = 0; iy < header->ysize; iy++) {
      image[ix + iy*4*nx] = imagebuffer[ix + iy*header->xsize];
    }
  }

/* undo the float swap that is done in putslice for the image only */
  if (FLOATSWAP) floatswap(nx*ny,(float*)image);

/* write out the data */
  puthdr(outfile,hbuff,HBMAX);
  putslice(outfile,nsbuff,sbuff);
  putslice(outfile,nvbuff,vbuff);

/* close the file */
  fprintf(stderr,"putimage: %s written\n",filename);
  fclose(outfile);
}
