/* llpzap                                                           */
/* this program converts lightning data into cdf format keeping     */
/* time (converted to kiloseconds), latitude and longitude and the  */
/* polarity of the bolt                                             */
#include <stdio.h>
#include "cdfhdr.h"

#define MAXLINE 200   /* assumed max length of a line of lightning data */
#define place(a)  addfld(hbuffer,HBMAX,'v',a,10.0,10.0,'s',0);

char hbuffer[HBMAX][LINE];  /* header buffer */
float *sbuffout;            /* slice buffer */
struct tstruct {
  int year,month,date,hour,min,sec;
};                          /* structure containing date info */
char dat[MAXLINE];          /* contains line of data */
float ztime();
long julian_date();

main(argc,argv)
int argc;
char *argv[];
{
  int len,loop;                     /* length of line of data */
  int fhour,fmin;                   /* from the data line */
  long nel;                         /* number of elements to write */
  float fsec,la,lo,po;              /* stuff taken from the data */
  float *hr,*ksec,*lat,*lon,*pol;   /* pointers to the location of data */
  float *dt,*yr,*mn,bad;
  struct tstruct gmt,gmtp;          /* storage for time/date info */
  float zdiff;                      /* difference between GMT and time zone */

/* check the usage */
  if (argc != 1) {
    fprintf(stderr,"Usage: llpzap  < in_file) > out_file\n");
    exit(1);
  }

/* make output header */
  nullhdr(hbuffer,HBMAX,"float");

/* add parameters */
  addpar(hbuffer,HBMAX,"bad","10000");
  addpar(hbuffer,HBMAX,"badlim","9999");
  bad = 10000.;

/* add variable fields */
  place("hour");
  place("date");
  place("month");
  place("year");
  place("ksec");
  place("lat");
  place("long");
  place("polarity");
  nel = elemcnt(hbuffer,HBMAX,'v');

/* set aside memory for slices */
  sbuffout=getbuff(nel);

/* set pointers */
  hr=getptr(hbuffer,HBMAX,sbuffout,'v','d',"hour");
  dt=getptr(hbuffer,HBMAX,sbuffout,'v','d',"date");
  yr=getptr(hbuffer,HBMAX,sbuffout,'v','d',"year");
  mn=getptr(hbuffer,HBMAX,sbuffout,'v','d',"month");
  ksec=getptr(hbuffer,HBMAX,sbuffout,'v','d',"ksec");
  lat=getptr(hbuffer,HBMAX,sbuffout,'v','d',"lat");
  lon=getptr(hbuffer,HBMAX,sbuffout,'v','d',"long");
  pol=getptr(hbuffer,HBMAX,sbuffout,'v','d',"polarity");

/* add comment line */
  sprintf(dat,"llpzap:\n");
  addcline(hbuffer,HBMAX,dat);

/* get GMT time of date stamp */
  len=getline(dat,MAXLINE);
  for (loop=0;loop<28;loop++) dat[loop]=dat[loop+3];
  dat[28]='\n';
  dat[29]='\0';
  zdiff = ztime(dat,&gmtp);

/* output header buffer */
  puthdr(stdout,hbuffer,HBMAX);

/* send nothing for static slice */
  putslice(stdout,0,sbuffout);

/* cycle through data */
  while ((len=getline(dat,MAXLINE)) != EOF) {

/* if the line was bad */
    if (len == 0) continue;

/* if a comment then get new date stamp */
    else if (strncmp(dat,"/* ",3) == 0) {
      for (loop=0;loop<28;loop++) dat[loop]=dat[loop+3];
      dat[28]='\n';
      dat[29]='\0';
      zdiff = ztime(dat,&gmtp);
      continue;
    }

/* if junk, skip over */
    else if ((dat[2]!=dat[5])&&(isdigit(dat[1])==0)&&(isdigit(dat[9])==0)) {
      continue;
    }

/* assume the remainder is real data */
    else {
      sscanf(dat,"%d:%d:%f %f %f %f",&fhour,&fmin,&fsec,&la,&lo,&po);
      hr[0] = fhour;
      ksec[0] = (fhour*3600.0+fmin*60.0+fsec)/1000.0;

/* fix date */
      fixdate(ksec[0]/3.6,zdiff,&gmtp,&gmt);

/* extract information */
      lat[0] = la;
      lon[0] = lo;
      pol[0] = po;
      dt[0] = gmt.date;
      yr[0] = gmt.year;
      mn[0] = gmt.month;

/* output slice and get ready for next one */
      putslice(stdout,nel,sbuffout);
    }
  }

/* send 24 lightning bolts to north pole 1/hour */
  for (loop=0;loop<24;loop++) {
    hr[0] = loop;
    ksec[0] = (loop*3600.0+1800.0)/1000.0;

/* fix date */
    fixdate(ksec[0]/3.6,zdiff,&gmtp,&gmt);

/* put in bogus info so that there will always be some data */
    lat[0] = 90.0;
    lon[0] = 0.0;
    pol[0] = bad;
    dt[0] = gmt.date;
    yr[0] = gmt.year;
    mn[0] = gmt.month;

/* output slice */
    putslice(stdout,nel,sbuffout);
  }
}

/* get a line of data from the light. file */
getline(s,lim)
char s[];
int lim;
{
  int c,i;

  for (i=0;i<lim-1 && (c=fgetc(stdin)) != EOF && c != '\n';++i) {
    s[i]=c;
    if ((c < '\040')||(c > '\176'))
      return(0);
  }
  if (c == '\n') {
    s[i]=c;
    ++i;
  }
  else if (c == EOF) return(EOF);
  s[i]='\0';
  return(i);
}

/* compute GMT time from time stamp */
float ztime(text,gmt)
char *text;
struct tstruct gmt;
{
  int fields;
  char b[6][LINE];
  char zone[LINE],time[LINE],month[LINE];
  int nmonth;
  float fzone;
  int year,day,hour,min,sec;
  double jday;
  long njday;

/* check line for length */
  if (strlen(text) >= LINE) {
    fprintf(stderr,"ztime: text line to long\n");
    exit(1);
  }

/* parse time stamp */
  fields = sscanf(text,"%s %s %s %s %s %s",b[0],b[1],b[2],b[3],b[4],b[5]);
  if (fields < 5 || fields > 6) {
    fprintf(stderr,"ztime: can't understand text %s\n",text);
    exit(1);
  }

/* check which is last field -- year or time zone */
  if (isdigit(b[fields - 1][0])) {
    year = atoi(b[--fields]);
    strcpy(zone,b[--fields]);
  }
  else {
    strcpy(zone,b[--fields]);
    year = atoi(b[--fields]);
  }
  strcpy(time,b[--fields]);
  day = atoi(b[--fields]);
  strcpy(month,b[--fields]);

/* determine month */
  if (strcmp(month,"Jan") == 0) nmonth = 1;
  else if (strcmp(month,"Feb") == 0) nmonth = 2;
  else if (strcmp(month,"Mar") == 0) nmonth = 3;
  else if (strcmp(month,"Apr") == 0) nmonth = 4;
  else if (strcmp(month,"May") == 0) nmonth = 5;
  else if (strcmp(month,"Jun") == 0) nmonth = 6;
  else if (strcmp(month,"Jul") == 0) nmonth = 7;
  else if (strcmp(month,"Aug") == 0) nmonth = 8;
  else if (strcmp(month,"Sep") == 0) nmonth = 9;
  else if (strcmp(month,"Oct") == 0) nmonth = 10;
  else if (strcmp(month,"Nov") == 0) nmonth = 11;
  else if (strcmp(month,"Dec") == 0) nmonth = 12;
  else {
    fprintf(stderr,"ztime: month %s not recognized\n",month);
    exit(1);
  }

/* determine time zone (add your favorite zone if desired) */
  if (strcmp(zone,"PST") == 0) fzone = 8.;
  else if (strcmp(zone,"PDT") == 0) fzone = 7.;
  else if (strcmp(zone,"MST") == 0) fzone = 7.;
  else if (strcmp(zone,"MDT") == 0) fzone = 6.;
  else if (strcmp(zone,"CST") == 0) fzone = 6.;
  else if (strcmp(zone,"CDT") == 0) fzone = 5.;
  else if (strcmp(zone,"EST") == 0) fzone = 5.;
  else if (strcmp(zone,"EDT") == 0) fzone = 4.;
  else if (strcmp(zone,"GMT") == 0) fzone = 0.;
  else {
    fprintf(stderr,"ztime: time zone %s not recognized\n",zone);
    exit(1);
  }

/* extract time */
  if (sscanf(time,"%d:%d:%d",&hour,&min,&sec) != 3) {
    fprintf(stderr,"ztime: don't understand time %s\n",time);
    exit(1);
  }

/* get Julian day */
  jday = (double)julian_date(year,nmonth,day);
  jday += ((((sec/60.) + min)/60. + hour + fzone)/24.);

/* get GMT Gregorian date */
  njday = jday;
  jday -= njday;
  gregorian_date(njday,&gmt.year,&gmt.month,&gmt.date);
  gmt.hour = 24.*jday;
  jday -= gmt.hour/24.;
  gmt.min = 1440.*jday;
  jday -= gmt.min/1440.;
  gmt.sec = 86400.*jday;
  return(fzone);
}

/* fixdate -- if GMT hour of lightning is less than timestamp time
 * in GMT (assumed to occur at local midnight), then it is the
 * next day in GMT -- only works in western hemisphere
 */
fixdate(hour,zdiff,gmtp,gmt)
float hour;                     /* time of lightning */
float zdiff;                    /* GMT time of local midnight */
struct tstruct *gmtp;           /* uncorrected time/date */
struct tstruct *gmt;            /* corrected time/date */
{
  long jday;
  *gmt = *gmtp;
  if (zdiff > 0 && hour > zdiff) return;
  else {
    jday = julian_date(gmt->year,gmt->month,gmt->date) + 1;
    gregorian_date(jday,&(gmt->year),&(gmt->month),&(gmt->date));
  }
}

/* this routine unfortunately doesn't exist on all system libraries */
isdigit(c)
char c;
{
  if (c >= '0' && c <= '9') return(1);
  else return(0);
}
