/* nwssa1 -- Main routine for surface weather data interpreter 
 * -- this routine calls nwssa2
 */

#include <stdio.h>
#include "cdfhdr.h"
#define SAMAX 500
#define LMAX 150

struct satable {        /* station data structure */
  float num;            /* station number */
  char station[4];      /* station name */
  float lat,lon;        /* latitude and longitude */
  float elev;           /* elevation in km */
} salist[SAMAX];
int listmax;            /* actual number of stations in list */

char hbuff[HBMAX][LINE];  /* header buffer */
long nsbuff,nvbuff;     /* cdf buffer lengths */
float *sbuff,*vbuff;    /* buffer pointers */
float *cdfstn,*cdfdate,*cdfmonth,*cdfyear,*cdfhour,*cdflat,*cdflong,
     *cdfelev,*cdftemp,*cdfdewpt,*cdfu,*cdfv,*cdfaltset,*cdfceiling,
     *cdfwx,*cdfmixrat; /* variable field pointers */
float bad,badlim;       /* bad data parameters */

main(argc,argv)
int argc;
char *argv[];
{
  char cline[LMAX];     /* buffer for stuff */
  char cbuff[LMAX];     /* another buffer */
  FILE *safile;         /* file containing station list */
  float latdeg,latmin,longdeg,longmin;  /* temporaries for lat/long */
  float el;             /* temporary for elevation */
/* check args */
  if (argc != 2) {
    fprintf(stderr,"Usage nwssa station_file\n");
    exit(1);
  }
/* first, open the station data file */
  if ((safile = fopen(argv[1],"r")) == NULL) {
    fprintf(stderr,"nwssa: can't open station file %s\n",argv[1]);
    exit(1);
  }
/* then, read it */
  fgets(cline,LMAX,safile);  /* throw away table header */
  listmax = 0;
  while ((listmax < SAMAX) && (fgets(cline,LMAX,safile) != NULL)) {
/* fill the table */
    if (sscanf(cline,"%f %s %f %f %f %f %f",&salist[listmax].num,
         cbuff,&latdeg,&latmin,&longdeg,&longmin,&el) != 7) {
      fprintf(stderr,"nwssa: station data table is bad\n");
      exit(1);
    }
    salist[listmax].lat = latdeg + latmin/60.;
    salist[listmax].lon = longdeg + longmin/60.;
    salist[listmax].elev = el/3281.;  /* convert feet to km */
    if (strlen(cbuff) == 3) {
      strcpy(salist[listmax].station,cbuff);
    }
    else {
      fprintf(stderr,"nwssa: %s isn't proper in station data table\n",cbuff);
      exit(1);
    }
    listmax++;
  }
/* do common data format initialization */
/* header first */
  nullhdr(hbuff,HBMAX,"float");
  bad = 10000.;
  badlim = 9999.;
  addcline(hbuff,HBMAX,"nwssa:\n");
  sprintf(cline,"%f",bad);
  addpar(hbuff,HBMAX,"bad",cline);
  sprintf(cline,"%f",badlim);
  addpar(hbuff,HBMAX,"badlim",cline);
  addfld(hbuff,HBMAX,'v',"station",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"date",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"month",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"year",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"hour",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"lat",100.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"long",100.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"elev",1000.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"temp",10.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"dewpt",10.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"mixrat",10.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"u",10.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"v",10.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"altset",10.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"ceiling",10.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"wx",1.,0.,'s',0);
/* write the header */
  puthdr(stdout,hbuff,HBMAX);
/* write the (null) static slice buffer */
  nsbuff = 0;
  putslice(stdout,nsbuff,sbuff);
/* create the variable slice buffer */
  nvbuff = elemcnt(hbuff,HBMAX,'v');
  vbuff = getbuff(nvbuff);
/* set the field pointers */
  cdfstn = getptr(hbuff,HBMAX,vbuff,'v','d',"station");
  cdfdate = getptr(hbuff,HBMAX,vbuff,'v','d',"date");
  cdfmonth = getptr(hbuff,HBMAX,vbuff,'v','d',"month");
  cdfyear = getptr(hbuff,HBMAX,vbuff,'v','d',"year");
  cdfhour = getptr(hbuff,HBMAX,vbuff,'v','d',"hour");
  cdflat = getptr(hbuff,HBMAX,vbuff,'v','d',"lat");
  cdflong = getptr(hbuff,HBMAX,vbuff,'v','d',"long");
  cdfelev = getptr(hbuff,HBMAX,vbuff,'v','d',"elev");
  cdftemp = getptr(hbuff,HBMAX,vbuff,'v','d',"temp");
  cdfdewpt = getptr(hbuff,HBMAX,vbuff,'v','d',"dewpt");
  cdfmixrat = getptr(hbuff,HBMAX,vbuff,'v','d',"mixrat");
  cdfu = getptr(hbuff,HBMAX,vbuff,'v','d',"u");
  cdfv = getptr(hbuff,HBMAX,vbuff,'v','d',"v");
  cdfaltset = getptr(hbuff,HBMAX,vbuff,'v','d',"altset");
  cdfceiling = getptr(hbuff,HBMAX,vbuff,'v','d',"ceiling");
  cdfwx = getptr(hbuff,HBMAX,vbuff,'v','d',"wx");
/* call the lex routine */
  yylex();
  exit(0);
}
