/* nwsua2 -- This program takes National Weather Service sounding
 * data as received by the Zephyr satellite and processed by zin and
 * select, and converts it to common data format.
 */

#include <stdio.h>
#include "cdfhdr.h"
#define WMAX 30
#define TMAX 30

char hbuff[HBMAX][LINE];                  /* header buffer */
float *sbuff,*vbuff;                      /* slice buffers */
long nsbuff,nvbuff;                       /* slice buffer lengths */
float *windex,*tindex;
float *hour,*date,*month,*year,*index;
float *cuwind,*cvwind,*celev,*cpres,*ctemp,*cdewpt;
                                          /* field pointers */
float bad;                                /* bad data value */
int loop;
int tcount,wcount;
int tmax,wmax;                            /* globals to transfer to subr */

main()
{
/* transfer constants to global variables */
  tmax = TMAX;
  wmax = WMAX;
  bad = 100000.;
/* create a header */
  nullhdr(hbuff,HBMAX,"float");
  addcline(hbuff,HBMAX,"nwsua:\n");
  addpar(hbuff,HBMAX,"bad","100000");
  addpar(hbuff,HBMAX,"badlim","99999");
  addfld(hbuff,HBMAX,'s',"windex",1.,0.,'s',1,"windex",WMAX);
  addfld(hbuff,HBMAX,'s',"tindex",1.,0.,'s',1,"tindex",TMAX);
  addfld(hbuff,HBMAX,'v',"hour",1.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"date",1.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"month",1.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"year",1.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"station",1.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"u",10.,0.,'s',1,"windex",WMAX);
  addfld(hbuff,HBMAX,'v',"v",10.,0.,'s',1,"windex",WMAX);
  addfld(hbuff,HBMAX,'v',"upres",10.,0.,'s',1,"windex",WMAX);
  addfld(hbuff,HBMAX,'v',"pres",10.,0.,'s',1,"tindex",TMAX);
  addfld(hbuff,HBMAX,'v',"temp",10.,0.,'s',1,"tindex",TMAX);
  addfld(hbuff,HBMAX,'v',"dewpt",10.,0.,'s',1,"tindex",TMAX);
/* create slice buffers */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  nvbuff = elemcnt(hbuff,HBMAX,'v');
  sbuff = getbuff(nsbuff);
  vbuff = getbuff(nvbuff);
/* get field pointers */
  windex = getptr(hbuff,HBMAX,sbuff,'s','d',"windex");
  tindex = getptr(hbuff,HBMAX,sbuff,'s','d',"tindex");
  hour = getptr(hbuff,HBMAX,vbuff,'v','d',"hour");
  date = getptr(hbuff,HBMAX,vbuff,'v','d',"date");
  month = getptr(hbuff,HBMAX,vbuff,'v','d',"month");
  year = getptr(hbuff,HBMAX,vbuff,'v','d',"year");
  index = getptr(hbuff,HBMAX,vbuff,'v','d',"station");
  cuwind = getptr(hbuff,HBMAX,vbuff,'v','d',"u");
  cvwind = getptr(hbuff,HBMAX,vbuff,'v','d',"v");
  celev = getptr(hbuff,HBMAX,vbuff,'v','d',"upres");
  cpres = getptr(hbuff,HBMAX,vbuff,'v','d',"pres");
  ctemp = getptr(hbuff,HBMAX,vbuff,'v','d',"temp");
  cdewpt = getptr(hbuff,HBMAX,vbuff,'v','d',"dewpt");
/* fill in index fields */
  for (loop = 0; loop < WMAX; loop++) windex[loop] = loop;
  for (loop = 0; loop < TMAX; loop++) tindex[loop] = loop;
/* write header and static slice */
  puthdr(stdout,hbuff,HBMAX);
  putslice(stdout,nsbuff,sbuff);
/* initialize month and year to zero */
  *year = 0.;
  *month = 0.;
/* read and interpret data stream on stdin and write variable slices */
  yylex();
  exit(0);
}
