%{
/* nwsua -- This lex subroutine converts National Weather Service
 * upper air soundings from teletype format to common data format.
 */
#include <math.h>
#define SCON 0.5144 /* knots to meters per second */
#define ACON 0.01745   /* degrees to radians conversion */
float uptab[54] =
{1010., 977., 943., 907., 875., 843., 807., 782., 752., 724., 696.,
 670., 643., 619., 595., 572., 550., 527., 506., 485., 465., 446.,
 428., 409., 393., 377., 360., 344., 329., 316., 302., 288., 275.,
 263., 250., 238., 227., 217., 206., 197., 188., 179., 171., 163.,
 155., 148., 141., 134., 128., 122., 116., 111., 106., 101.};
                    /* altitude to pressure conversion table */
char sname[10];     /* number of radiosonde station */
char tname[10];     /* time and date */
char ttemp[10];     /* temporary for time and date */
int linedate;       /* date from time stamp */
int savemonth = 0,saveyear = 0,lyear,lmonth,ldate;  /* temporaries */
int tdata;          /* flag to indicate temp and dewpt data in tdo */
float pres,temp,dewpt;  /* temporaries for thermo data */
int wstack;         /* number of winds left to analyze */
int wptr;           /* pointer to lsd wind alt */
int ndone;          /* wind temporary */
float speed,direct,elev,alt[5]; /* temps for wind data */
char line[5];       /* buffer */
%}
%Start init ready ready1 ready2 therm t1 t2 wind w1 w2
%%
					{BEGIN init;}
<init,ready>[A-Z][a-z][a-z][ ][A-Z][a-z][a-z][ A-Za-z0-9:]+	{getmy(yytext);}
<init,ready>"U"[A-Z][A-Z][A-Z][0-9]?[0-9]?	{BEGIN ready1;}
<ready1>[A-Z]+				{BEGIN ready2;}
<ready2>[0-9]+				{if (strlen(yytext) == 6) {
					   BEGIN ready;
					   strcpy(tname,yytext);
					 }
					 else BEGIN init;
					}
<ready>"TTBB"				{BEGIN therm; tdata = 0; bclear();}
<therm>[0-9/]+				{BEGIN t1;}
<t1>[0-9/]+				{if (strlen(yytext) == 5) {
					   BEGIN t2; strcpy(sname,yytext);
					 }
					 else BEGIN ready;
					}
<t2,ready>"PPBB"			{BEGIN wind; wstack = 0;}
<wind>[0-9/]+				{BEGIN w1;}
<w1>[0-9/]+				{BEGIN w2;
					   if (strcmp(sname,yytext) != 0)
					        BEGIN ready;
					}
<therm,t1,t2>"="			{BEGIN ready;}
<wind,w1,w2>"="				{wbuff(); BEGIN ready;}
<wind,w1,w2>[A-Z]			{unput(yytext[0]);wbuff();BEGIN ready;}
<t2>[0-9/]+				{tdo(yytext);}
<w2>[0-9/]+				{wdo(yytext);}
.|"\n"					;
%%

extern float *hour,*date,*month,*year;
extern float *index,*cpres,*ctemp,*cdewpt,*cuwind,*cvwind,*celev;
extern float *vbuff;
extern int nvbuff;
extern float bad;
extern int tmax,wmax;
static int loop;
static int tcount,wcount;
static char b1[15],b2[15],b3[15],b4[15],b5[15],b6[15];
static int nels;

/* get month and year from UNIX date line */
getmy(dateline)
char *dateline;
{
  nels = sscanf(dateline,"%s %s %s %s %s %s",b1,b2,b3,b4,b5,b6);
  if (nels == 6) {
    if (strncmp(b2,"Jan",3) == 0) savemonth = 1;
    else if (strncmp(b2,"Feb",3) == 0) savemonth = 2;
    else if (strncmp(b2,"Mar",3) == 0) savemonth = 3;
    else if (strncmp(b2,"Apr",3) == 0) savemonth = 4;
    else if (strncmp(b2,"May",3) == 0) savemonth = 5;
    else if (strncmp(b2,"Jun",3) == 0) savemonth = 6;
    else if (strncmp(b2,"Jul",3) == 0) savemonth = 7;
    else if (strncmp(b2,"Aug",3) == 0) savemonth = 8;
    else if (strncmp(b2,"Sep",3) == 0) savemonth = 9;
    else if (strncmp(b2,"Oct",3) == 0) savemonth = 10;
    else if (strncmp(b2,"Nov",3) == 0) savemonth = 11;
    else if (strncmp(b2,"Dec",3) == 0) savemonth = 12;
    else return;
    linedate = atoi(b3);
    saveyear = atoi(b6);
  }
}

#define TSLOP 10.0
#define PSLOP 3.0
/* interpret thermodynamic sounding number */
tdo(text)
char *text;
{
  char c;
  float twild,pwild,dup,ddown,dspan;
/* check data validity */
  if (strlen(text) != 5) {
    tdata = 0;
    return;
  }
  if (tdata == 0) {
/* pressure data */
    tdata = 1;
/* first two characters of pressure word must be equal */
    if (text[0] != text[1]) {
      tdata = 0;
      return;
    }
/* pressure must not be missing */
    if ((text[2] == '/')||(text[3] == '/')||(text[4] == '/')) {
      tdata = 0;
      return;
    }
    pres = atof(text + 2);
/* pressure represented mod 1000 */
    if (pres < 100.) pres += 1000.;
  }
/* temp and dewpoint data */
  else if (tdata == 1) {
    tdata = 0;
/* dewpoint depression first */
    c = text[3];
/* eliminate missing dewpoint */
    if ((c == '/') || (text[4] == '/')) dewpt = bad;
/* digit 4 >= 5 means whole number + 50 */
    else if ((c == '5')||(c == '6')||(c == '7')||(c == '8')||(c == '9'))
         dewpt = atof(text + 3) - 50.;
/* otherwise in tenths */
    else dewpt = atof(text + 3)/10.;
/* null terminate temperature part of string */
    text[3] = '\0';
/* absolute value of temperature */
    temp = atof(text)/10.;
    c = text[2];
/* temp negative if last digit odd */
    if ((c == '1')||(c == '3')||(c == '5')||(c == '7')||(c == '9')) {
      temp = -temp;
    }
/* construct dewpoint from temp and depression */
    if (dewpt < bad) dewpt = temp - dewpt;
    cpres[tcount] = pres;
    ctemp[tcount] = temp;
    cdewpt[tcount] = dewpt;
    if (tcount < tmax - 1) tcount++;
  }
/* do some error checking on temperature profiles */
/* first, pressures must be in decreasing order */
  for (loop = 1; loop < tcount; loop++) {
    if (cpres[loop] > cpres[loop - 1]) {
      tcount = tfix(tcount,loop);
    }
  }
/* check for wild temperatures and pressures */
  for (loop = 1; loop < tcount - 1; loop++) {
    dup = fabs(ctemp[loop + 1] - ctemp[loop]);
    ddown = fabs(ctemp[loop] - ctemp[loop - 1]);
    dspan = fabs(ctemp[loop + 1] - ctemp[loop - 1]);
    twild = dup + ddown - dspan - TSLOP;
    dup = fabs(cpres[loop + 1] - cpres[loop]);
    ddown = fabs(cpres[loop] - cpres[loop - 1]);
    dspan = fabs(cpres[loop + 1] - cpres[loop - 1]);
    pwild = dup + ddown - dspan - PSLOP;
/* if a wild value is found, eliminate that level */
    if ((twild > 0.) || (pwild > 0.)) {
      tcount = tfix(tcount,loop);
    }
  }
}

/* delete a line from thermodynamic data */
int tfix(tc,l)
int tc,l;
{
  int i;
  for (i = l; i < tc - 1; i++ ) {
    cpres[i] = cpres[i + 1];
    ctemp[i] = ctemp[i + 1];
    cdewpt[i] = cdewpt[i + 1];
  }
  cpres[tc - 1] = ctemp[tc - 1] = cdewpt[tc - 1] = bad;
  return(tc - 1);
}

/* interpret wind sounding number */
wdo(text)
char *text;
{
/* check for bad data */
  if (strlen(text) != 5) return;
  if (text[0] == '9') {
/* we have an altitude designator */
    if (wstack != 0) {
      wstack = 0;
    }
    wptr = 2;
    while (wptr < 5) {
      line[0] = text[1];
      if ((line[1] = text[wptr]) == '/') break;
      line[2] = '\0';
      alt[wstack] = atof(line);
      wptr++;
      wstack++;
    }
    ndone = 0;
  }
  else {
/* we have some winds */
    if (wstack <= 0) {
      return;
    }
    elev = alt[ndone];
    if (text[0] != '/') {
      speed = atof(text + 3);
      text[3] = '\0';
      direct = atof(text);
    }
    else speed = direct = bad;
    ndone++;
    wstack--;
/* get east and north wind components */
    cuwind[wcount] = -speed*sin(direct*ACON)*SCON;
    cvwind[wcount] = -speed*cos(direct*ACON)*SCON;
/* turn heights back into pressures */
    if (elev <= 0.) celev[wcount] = cpres[0];
    else if (elev < 53.4) {
      celev[wcount] = uptab[(int)(elev + .5)];
    }
    if (wcount < wmax - 1) wcount++;
  }
}

/* begin cdf buffer */
bclear()
{
  tcount = wcount = 0;
  for (loop = 0; loop < wmax; loop++) {
    cuwind[loop] = cvwind[loop] = celev[loop] = bad;
  }
  for (loop = 0; loop < tmax; loop++) {
    cpres[loop] = ctemp[loop] = cdewpt[loop] = bad;
  }
}

/* write cdf buffer */
wbuff()
{
  strcpy(ttemp,tname);
  ttemp[4] = '\0';
  *hour = atof(ttemp + 2);
  ttemp[2] = '\0';
  *date = atof(ttemp);
  *index = atof(sname);
  lyear = saveyear;
  lmonth = savemonth;
  if ((lmonth < 1) || (lmonth > 12)) {
    *year = 0.;
    *month = 0.;
    putslice(stdout,nvbuff,vbuff);
    return;
  }
  ldate = *date + .5;
/* date stamp new month, sounding old month */
  if (ldate - linedate > 15) {
    lmonth--;
    if (lmonth == 0) {
      lmonth = 12;
      lyear--;
    }
  }
/* date stamp old month, sounding new month */
  if (ldate - linedate < -15) {
    lmonth++;
    if (lmonth == 13) {
      lmonth = 1;
      lyear++;
    }
  }
  *month = lmonth;
  *year = lyear;
  putslice(stdout,nvbuff,vbuff);
}
