/* nwsuafix */
/* takes nws data in a cdf format and performs certain calculations  */
/* it finds the mixing ratio, the u and v wind velocities, the       */
/* virtual temp and places them in steady increments of pressure     */
/* the geopotenial height for a standard atmoshpere is found and the */
/* geopotenial height for the acutal is compared to the standard and */
/* placed with a field as with the stand. which is placed in the     */
/* static field.                                                     */

#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

#define  DPRES    -50.0  /* increment size of pressure in mbars */
#define  PRES0   1000.0  /* starting place of pressure in mbars */
#define  PRESFIN  100.0  /* ending pressure in mbars */
#define  CTOK    273.16  /* constant for changing celsius to kelvin */
#define  G          9.8  /* avg. acc of gravity in meters/s^2 */
#define  LAPSE    0.007  /* rate at which temp dec. (units Kelvin/meters) */
#define  SP0     1000.0  /* inital pres (at sea level) for standard atmos. */
#define  ST0      300.0  /* inital temp for standard atmosphere at 1000 mbars */
#define  GAS     287.04  /* gas constant for dry air joule/gram*Kelvin */
#define  EPS      0.622  /* ratio of molecular weights */
#define  VCONST   0.377  /* constant for virtual temp determination */
#define  KAPPA    0.286  /* potential temperature constant */
#define  A       17.269  /* used with water vapor saturation routine */
#define  B        35.86
#define  ESO     6.1078
#define  MAXSTNS    600  /* maximum number of stations allowed */

#define  gptrout(A)  getptr(bufferw,HBMAX,vbuffout,'v','d',A);
#define  gptrin(B)   getptr(buffer,HBMAX,vbuff,'v','d',B);

FILE *fopen(),*fd;

char buffer[HBMAX][LINE];      /* buffers for header data */
char bufferw[HBMAX][LINE];     /* buffers for header data */
char cline[LINE];              /* character line buffer */
float *sbuff,*vbuff,*vbuffout; /* buffer that contain slice & computed data */
float *vtemp,*mixrat,*geoht;   /* buffers to hold calculated data */
float *presptr,*temptr,*vptr;  /* float pointers to the respective fields */
float *dewptr,*upresptr,*uptr; /* in input buffer */
float *station;
float *lntemp,*lnvtemp;        /* pointers to start of data thats to be */
float *lnmr,*lngeo,*lndp;      /* sent as output after linear approx */
float *lnvtheta,*lnu,*lnv;
float *latptr,*lonptr;         /* pointers to lat and long locations */
float *psurf;                  /* surface pressure */
float *geo0;                   /* a pointer to the standard geoht values */
float bad,badlim;              /* values for bad and badlim */
float linprox();               /* linear approx routine */
float es();                    /* saturation vapor pressure routine -- */
                               /* comes from thermo.c */

main(argc,argv)
int argc;
char *argv[];
{
  int loop,loop1,num;             /* used in looping */
  int whpair,flag;                /* used in finding correct linprox */
  int newdim,olddim;              /* size of dim (in old and new files) */
  int sta,lat,lon,el;             /* used in with data frome station file */
  int table[MAXSTNS][4],numst;    /* contains data from stations */
  char pvalue[LINE];              /* used in getting parameter data */
  char pres[LINE],upres[LINE];    /* names of command line aguments */
  char u[LINE],v[LINE];           /*   "   "     "     "      "     */
  char temp[LINE],dewpt[LINE];    /*   "   "     "     "      "     */
  char datfil[LINE];              /* file that contains lat/long/elv data */
  char cl[LINE],nm[LINE];         /* comment line data & station data */
  char scalar[5][LINE];           /* name of the time scalars */
  long nel,nelw;                  /* size of data to be read and written */
  float prh,prl,prmid;            /* used in linear approx. */
  float pth,ptl,ptmid;            /* mixrat, velocities, etc. */
  float gndprs,prs,prs1,misc;     /* used in computing geopht */
  float *buffpos,*posold,*posnew; /* pointers for transfering info */
  float temp1,temp2;              /* temp storage and floating loop */
  float esval,vtempfact;          /* temporaries */
  float dpres,pres0,presfin;	  /* values for initial pres and increment */
  struct field *fp;               /* pointer to the field structure */

/* check command line arguments */
  if ((argc != 8)&&(argc != 11)) {
     fprintf(stderr,"nwsuafix pres temp dewpt upres u v station_file");
     fprintf(stderr," [pres0 dpres num_pres_levels]\n");
    exit(1);
  }

/* move stuff out of argument line */
  strcpy(pres,argv[1]);
  strcpy(temp,argv[2]);
  strcpy(dewpt,argv[3]);
  strcpy(upres,argv[4]);
  strcpy(u,argv[5]);
  strcpy(v,argv[6]);
  strcpy(datfil,argv[7]);

/* see if opitional arguments are given  if not revert to default values */
  if (argc==11) {
    pres0=atof(argv[8]);
    dpres=atof(argv[9]);
    num=atoi(argv[10]);
    if (num < 1)
      reporterr("number of pressure levels less than 1");
    presfin=(pres0+dpres*(num-1));
    if ((presfin > pres0)||(dpres >= 0)||(presfin <= 0.0))
      reporterr("error with pres0 dpres or num_pres_levels");
  }
  else {
    pres0=PRES0;
    dpres=DPRES;
    presfin=PRESFIN;
  }

/* get header from stdin */
  gethdr(stdin,buffer,HBMAX);

/* check format */
  if (getfmt(buffer,HBMAX) != 'f')
    reporterr("file must be float!");

/* check command line arguments for validity */
  for (loop=0;loop<6;loop++) {
    fp=seekfld(buffer,HBMAX,'v',argv[loop+1]);
    if ((fp == NULL)||(fp->dim != 1)) {
       sprintf(cline,"error with %s field",argv[loop+1]);
      reporterr(cline);
    }
  }

/* make output header */
  nullhdr(bufferw,HBMAX,"float");

/* transfer comments */
  copycmt(bufferw,HBMAX,buffer,HBMAX);

/* add comment line */
   sprintf(cl,"nwsuafix: %s %s %s\n",pres,temp,dewpt);
  addcline(bufferw,HBMAX,cl);
   sprintf(cl,"  %s %s %s %s\n",upres,u,v,datfil);
  addcline(bufferw,HBMAX,cl);
  if (argc == 11) {
     sprintf(cl,"  %.2f %.2f %.2f\n",pres0,dpres,num);
    addcline(bufferw,HBMAX,cl);
  }

/* transfer old parameters and add new parameters */
  copypar(bufferw,HBMAX,buffer,HBMAX);
   sprintf(pvalue,"%f",pres0);
  addpar(bufferw,HBMAX,"pres0",pvalue);
   sprintf(pvalue,"%f",dpres);
  addpar(bufferw,HBMAX,"dpres",pvalue);

/* create new static fields */
  newdim = (int)((presfin-pres0)/dpres + 1);
  fp=seekfld(buffer,HBMAX,'v',pres);
  olddim=fp->dsize1;
  addfld(bufferw,HBMAX,'s',"pres",1000.0,fp->sadd,fp->bytes,1,"pres",newdim);
  addfld(bufferw,HBMAX,'s',"geoht0",1000.0,fp->sadd,fp->bytes,1,
          "pres",newdim);

/* transfer scalar variable fields and add lat and long */
  fp=getfld(buffer,HBMAX,'v',0);
  addfld(bufferw,HBMAX,'v',"lat",1000.0,fp->sadd,fp->bytes,0);
  addfld(bufferw,HBMAX,'v',"long",1000.0,fp->sadd,fp->bytes,0);
  for (loop=0;loop < 5;loop++) {
    if (loop==0) strcpy(scalar[loop],"hour");
    else if (loop==1) strcpy(scalar[loop],"date");
    else if (loop==2) strcpy(scalar[loop],"month");
    else if (loop==3) strcpy(scalar[loop],"year");
    else strcpy(scalar[loop],"station");
    fp=seekfld(buffer,HBMAX,'v',scalar[loop]);
    addfld(bufferw,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,0);
  }

/* same for the 1d fields such as u and v etc */
  for (loop=2;loop < 7;loop++) {
    fp=seekfld(buffer,HBMAX,'v',argv[loop]);

/* replace wind pressure levels with geopotential height perturbation */
    if (loop==4) strcpy(fp->fname,"geoht");
    addfld(bufferw,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,1,
            pres,newdim);
  }

/* add mixing ratio, temperature, potential temperature and surface presure */
  addfld(bufferw,HBMAX,'v',"mixrat",100.0,0.0,'s',1,pres,newdim);
  addfld(bufferw,HBMAX,'v',"vtemp",10.0,0.0,'s',1,pres,newdim);
  addfld(bufferw,HBMAX,'v',"vtheta",10.0,0.0,'s',1,pres,newdim);
  addfld(bufferw,HBMAX,'v',"psurf",3.0,0.0,'s',0);

/* send updated header to stdout */
  puthdr(stdout,bufferw,HBMAX);

/* determine values for bad and badlim */
  if (seekpar(buffer,HBMAX,"bad",pvalue)==FAIL)
    bad=BAD;
  else
    bad=atof(pvalue);
  if (seekpar(buffer,HBMAX,"badlim",pvalue)==FAIL)
    badlim=BADLIM;
  else
    badlim=atof(pvalue);

/* make room for static slice (discard orginal slice) */
  nel=elemcnt(buffer,HBMAX,'s');
  sbuff=getbuff(nel);
  getslice(stdin,nel,sbuff);
  free(sbuff);

/* set aside memory for the new one and pointer to its data */
  nel=elemcnt(bufferw,HBMAX,'s');
  sbuff=getbuff(nel);
  geo0=getptr(bufferw,HBMAX,sbuff,'s','d',"geoht0");

/* send data to static slice (pres and geoht0) */
  buffpos=getptr(bufferw,HBMAX,sbuff,'s','d',"pres");
  for (loop = 0;loop < newdim;loop++) {
    prs = pres0 + dpres*loop;
    buffpos[loop] = prs;
    temp1 = prs/SP0;
    temp2 = GAS*LAPSE/G;
    misc = pow(temp1,temp2);
    geo0[loop]=(ST0/LAPSE)*(1.-misc)/1000.0;
  }

/* output the buffer */
  putslice(stdout,nel,sbuff);

/* update variable fields */
/* allocate memory for variable slices and buffers */
  nelw=elemcnt(bufferw,HBMAX,'v');
  nel=elemcnt(buffer,HBMAX,'v');
  vbuffout=getbuff(nelw);
  vbuff=getbuff(nel);
  mixrat=getbuff(olddim);
  vtemp=getbuff(olddim);
  geoht=getbuff(olddim);

/* set up pointers to in and out data fields */
  lnvtheta=gptrout("vtheta");
  lnvtemp=gptrout("vtemp");
  lntemp=gptrout("temp");
  lnmr=gptrout("mixrat");
  lngeo=gptrout("geoht");
  psurf=gptrout("psurf");
  lonptr=gptrout("long");
  latptr=gptrout("lat");
  lndp=gptrout("dewpt");
  lnu=gptrout("u");
  lnv=gptrout("v");
  upresptr=gptrin(upres);
  presptr=gptrin(pres);
  dewptr=gptrin(dewpt);
  temptr=gptrin(temp);
  uptr=gptrin(u);
  vptr=gptrin(v);

/* open the station file */
  if ((fd=fopen(datfil,"r")) == NULL) {
     sprintf(cline,"%s cannot be opened",datfil);
    reporterr(cline);
  }

/* load the data in to a table for easy use */
  loop1=0;
  while (fgets(cline,LINE - 1,fd) != NULL) {
    loop=sscanf(cline,"%d %d %d %d %s %s",&sta,&lat,&lon,&el,cl,nm);
    if (loop1 >= MAXSTNS)
      reporterr("too many stations in station table");
    if (loop < 4) {
       sprintf(cline,"format error in station table (line = %d)",loop1);
      reporterr(cline);
    }
    table[loop1][0]=sta;
    table[loop1][1]=lat;
    table[loop1][2]=lon;
    table[loop1++][3]=el;
  }
  fclose(fd);
  numst=loop1;

/* cycle through slices */
  station=getptr(buffer,HBMAX,vbuff,'v','r',"station");
  while (getslice(stdin,nel,vbuff)!=EOF) {
    *psurf = gndprs = presptr[0];

/* find latitude, longitude and elevation from table for each station */
/* if flag = 0 after going thru station table then not found and not sent out */
    flag=0;
    for (loop=0;loop<numst;loop++) {
      if (station[0]==table[loop][0]) {
        lat=table[loop][1];
        lon=table[loop][2];
        el=table[loop][3];     
        flag=1;
        break;
      }
    }

/* place latitude and longitude in vbuffout */
    latptr[0]=lat/100+(lat%100)/60.0;
    lonptr[0]=lon/100+(lon%100)/60.0;

/* send hour, date, month, year, station */
    for (loop=0;loop < 5;loop++) {
      posold=getptr(buffer,HBMAX,vbuff,'v','r',scalar[loop]);
      posnew=getptr(bufferw,HBMAX,vbuffout,'v','r',scalar[loop]);
      *posnew = *posold;
    }

/* loop through the original data and calculate mixrat, vtemp, and geoht */
    for (loop=0;loop<olddim;loop++) {
      prs=presptr[loop];

/* calculate the mixing ratio */
      if ((fabs(prs) > badlim)||(fabs(dewptr[loop]) > badlim))
        mixrat[loop]=bad;
      else {
        misc=dewptr[loop]+CTOK;
        esval = es(misc);
        mixrat[loop]=(EPS*esval/(prs-esval))*1000.0;
        vtempfact = 1. - VCONST*es(misc)/prs;
      }

/* calculate virtual temp */
      if (fabs(mixrat[loop]) > badlim)
        vtemp[loop]=temptr[loop];
      else if (fabs(temptr[loop]) > badlim)
        vtemp[loop]=bad;
      else {
        vtemp[loop]=(temptr[loop]+CTOK)/vtempfact - CTOK;
      }
    }

/* find the geopotential height */
    geoht[0]=el/1000.0;
    for (loop=1;loop < olddim;loop++) {
      prs=presptr[loop];
      prs1=presptr[loop-1];
      if ((fabs(prs) > badlim)||(vtemp[loop] > badlim)||(fabs(prs1) > badlim)||
            (geoht[loop-1]>badlim)||(vtemp[loop-1]>badlim))
        geoht[loop]=bad;
      else {
        misc = (GAS/G)*((vtemp[loop-1] + vtemp[loop])/2. + CTOK);
        geoht[loop]=geoht[loop-1] + misc*log(prs1/prs)/1000.0;
      }
    }

/* now cycle through the computed data and the wind velocities and linearly */
/* approx. the values at constant increments of pressure (i.e. dpres) */
    whpair=0;
    for (loop=0;loop < newdim;loop++) {
      prmid = pres0 + dpres*loop;

/* if given pres is too high (ie below ground level) send bad */
      if (prmid > gndprs) {
        lnvtemp[loop]=lnvtheta[loop]=lntemp[loop]=bad;
        lngeo[loop]=lnmr[loop]=lndp[loop]=bad;
      }

/* find the correct set of pressures to find the approximation between */
      else {
        prh=presptr[whpair];
        prl=presptr[whpair+1];
        while ((prmid < prl)&&(prl < badlim)) {
          whpair++;
          prh=presptr[whpair];
          prl=presptr[whpair+1];
        }
/* flag pressures out of order */
        if ((prl > prh) && (fabs(prl) < badlim)) flag = 0;

/* linear approx for vtemp */
        pth=vtemp[whpair];
        ptl=vtemp[whpair+1];
        ptmid=linprox(prh,prmid,prl,pth,ptl);
        lnvtemp[loop]=ptmid;

/* for mixing ratio */
        pth=mixrat[whpair];
        ptl=mixrat[whpair+1];
        ptmid=linprox(prh,prmid,prl,pth,ptl);
        lnmr[loop]=ptmid;

/* for dewpoint */
        pth=dewptr[whpair];
        ptl=dewptr[whpair+1];
        ptmid=linprox(prh,prmid,prl,pth,ptl);
        lndp[loop]=ptmid;

/* for temperature */
        pth=temptr[whpair];
        ptl=temptr[whpair+1];
        ptmid=linprox(prh,prmid,prl,pth,ptl);
        lntemp[loop]=ptmid;

/* for geopotential height */
        pth=geoht[whpair];
        ptl=geoht[whpair+1];
        ptmid=linprox(prh,prmid,prl,pth,ptl);
        if (ptmid > badlim) lngeo[loop]=bad;
        else lngeo[loop]=ptmid-geo0[loop];

/* compute virtual potential temperature */
        if (fabs(lnvtemp[loop]) < badlim) {
          lnvtheta[loop] = (lnvtemp[loop] + CTOK)*pow(pres0/prmid,KAPPA);
        }
        else lnvtheta[loop] = bad;
      }
    }

/* same as above except for the wind velocities u and v */
    whpair=0;
    for (loop=0;loop < newdim;loop++) {
      prmid = pres0 + dpres*loop;
      if (prmid > gndprs) {
        lnu[loop]=bad;
        lnv[loop]=bad;
      }
      else {
        prh=upresptr[whpair];
        prl=upresptr[whpair+1];
        while ((prmid < prl)&&(prl < badlim)) {
          whpair++;
          prh=upresptr[whpair];
          prl=upresptr[whpair+1];
        }

/* flag pressures out of order */
        if ((prl > prh) && (fabs(prl) < badlim)) flag = 0;

/* for u */
        pth=uptr[whpair];
        ptl=uptr[whpair+1];
        ptmid=linprox(prh,prmid,prl,pth,ptl);
        lnu[loop]=ptmid;

/* for v */
        pth=vptr[whpair];
        ptl=vptr[whpair+1];
        ptmid=linprox(prh,prmid,prl,pth,ptl);
        lnv[loop]=ptmid;
      }
    }

/* if station wasn't found in list or if pressures out of order then skip it */
    if (flag != 0) putslice(stdout,nelw,vbuffout);
  }
  exit(0);
}


/* this routine linearly interpolates */
float linprox(p1,p2,p3,dp1,dp2)
float p1,p2,p3,dp1,dp2;
{
  if ((p1>badlim)||(p2>badlim)||(p3>badlim)||(dp1>badlim)||(dp2>badlim))
    return(bad);
  else
    return(dp1-((p1-p2)*(dp1-dp2))/(p1-p3));
}

/* teten's formula for water saturation vapor pressure */
float es(t)
float t;
{
  return(ESO*exp(A*(t - CTOK)/(t - B)));
}

reporterr(eline)
char eline[];
{
   fprintf(stderr,"nwsuafix: %s\n",eline);
  exit(1);
}
