/* scdua -- This program decodes sounding data from NCAR archives, as
 * written by a Fortran program that reads these archives.
 */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

#define MAXLINE 200
#define MAXNUM  100

char hbuffer[HBMAX][LINE];  /* header buffer */
float *sbuffout;            /* slice buffer */

main(argc,argv)
int argc;
char *argv[];
{
  int len,loop,flag,rstat,lnum,loop1;
  int rday,rmonth,ryear,rhour;
  int wi,ti;
  int syear,smonth,sday,shour,sstat,sint;
  char dat1[MAXLINE],dat2[MAXLINE],cl[LINE];
  char id[LINE],stacked[LINE];
  char *st;
  long nel;
  float sfl,apres[MAXNUM],atemp[MAXNUM];
  float adp[MAXNUM],awdir[MAXNUM],awspeed[MAXNUM];
  float *hr,*station;
  float *dt,*yr,*mn,bad;
  float *uwind,*vwind,*upres;
  float *pres,*dewpt,*temp;

/* check command line */
  if (argc != 6) {
    fprintf(stderr,"Usage: scdua day month year hour station\n");
    fprintf(stderr,"       hour is 0 or 12\n");
    exit(1);
  }

/* retrieve the command line arguments */
  rday=atoi(argv[1]);
  rmonth=atoi(argv[2]);
  ryear=atoi(argv[3]);
  rhour=atoi(argv[4])*100;
  rstat=atoi(argv[5]);

/* initialize read in stuff */
  flag=0;
  for (loop=0;loop<MAXNUM;loop++)
    apres[loop]=atemp[loop]=adp[loop]=awdir[loop]=awspeed[loop]=10000.0;

  sprintf(dat1,"start");
/* read in the data */
  while (flag==0) {
    st=fgets(dat2,MAXLINE,stdin);
    if (st==NULL) {
      fprintf(stderr,"scdua: requested sounding not found\n");
      exit(1);
    }

/* look for STACKED DATA */
    len=sscanf(dat2,"%s %s %d",stacked,cl,&lnum);
    if (strncmp("STACKED",stacked,7)!=0) {
      strcpy(dat1,dat2);
      continue;
    }
    len=sscanf(dat1,"%d %d %d %d %d %s %f %f %f %d %f %d %d",&sint,&syear,
       &smonth,&sday,&shour,id,&sfl,&sfl,&sfl,&sstat,&sfl,&sint,&sint);
    if (strncmp(id,"ADPUPAC",7)!=0) {
      fprintf(stderr,"scdua: error with data\n");
      exit(1);
    }

/* check the station#, time and date */
    if ((sday==rday)&&(smonth==rmonth)&&(syear==ryear)&&(shour==rhour)&&
       (sstat==rstat)) {

/* if yes then cycle through the data and place in respective slots */
      for (loop=0;loop<lnum;loop++) {
        st=fgets(dat2,MAXLINE,stdin);
        len=sscanf(dat2,"%d %f %f %f %f %f %f",&sint,&apres[loop],
          &sfl,&atemp[loop],&adp[loop],&awdir[loop],&awspeed[loop]);
      }
      break;
    }
  }
  ti=lnum;
  for (loop=0,wi=0;loop<ti;loop++)
    if (awdir[loop] < 999.0) ++wi;

/* make output header */
  nullhdr(hbuffer,HBMAX,"float");

/* add parameters */
  addpar(hbuffer,HBMAX,"bad","10000");
  addpar(hbuffer,HBMAX,"badlim","9999");
  bad = 10000;

/* add static and variable fields */
  addfld(hbuffer,HBMAX,'s',"tindex",10.0,10.0,'s',1,"tindex",ti);
  addfld(hbuffer,HBMAX,'s',"windex",10.0,10.0,'s',1,"windex",wi);
  addfld(hbuffer,HBMAX,'v',"hour",10.0,10.0,'s',0);
  addfld(hbuffer,HBMAX,'v',"date",10.0,10.0,'s',0);
  addfld(hbuffer,HBMAX,'v',"month",10.0,10.0,'s',0);
  addfld(hbuffer,HBMAX,'v',"year",10.0,10.0,'s',0);
  addfld(hbuffer,HBMAX,'v',"station",10.0,10.0,'s',0);
  addfld(hbuffer,HBMAX,'v',"u",10.0,10.0,'s',1,"windex",wi);
  addfld(hbuffer,HBMAX,'v',"v",10.0,10.0,'s',1,"windex",wi);
  addfld(hbuffer,HBMAX,'v',"upres",10.0,10.0,'s',1,"windex",wi);
  addfld(hbuffer,HBMAX,'v',"pres",10.0,10.0,'s',1,"tindex",ti);
  addfld(hbuffer,HBMAX,'v',"dewpt",10.0,10.0,'s',1,"tindex",ti);
  addfld(hbuffer,HBMAX,'v',"temp",10.0,10.0,'s',1,"tindex",ti);

/* add two comment line */
  sprintf(cl,"scdua:\n");
  addcline(hbuffer,HBMAX,cl);
  sprintf(cl," date: %d %d %d\n",rday,rmonth,ryear);
  addcline(hbuffer,HBMAX,cl);
  sprintf(cl," time: %d\n",rhour/100);
  addcline(hbuffer,HBMAX,cl);
  sprintf(cl," station: %d\n",rstat);
  addcline(hbuffer,HBMAX,cl);

/* output the header */
  puthdr(stdout,hbuffer,HBMAX);

/* set aside memory for static slice */
  nel = elemcnt(hbuffer,HBMAX,'s');
  sbuffout=getbuff(nel);

/* put info into the index variables */
  hr=getptr(hbuffer,HBMAX,sbuffout,'s','d',"tindex");
  for (loop=0;loop<ti;loop++) hr[loop]=(float)(loop);
  hr=getptr(hbuffer,HBMAX,sbuffout,'s','d',"windex");
  for (loop=0;loop<wi;loop++) hr[loop]=(float)(loop);

/* output static slice */
  putslice(stdout,nel,sbuffout);
  
/* set aside memory for variable slice */
  free(sbuffout);
  nel = elemcnt(hbuffer,HBMAX,'v');
  sbuffout=getbuff(nel);

/* set pointers */
  hr=getptr(hbuffer,HBMAX,sbuffout,'v','d',"hour");
  dt=getptr(hbuffer,HBMAX,sbuffout,'v','d',"date");
  yr=getptr(hbuffer,HBMAX,sbuffout,'v','d',"year");
  mn=getptr(hbuffer,HBMAX,sbuffout,'v','d',"month");
  station=getptr(hbuffer,HBMAX,sbuffout,'v','d',"station");

/* place scalar data */
  *hr=(float)(shour/100);
  *dt=(float)(sday);
  *yr=(float)(syear);
  *mn=(float)(smonth);
  *station=(float)(sstat);

/* get pointers to rest of fields */
  uwind=getptr(hbuffer,HBMAX,sbuffout,'v','d',"u");
  vwind=getptr(hbuffer,HBMAX,sbuffout,'v','d',"v");
  upres=getptr(hbuffer,HBMAX,sbuffout,'v','d',"upres");
  pres=getptr(hbuffer,HBMAX,sbuffout,'v','d',"pres");
  dewpt=getptr(hbuffer,HBMAX,sbuffout,'v','d',"dewpt");
  temp=getptr(hbuffer,HBMAX,sbuffout,'v','d',"temp");

/* put in temp, pres, etc. */
  for (loop=0,loop1=0;loop<ti;loop++) {
    pres[loop]=apres[loop];
    if ((adp[loop]<99.0)&&(atemp[loop]<900.0))
      dewpt[loop]=atemp[loop]-adp[loop];
    else dewpt[loop]=bad;
    if (atemp[loop]<900.0) temp[loop]=atemp[loop];
    else temp[loop]=bad;

/* same for winds */
    if ((awdir[loop] < 990.0)&&(awspeed[loop] < 990.0)) {
      upres[loop1]=pres[loop];
      vwind[loop1]= -1.0*(awspeed[loop]*0.5144)*cos(awdir[loop]/57.296);
      uwind[loop1]= -1.0*(awspeed[loop]*0.5144)*sin(awdir[loop]/57.296);
      ++loop1;
    }
  }
  putslice(stdout,nel,sbuffout);
  exit(0);
}
