/* pidemo -- A demonstration program to illustrate how to use the
 * dispimage subroutines in a simple context.
 */

#include <stdio.h>
#include <math.h>

#define SX 512                  /* horizontal image size in pixels */
#define SY 300                  /* vertical size in pixels */
#define OX 64			/* horizontal offset */
#define OY 50                   /* vertical offset */
#define NX (SX + 2*OX)          /* size of window */
#define NY (SY + 2*OY)          /* size of window */

unsigned char mono0[NY][NX/8];  /* monochrome image */
unsigned char mono1[NY][NX/8];  /* monochrome image */
unsigned char color0[NY][NX];   /* color image */
unsigned char color1[NY][NX];   /* color image */
int ncolors;                    /* number of colors */
int colornumber;                /* color choice for polyfill */
long cvals[256];                /* pixel values */
int image_depth;                /* depth of image supported by display */
int xmouse[5],ymouse[5];        /* mouse position */
int xbutton[5];                 /* mouse button */
int ix,jx,iy,loop,poop;         /* looping variables */
int intrhand();                 /* interrupt handling routine */
int i,j,k;
int panicbutton, quitbutton, killmebutton, erasemebutton;
int image;
int button;
int color[10];

main()
{

/* set up display */
  image_depth = setupdisp("pidemo",NX,NY);
  fprintf(stderr,"pidemo: display set up\n");

/* get the colors */
  ncolors = getcolors(cvals);

/*
  setbackground(0);
*/

  color[0] = -1;
  color[1] = cvals[0];
  color[2] = cvals[1];
  color[3] = cvals[24];
  color[4] = cvals[28];
  color[5] = cvals[31];

/* make up bogus color and monochrome images */
  for (ix = 0; ix < NX; ix++) {
    for (iy = 0; iy < NY; iy++) {
      jx = ix/8;
      loop = jx - (jx/ncolors)*ncolors;
      poop = loop + ncolors/2;
      if (poop >= ncolors) poop -= ncolors;
      color0[iy][ix] = cvals[loop];
      color1[iy][ix] = cvals[poop];
      if (ix - jx*8 == 0) {
        if (jx - (jx/2)*2 == 0) {
          mono0[iy][jx] = 0;
          mono1[iy][jx] = -1;
        }
        else {
          mono0[iy][jx] = -1;
          mono1[iy][jx] = 0;
        }
      }
    }
  }

/* send the first image to the server */
  if (image_depth == 1) sendimage(0,OX,OY,SX,SY,mono0);
  else sendimage(0,OX,OY,SX,SY,color0);

/* draw lines near the top of the image, slanting down to right */
  k = 0;
  for(i = 0; i <= (image_depth == 1?2:5) ; i++)
    for(j = 1; j <= 4; j++)
    {
      drawline(0,10,k,450,100+k,j,color[i]);
      drawline(0,450,100+k,630,100+k,j,color[i]);
      k += 10;
    }

/* draw some text near bottom of image */
  for(i = 1; i <= (image_depth == 1?2:5) ; i++)
    drawtext(0,100,210+15*i,color[i],"This is a text sample -- image 0");

/* draw a circle around a disk in the upper-right corner in invert mode */
  drawcircle(0,450,60,50,-1);
  fillcircle(0,450,60,30,-1);

/* draw a circle around a disk in the upper-right corner in foreground */
  drawcircle(0,570,60,50,fg());
  fillcircle(0,570,60,30,fg());

/* draw a bunch of dots at top center */
  for(i = 0; i <= (image_depth == 1?2:5) ; i++)
    for(j = 5; j <= 45; j+=5)
      drawpoint(0,i*5+300,j,color[i]);

/* draw a foreground and inverted rectangle at the bottom */
  fillrectangle(0,10,320,110,380,-1);
  fillrectangle(0,310,320,410,380,fg());

/* send the second image to the server */
  if (image_depth == 1) sendimage(1,OX,OY,SX,SY,mono1);
  else sendimage(1,OX,OY,SX,SY,color1);

/* draw a line near the top of the image, slanting down to right */
  drawline(1,10,10,450,100,4,-1);

/* draw some text near bottom of image */
  drawtext(1,100,250,-1,"This is a text sample -- image 1");

  panicbutton=newbutton(image,460,300,5,"PANIC"); 
  quitbutton=newbutton(image,160,300,5,"QUIT");
  killmebutton=newbutton(image,460,170,7,"KILL ME");
  erasemebutton=newbutton(image,160,170,8,"ERASE ME");

  dispimage(image);
  while(1)
  {
    mousepos(image,xmouse,ymouse,xbutton);
    if(button = buttonhit(image,*xmouse,*ymouse))
    {   
      if(button == panicbutton)
        fprintf(stderr,"*** Panic button hit! ***\n Cause:  %s\n",
           dialoguebox(image,200,65,30,"What cause is there for panic?"));
      else if(button == quitbutton)
        exit(0);
      else if(button == killmebutton)
      {
        fprintf(stderr,"I'm melting!  I'm melting!\n");
        killbutton(button);
      }
      else if(button == erasemebutton)
      {
        fprintf(stderr,"I'm melting!  I'm melting!\n");
        erasebutton(button);
      }
    }
  }
}
