/* Radcart -- This program interpolates data from a single radar
 * onto a Cartesian grid.  The data are expected in the form
 * of a file with a single variable slice containing fields
 * as a function of range and raycnt.  The usual way to generate
 * this file is by running raduf on a universal format file, and
 * then running cdfcat on the sequence variable raycnt.  Finally
 * run cdfuniq to reduce the dimension of unvarying variables.
 */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"
#define  NEWPAR(A,B)    addpar(bufferw,HBMAX,A,B);
#define  PI             3.1415927
#define  RTODEG         (180./PI)
#define  DEGTOR         (PI/180.)
#define  DEGTOKM        111.17
#define  PPI            1
#define  RHI            3
#define  MAXFLDS        50
#define  MAXFANGLS      400
#define  FANGLTEST      0.01

char buffer[HBMAX][LINE]; /* buffer for header info */
char bufferw[HBMAX][LINE]; /* buffer for revised header */
float bad,badlim;         /* used w/bad data */
float *sbuff,*sbuffout,*vbuff,*vbuffout;    /* buffers for slice data */
long nsbuff,nsbuffout,nvbuff,nvbuffout;     /* sizes of above buffers */
float *fangl,*azim,*elev,*range;            /* pointers to input data */
float *xradar,*yradar,*zradar;              /* location of radar */
float *oldfptr[MAXFLDS];  /* pointers to old 2-d fields */
float *newfptr[MAXFLDS];  /* pointers to new 3-d fields */
float *idist;             /* pointer to interpolation distance field */
char pvalue[LINE];        /* paramater values */
char cl[LINE];            /* line for comment data */
float *din,*dout;         /* temporary pointers to fields */
struct field *fp;         /* pointer to field structure */

main(argc, argv)
int argc;
char *argv[];
{
  float xorg,yorg,zorg;     /* origin of new coord sys rel to radar */
  float dx,dy,dz;           /* increment values for each cartesian axis */
  float r,theta,phi,cotheta; /* spherical coords of cart grid point */
  float dr,dtheta,dphi,ds;  /* differences between cart and ray locations */
  float lowdtheta,highdtheta,lowdphi,highdphi;
  float domega,lowdomega,highdomega;
  float maxaz,minaz;        /* max/min azimuth */
  float maxel,minel;        /* max/min elevation */
  float maxr,minr;          /* man/min range */
  float azjump,domtest;     /* azimuth domain test temporaries */
  int dfangl[MAXFANGLS];    /* rays at start of new fixed angles */
  float fanglval[MAXFANGLS]; /* values of new fixed angles */
  int ndfangl;              /* number of new fixed angles */
  int bestfangl;            /* closest fixed angle index (to ndfangl) */
  int lowfangl,highfangl;   /* bounding indices */
  int lowray,highray;       /* bounding rays */
  int bestray,bestrange;    /* closest approximations to cart point */
  int ofields,nfields;      /* numbers of old and new 2-d and 3-d fields */
  int vpos,loop,loop1;      /* used for looping and testing */
  int nranges,nrays;        /* sizes of range and raycnt dimensions */
  int nx,ny,nz;             /* dimension size for new coord. system */
  int x,y,z;                /* indices for cycling thru x, y and z */
  long index3;              /* index for addressing 3-d fields */
  int swp;                  /* sweep mode */
  float x0,y0,z0;    /* initial bounds for new coordinate system */
  float x1,y1,z1;    /* point in new coordinate system rel to radar */
/* check argument list */
  if (argc != 13) {
    fprintf(stderr,"Usage: radcart x0 dx nx y0 dy ny z0 dz nz");
    fprintf(stderr," x_origin y_origin z_origin\n");
    exit(1);
  }
/* move stuff out of arguments */
  x0=atof(argv[1]);
  dx=atof(argv[2]);
  nx=atoi(argv[3]);
  y0=atof(argv[4]);
  dy=atof(argv[5]);
  ny=atoi(argv[6]);
  z0=atof(argv[7]);
  dz=atof(argv[8]);
  nz=atoi(argv[9]);
  xorg=atof(argv[10]);
  yorg=atof(argv[11]);
  zorg=atof(argv[12]);
/* get header buffer from standard input */
  gethdr(stdin,buffer,HBMAX);
/* if format is not float, die */
  if (getfmt(buffer,HBMAX) != 'f') {
    fprintf(stderr,"radcart: float format expected\n");
    exit(1);
  }
/* allocate input buffers */
  nsbuff = elemcnt(buffer,HBMAX,'s');
  sbuff = getbuff(nsbuff);
  nvbuff = elemcnt(buffer,HBMAX,'v');
  vbuff = getbuff(nvbuff);
/* make new header */
  nullhdr(bufferw,HBMAX,"float");
/* transfer header info */
  copycmt(bufferw,HBMAX,buffer,HBMAX);
  copypar(bufferw,HBMAX,buffer,HBMAX);
  copyfld(bufferw,HBMAX,buffer,HBMAX,'s');
/* add new index fields to the static slice */
  fp=getfld(buffer,HBMAX,'s',0);
  addfld(bufferw,HBMAX,'s',"x",fp->smul,fp->sadd,fp->bytes,1,"x",nx);
  addfld(bufferw,HBMAX,'s',"y",fp->smul,fp->sadd,fp->bytes,1,"y",ny);
  addfld(bufferw,HBMAX,'s',"z",fp->smul,fp->sadd,fp->bytes,1,"z",nz);
/* add radar location relative to origin */
  addfld(bufferw,HBMAX,'s',"xradar",1000.,0.,'l',0);
  addfld(bufferw,HBMAX,'s',"yradar",1000.,0.,'l',0);
  addfld(bufferw,HBMAX,'s',"zradar",1000.,0.,'l',0);
/* add new paramaters */
  NEWPAR("x0",argv[1]);
  NEWPAR("dx",argv[2]);
  NEWPAR("y0",argv[4]);
  NEWPAR("dy",argv[5]);
  NEWPAR("z0",argv[7]);
  NEWPAR("dz",argv[8]);
/* add comment lines */
  sprintf(cl,"%s: %s %s %s %s\n",argv[0],argv[1],argv[2],argv[3],argv[4]);
  addcline(bufferw,HBMAX,cl);
  sprintf(cl," %s %s %s %s\n",argv[5],argv[6],argv[7],argv[8]);
  addcline(bufferw,HBMAX,cl);
  sprintf(cl," %s %s %s %s\n",argv[9],argv[10],argv[11],argv[12]);
  addcline(bufferw,HBMAX,cl);
/* cycle thru the variable fields, transferring the 0 and 1 dim fields */
  vpos=0;
  ofields = 0;
  while ((fp=getfld(buffer,HBMAX,'v',vpos++)) != NULL) {
    if ((fp->dim==0)||(fp->dim==1)) {
      addfld(bufferw,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,
             fp->dim,fp->dname1,fp->dsize1);
    }
/* two dim fields are destined to be interpolated to 3-d */
    else if (fp->dim==2) {
      addfld(bufferw,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,3,
             "x",nx,"y",ny,"z",nz);
/* find pointers to these fields in input buffer */
      oldfptr[ofields++] = getptr(buffer,HBMAX,vbuff,'v','d',fp->fname);
    }
/* unexpected fields of higher dimensionality */
    else {
      fprintf(stderr,"radcart: 3 or 4 dimensional field on input\n");
      exit(1);
    }
  }
/* add a field for interpolation distance */
  addfld(bufferw,HBMAX,'v',"idist",100.,0.,'s',3,"x",nx,"y",ny,"z",nz);
/* write out the new header */
  puthdr(stdout,bufferw,HBMAX);
/* allocate the output slice buffers */
  nsbuffout = elemcnt(bufferw,HBMAX,'s');
  sbuffout = getbuff(nsbuffout);
  nvbuffout = elemcnt(bufferw,HBMAX,'v');
  vbuffout = getbuff(nvbuffout);
/* define pointers to the new 3-d fields */
  vpos = 0;
  nfields = 0;
  while ((fp=getfld(bufferw,HBMAX,'v',vpos++)) != NULL) {
    if (fp->dim == 3) {
      newfptr[nfields++] = getptr(bufferw,HBMAX,vbuffout,'v','d',fp->fname);
    }
  }
/* interpolation distance gets its own pointer */
  idist = newfptr[nfields - 1];
/* find values for bad and badlim */
  if (seekpar(buffer,HBMAX,"bad",pvalue)==FAIL) bad=BAD;
  else bad=atof(pvalue);
  if (seekpar(buffer,HBMAX,"badlim",pvalue)==FAIL) badlim=BADLIM;
  else badlim=atof(pvalue);
/* read the static slice */
  getslice(stdin,nsbuff,sbuff);
/* add the new index field info to the output slice buffer */
  dout=getptr(bufferw,HBMAX,sbuffout,'s','d',"x");
  for (loop=0;loop<nx;loop++) dout[loop]=x0+loop*dx;
  dout=getptr(bufferw,HBMAX,sbuffout,'s','d',"y");
  for (loop=0;loop<ny;loop++) dout[loop]=y0+loop*dy;
  dout=getptr(bufferw,HBMAX,sbuffout,'s','d',"z");
  for (loop=0;loop<nz;loop++) dout[loop]=z0+loop*dz;
/* add radar locations */
  xradar=getptr(bufferw,HBMAX,sbuffout,'s','d',"xradar");
  *xradar = -xorg;
  yradar=getptr(bufferw,HBMAX,sbuffout,'s','d',"yradar");
  *yradar = -yorg;
  zradar=getptr(bufferw,HBMAX,sbuffout,'s','d',"zradar");
  *zradar = -zorg;
/* transfer the old static slice data */
  vpos=0;
  while ((fp=getfld(buffer,HBMAX,'s',vpos++)) != NULL) {
    dout=getptr(bufferw,HBMAX,sbuffout,'s','d',fp->fname);
    for (loop=0;loop<fp->tsize;loop++) dout[loop]=sbuff[fp->ptr+loop];
  }
/* output the static slice */
  putslice(stdout,nsbuffout,sbuffout);
/* input the variable slice */
  getslice(stdin,nvbuff,vbuff);
/* transfer the old variable slice data */
  vpos=0;
  while ((fp=getfld(buffer,HBMAX,'v',vpos++)) != NULL) {
    if (fp->dim <= 1) {
      dout=getptr(bufferw,HBMAX,vbuffout,'v','d',fp->fname);
      for (loop=0;loop<fp->tsize;loop++) dout[loop]=vbuff[fp->ptr+loop];
    }
  }
/* define pointers for range, azimuth, and elevation */
  range=getptr2(buffer,HBMAX,sbuff,'s','d',"range",&fp);
  nranges = fp->tsize;
  azim=getptr2(buffer,HBMAX,vbuff,'v','d',"azim",&fp);
  nrays = fp->dsize1;
  elev=getptr(buffer,HBMAX,vbuff,'v','d',"elev");
/* find the sweep mode and fixed angle array */
  if ((fp=seekfld(buffer,HBMAX,'v',"sweepm"))==NULL) {
    fprintf(stderr,"radcart: sweep mode field not found\n");
    exit(1);
  }
  swp=vbuff[fp->ptr];
  if ((swp != PPI) && (swp != RHI)) {
    fprintf(stderr,"radcart: unsupported sweep mode %d\n",swp);
    exit(1);
  }
  if ((fangl = getptr(buffer,HBMAX,vbuff,'v','r',"fangl")) == NULL) {
    fprintf(stderr,"radcart: fixed angle field not found\n");
    exit(1);
  }
/* find the max and min ranges */
  minr = range[0];
  maxr = range[nranges - 1];
/* find the max and min angles for elevation */
  minel = 90.;
  maxel = 0.;
  for (loop=0;loop<nrays;loop++) {
    if (maxel < elev[loop]) maxel = elev[loop];
    if (minel > elev[loop]) minel = elev[loop];
  }
/* make azimuth and fixed angle monotonic and make the two commensurate*/
  for (loop=1;loop<nrays;loop++) {
    azjump=azim[loop]-azim[loop-1];
    if (azjump < -180.) azim[loop] += 360.;
    if (azjump > 180.) azim[loop] -= 360.;
    if (swp == RHI) {
      azjump=fangl[loop]-azim[loop];
      if (azjump < -180.) fangl[loop] += 360.;
      if (azjump > 180.) fangl[loop] -= 360.;
      azjump=fangl[loop]-azim[loop];
      if (azjump < -180.) fangl[loop] += 360.;
      if (azjump > 180.) fangl[loop] -= 360.;
    }
  }
/* find min and max azimuth */
  minaz = 720.;
  maxaz = -360.;
  for (loop=0;loop<nrays;loop++) {
    if (maxaz < azim[loop]) maxaz = azim[loop];
    if (minaz > azim[loop]) minaz = azim[loop];
  }
/* if azimuths exceed 360, subtract 360 */
  if (maxaz > 360.) {
    minaz -= 360.;
    maxaz -= 360.;
    for (loop=0;loop<nrays;loop++) {
      azim[loop] -= 360.;
      if (swp == RHI) fangl[loop] -= 360.;
    }
  }
/* azimuths are now presumably monotonic and in the range [-360,360] */
/* find rays that are at start of new fixed angles */
  dfangl[0] = 0;
  fanglval[0] = fangl[0];
  ndfangl = 1;
  for (loop = 1; loop < nrays; loop++) {
    if (fabs(fangl[loop] - fangl[loop - 1]) > FANGLTEST) {
      fanglval[ndfangl] = fangl[loop];
      dfangl[ndfangl++] = loop;
      if (ndfangl >= MAXFANGLS - 1) {
        fprintf(stderr,"radcart: too many fixed angle changes\n");
        exit(1);
      }
    }
  }
  dfangl[ndfangl] = nrays;
/* cycle thru the new coordinate system */
  for (z=0;z<nz;z++) {
    for (y=0;y<ny;y++) {
      for (x=0;x<nx;x++) {
        index3 = z + nz*y + nz*ny*x;
/* compute position of desired point relative to radar location */
        x1=x0+dx*x+xorg;
        y1=y0+dy*y+yorg;
        z1=z0+dz*z+zorg;
/* compute the linear offset from the beginning of the field */
        index3=z+y*nz+x*ny*nz;
/* r is the range of the new point */
        r = sqrt(x1*x1 + y1*y1 + z1*z1);
/* theta is the elevation angle at which the new point is located */
        if ((x1==0.)&&(y1==0.)&&(z1==0.)) theta=0.;
        else theta = RTODEG*atan2(z1,sqrt(x1*x1+y1*y1));
        cotheta = cos(theta*DEGTOR);
/* phi is the azimuth angle at which the new point is located */
        if ((x1==0.)&&(y1==0.)) phi=0.;
        else phi = RTODEG*atan2(x1,y1);
/* put phi in correct 360 degree domain for comparing with azimuth */
        domtest = phi - (minaz + maxaz)/2.;
        if (domtest > 180.) phi -= 360.;
        else if (domtest < -180.) phi += 360.;
/* does the point exceed the limits on the data? */
/* if yes then set that point to bad */
        if ((r < minr) || (r > maxr) || (theta < minel) ||
             (theta > maxel) || (phi < minaz) || (phi > maxaz)) {
          for (loop = 0; loop < nfields; loop++) {
            *(newfptr[loop] + index3) = bad;
          }
        }
/* cartesian point is within scanned sector */
        else {
/* find the magic ray */
/* ppi scans first */
          if (swp == PPI) {
/* compare elevation with fixed angle */
            dtheta = 90.;
            bestfangl = 0;
            for (loop = 0; loop < ndfangl; loop++) {
              if (fabs(theta - fanglval[loop]) < dtheta) {
                dtheta = fabs(theta - fanglval[loop]);
                bestfangl = loop;
              }
            }
/* within that fixed angle, get best azimuth */
            dphi = 90.;
            bestray = 0;
            for (loop=dfangl[bestfangl];loop<dfangl[bestfangl+1];loop++) {
              if (fabs(phi - azim[loop]) < dphi) {
                dphi = fabs(phi - azim[loop]);
                bestray = loop;
              }
            }
            dtheta = fabs(theta - elev[bestray]);
            domega = cotheta*dphi;
            domega = domega*domega + dtheta*dtheta;
/* within next lower fixed angle, get best azimuth */
            lowfangl = bestfangl - 1;
            if (lowfangl < 0) lowfangl = 0;
            lowdphi = 90.;
            lowray = 0;
            for (loop=dfangl[lowfangl];loop<dfangl[lowfangl+1];loop++) {
              if (fabs(phi - azim[loop]) < lowdphi) {
                lowdphi = fabs(phi - azim[loop]);
                lowray = loop;
              }
            }
            lowdtheta = fabs(theta - elev[lowray]);
            lowdomega = cotheta*lowdphi;
            lowdomega = lowdomega*lowdomega + lowdtheta*lowdtheta;
/* within next higher fixed angle, get best azimuth */
            highfangl = bestfangl + 1;
            if (highfangl >= ndfangl) highfangl = ndfangl - 1;
            highdphi = 90.;
            highray = 0;
            for (loop=dfangl[highfangl];loop<dfangl[highfangl+1];loop++) {
              if (fabs(phi - azim[loop]) < highdphi) {
                highdphi = fabs(phi - azim[loop]);
                highray = loop;
              }
            }
            highdtheta = fabs(theta - elev[highray]);
            highdomega = cotheta*highdphi;
            highdomega = highdomega*highdomega + highdtheta*highdtheta;
/* of the three, find the smallest projected angle difference */
            if (lowdomega < domega) {
              bestray = lowray;
              dphi = lowdphi;
              dtheta = lowdtheta;
            }
            if (highdomega < domega) {
              bestray = highray;
              dphi = highdphi;
              dtheta = highdtheta;
            }
          }
/* rhi scans next */
          else {
/* compare azimuth with fixed angle */
            dphi = 90.;
            bestfangl = 0;
            for (loop = 0; loop < ndfangl; loop++) {
              if (fabs(phi - fanglval[loop]) < dphi) {
                dphi = fabs(phi - fanglval[loop]);
                bestfangl = loop;
              }
            }
            domega = cotheta*dphi;
            domega = domega*domega + dtheta*dtheta;
/* within that fixed angle, get best elevation */
            dtheta = 90.;
            bestray = 0;
            for (loop=dfangl[bestfangl];loop<dfangl[bestfangl+1];loop++) {
              if (fabs(theta - elev[loop]) < dtheta) {
                dtheta = fabs(theta - elev[loop]);
                bestray = loop;
              }
            }
            dphi = fabs(phi - azim[bestray]);
/* within next lower fixed angle, get best elevation */
            lowfangl = bestfangl - 1;
            if (lowfangl < 0) lowfangl = 0;
            lowdtheta = 90.;
            lowray = 0;
            for (loop=dfangl[lowfangl];loop<dfangl[lowfangl+1];loop++) {
              if (fabs(theta - elev[loop]) < lowdtheta) {
                lowdtheta = fabs(theta - elev[loop]);
                lowray = loop;
              }
            }
            lowdphi = fabs(phi - azim[lowray]);
            lowdomega = cotheta*lowdphi;
            lowdomega = lowdomega*lowdomega + lowdtheta*lowdtheta;
/* within next higher fixed angle, get best elevation */
            highfangl = bestfangl + 1;
            if (highfangl >= ndfangl) highfangl = ndfangl - 1;
            highdtheta = 90.;
            highray = 0;
            for (loop=dfangl[highfangl];loop<dfangl[highfangl+1];loop++) {
              if (fabs(theta - elev[loop]) < highdtheta) {
                highdtheta = fabs(theta - elev[loop]);
                highray = loop;
              }
            }
            highdphi = fabs(phi - azim[highray]);
            highdomega = cotheta*highdphi;
            highdomega = highdomega*highdomega + highdtheta*highdtheta;
/* of the three, find the smallest projected angle */
            if (lowdomega < domega) {
              bestray = lowray;
              dtheta = lowdtheta;
              dphi = lowdphi;
            }
            if (highdomega < domega) {
              bestray = highray;
              dtheta = highdtheta;
              dphi = highdphi;
            }
          }
/* now find the best range value */
          dr = 100.;
          bestrange = 0;
          for (loop = 0; loop < nranges; loop++) {
            if (fabs(r - range[loop]) < dr) {
              dr = fabs(r - range[loop]);
              bestrange = loop;
            }
          }
/* find the linear distance between cartesian point and best ray/range val */
          dtheta *= DEGTOR;
          dphi *= DEGTOR;
          ds = cotheta*dphi;
          ds = r*r*(ds*ds + dtheta*dtheta) + dr*dr;
          ds = sqrt(ds);
/* transfer data from the ray/range grid to the cartesian grid */
          for (loop = 0; loop < ofields; loop++) {
            *(newfptr[loop] + index3) =
                 *(oldfptr[loop] + bestrange + nranges*bestray);
          }
/* fill in the idist field */
          idist[index3] = ds;
        }
/* finished with this cartesian point */
      }
    }
  }
/* done! put the variable slice */
  putslice(stdout,nvbuffout,vbuffout);
  exit(0);
}
