/* Radray -- This program extracts rays of radar data from CEDRIC-synthesized
 * data.  The radial velocity along the ray is computed.
 */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"
#define  newpar(A,B)    addpar(bufferw,HBMAX,A,B);
#define  PI             3.1415927
int loc[9];                  /* defines "unit cell" */
char buffer[HBMAX][LINE];    /* buffer for header info */
char bufferw[HBMAX][LINE];   /* buffer for revised header */
float *sbuff,*sbuffout;      /* buffers for slice data */
float *xl,*yl,*zl;           /* buffers for interpolation */
float dx,dy,dz;              /* increment size of x y and z */
float bad,badlim;            /* used w/bad data */
float lineprox();            /* linear approx routine */
float triline();             /* obtain point on ray */

main(argc, argv)
int argc;
char *argv[];
{
  int vpos,loop,loop1;      /* used for looping and testing */
  int num,nr,flag,flagb;    /* storage of values */
  int xu,yu,zu;             /* coord. for lower-left corner of unit cell */
  int nx,ny,nz;             /* number of elements in each dimension */
  long nel,nelw;            /* number of elements to read and write */
  char pvalue[LINE];        /* paramater values */
  char cl[LINE];            /* line for comment data */
  char uname[LINE],vname[LINE],wname[LINE]; /* name of wind velocity fields */
  char xname[LINE],yname[LINE],zname[LINE]; /* name of x y and z dimensions */
  float xi,yi,zi,value;     /* used in approxamation */
  float xorg,yorg,zorg;     /* coord. for the orgin of the new grid */
  float x0,y0,z0;           /* the start of the data grid */
  float xm,ym,zm;           /* maximun values for x y and z */
  float r0,dr;              /* info about range */
  float azimuth,elev;        /* elevation and azimuth form command line */
  float phi,theta,rho;      /* temp storage */
  float *din,*dout;         /* pointer to start of data */
  float *u,*v,*w;           /* buffers for wind velocity data */
  struct field *fp;         /* pointer to the field structure of data */

/* check argument list */
  if (argc != 12) {
    fprintf(stderr,"Usage: radray x y z azimuth elev r0 dr nr");
    fprintf(stderr," uname vname wname\n");
    exit(1);
  }
/* move stuff out of arguments */
  xorg=atof(argv[1]);
  yorg=atof(argv[2]);
  zorg=atof(argv[3]);
  azimuth=atof(argv[4]);
  azimuth=90.-azimuth;
  if (azimuth > 360.)
    azimuth -= 360.;
  else if (azimuth < 0.)
    azimuth += 360.;
  elev=atof(argv[5]);
  if ((elev > 90.)||(elev < -90.)) {
    fprintf(stderr,"radray: elevation must be between -90 and 90 degrees\n");
    exit(1);
  }
  r0=atof(argv[6]);
  dr=atof(argv[7]);
  nr=atoi(argv[8]);
  strcpy(uname,argv[9]);
  strcpy(vname,argv[10]);
  strcpy(wname,argv[11]);
/* get header buffer from standard input */
  gethdr(stdin,buffer,HBMAX);
/* if format is not float, die */
  if (getfmt(buffer,HBMAX) != 'f') {
    fprintf(stderr,"radcart: must be in float format!\n");
    exit(1);
  }
/* check validity of u v and w fields */
  flag=0;
  if (((fp=seekfld(buffer,HBMAX,'v',uname))==NULL)||(fp->dim != 3))
    flag++;
  if (((fp=seekfld(buffer,HBMAX,'v',vname))==NULL)||(fp->dim != 3))
    flag++;
  if (((fp=seekfld(buffer,HBMAX,'v',wname))==NULL)||(fp->dim != 3))
    flag++;
  if (flag > 0) {
    fprintf(stderr,"radray: u v or w field invalid or nonexistent\n");
    exit(0);
  }
/* make new header */
  nullhdr(bufferw,HBMAX,"float");
/* transfer header info */
  copycmt(bufferw,HBMAX,buffer,HBMAX);
  copypar(bufferw,HBMAX,buffer,HBMAX);
  sprintf(cl,"%f",r0);
  addpar(bufferw,HBMAX,"range0",cl);
  sprintf(cl,"%f",dr);
  addpar(bufferw,HBMAX,"drange",cl);
/* obtain the values for bad and badlim */
  if (seekpar(buffer,HBMAX,"bad",pvalue)==FAIL)
    bad=BAD;
  else
    bad=atof(pvalue);
  if (seekpar(buffer,HBMAX,"badlim",pvalue)==FAIL)
    badlim=BADLIM;
  else
    badlim=atof(pvalue);
/* find info about x y and z */
  vpos=flag=0;
  while ((fp=getfld(buffer,HBMAX,'v',vpos++))!=NULL) {
    if (fp->dim==3) {
      strcpy(xname,fp->dname1);
      nx=fp->dsize1;
      strcpy(yname,fp->dname2);
      ny=fp->dsize2;
      strcpy(zname,fp->dname3);
      nz=fp->dsize3;
      flag++;
      break;
    }
  }
  if (flag==0) {
    fprintf(stderr,"radray: file doen't contain 3d fields\n");
    exit(1);
  }
  x0=y0=z0=0.0;
  sprintf(cl,"%s0",xname);
  if (seekpar(buffer,HBMAX,cl,pvalue)!=FAIL)
    x0=atof(pvalue);
  sprintf(cl,"%s0",yname);
  if (seekpar(buffer,HBMAX,cl,pvalue)!=FAIL)
    y0=atof(pvalue);
  sprintf(cl,"%s0",zname);
  if (seekpar(buffer,HBMAX,cl,pvalue)!=FAIL)
    z0=atof(pvalue);
  sprintf(cl,"d%s",xname);
  seekpar(buffer,HBMAX,cl,pvalue);
    dx=atof(pvalue);
  sprintf(cl,"d%s",yname);
  seekpar(buffer,HBMAX,cl,pvalue);
    dy=atof(pvalue);
  sprintf(cl,"d%s",zname);
  seekpar(buffer,HBMAX,cl,pvalue);
    dz=atof(pvalue);
  xm=x0+nx*dx;
  ym=y0+ny*dy;
  zm=z0+nz*dz;
/* add comment line */
  sprintf(cl,"radray: %s %s %s\n",argv[1],argv[2],argv[3]);
  addcline(bufferw,HBMAX,cl);
  sprintf(cl,"  %s %s %s %s %d\n",argv[4],argv[5],argv[6],argv[7],nr);
  addcline(bufferw,HBMAX,cl);
  sprintf(cl,"  %s %s %s\n",uname,vname,wname);
  addcline(bufferw,HBMAX,cl);
/* add index field to static area */
  seekfld(buffer,HBMAX,'s',xname);
  addfld(bufferw,HBMAX,'s',"range",fp->smul,fp->sadd,fp->bytes,1,"range",nr);
/* update the variable area */
  vpos=0;
  while ((fp=getfld(buffer,HBMAX,'v',vpos++))!=NULL) {
    if (fp->dim == 0) {
      addfld(bufferw,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,0);
      continue;
    }
    if ((strcmp(fp->fname,uname)==0)||(strcmp(fp->fname,vname)==0)||
         (strcmp(fp->fname,wname)==0)||(fp->dim != 3)) {
      continue;
    }
    addfld(bufferw,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,
           1,"range",nr);
  }
  addfld(bufferw,HBMAX,'v',"vrad",1000.0,.0,'s',1,"range",nr);
/* output the new header */
  puthdr(stdout,bufferw,HBMAX);
/* get static slice */
  nel=elemcnt(buffer,HBMAX,'s');
  nelw=elemcnt(bufferw,HBMAX,'s');
  sbuff=getbuff(nel);
  getslice(stdin,nel,sbuff);
  sbuffout=getbuff(nelw);
/* place data in the static slice */
  for (loop=0;loop<nr;loop++)
    sbuffout[loop]=r0+loop*dr;
/* send static slice out */
  putslice(stdout,nelw,sbuffout);
  free(sbuffout);
  free(sbuff);
/* get the variable slice there should only be one */
  nel=elemcnt(buffer,HBMAX,'v');
  nelw=elemcnt(bufferw,HBMAX,'v');
  sbuff=getbuff(nel);
  sbuffout=getbuff(nelw);
  getslice(stdin,nel,sbuff);
/* set aside memory for data buffers */
  u=getbuff(nr);
  v=getbuff(nr);
  w=getbuff(nr);
/* find the locations of the x y and z points along the ray */
  xl=getbuff(nr);
  yl=getbuff(nr);
  zl=getbuff(nr);
  for (loop=0;loop<nr;loop++) {
    phi=(90.-elev)*PI/180.;
    rho=r0+loop*dr;
    xl[loop]=rho*sin(phi)*cos(azimuth*PI/180.)+xorg;
    yl[loop]=rho*sin(phi)*sin(azimuth*PI/180.)+yorg;
    zl[loop]=rho*cos(phi)+zorg;
  }
/* cycle through the variable data and extract the ray needed */
  vpos=0;
  while ((fp=getfld(buffer,HBMAX,'v',vpos++))!=NULL) {
    if ((fp->dim == 1)||(fp->dim == 2))
      continue;
/* is the field one of the wind velocity fields? If so set flag */
    flag=0;
    if (strcmp(fp->fname,uname)==0)
      flag=1;
    else if (strcmp(fp->fname,vname)==0)
      flag=2;
    else if (strcmp(fp->fname,wname)==0)
      flag=3;
/* set din and dout to where to get data and where to send info */
    din=getptr(buffer,HBMAX,sbuff,'v','d',fp->fname);
    dout=getptr(bufferw,HBMAX,sbuffout,'v','r',fp->fname);
/* is the field a scalar then send it without change */
    if (fp->dim == 0) {
      dout[0]=din[0];
      continue;
    }
/* otherwise loop through the data and extract the ray */
/* loop along the range of the ray */
    for (loop=0;loop<nr;loop++) {
/* if the point is outside of domain of the data then set to bad */
      if ((xl[loop]<x0)||(xl[loop]>xm)||(yl[loop]<y0)||(yl[loop]>ym)||
           (zl[loop]<z0)||(zl[loop]>zm)) {
        if (flag==0)
          dout[loop]=bad;
        else if (flag==1)
          u[loop]=bad;
        else if (flag==2)
          v[loop]=bad;
        else if (flag==3)
          w[loop]=bad;
      }
/* otherwise, extract point */
      else {
        xu=(xl[loop]-x0)/dx;
        yu=(yl[loop]-y0)/dy;
        zu=(zl[loop]-z0)/dz;
        xi=xu*dx+x0;
        yi=yu*dy+y0;
        zi=zu*dz+z0;
/* define the edges of the "unit cell" */
        loc[1]=xu*ny*nz+yu*nz+zu;
        loc[2]=loc[1]+1;
        loc[3]=loc[1]+nz;
        loc[4]=loc[3]+1;
        loc[5]=loc[1]+ny*nz;
        loc[6]=loc[5]+1;
        loc[7]=loc[5]+nz;
        loc[8]=loc[7]+1;
/* obtain the value of the point on the ray */
        value=triline(loop,xi,yi,zi,fp->ptr);
/* place point in the outfile unless u v or w */
        if (flag==0)
          dout[loop]=value;
        else if (flag==1)
          u[loop]=value;
        else if (flag==2)
          v[loop]=value;
        else if (flag==3)
          w[loop]=value;
      }
    }
  }
/* define vector pointing in ray direction */
  xi=xl[1] - xl[0];
  yi=yl[1] - yl[0];
  zi=zl[1] - zl[0];
  dout=getptr(bufferw,HBMAX,sbuffout,'v','d',"vrad");
  for (loop=0;loop<nr;loop++) {
    if ((fabs(u[loop])>badlim)||(fabs(v[loop])>badlim)||
         (fabs(w[loop])>badlim)) {
      dout[loop]=bad;
    }
    else {
/* get velocity component in ray direction */
       value=(xi*u[loop]+yi*v[loop]+zi*w[loop]);
       dout[loop]=value/sqrt(xi*xi+yi*yi+zi*zi);
    }
  }
  putslice(stdout,nelw,sbuffout);
  exit(0);
}

/* rountine for getting the point on the ray (tri-linear approx) */
/* uses linear approximation routine */
float triline(pos,xcell,ycell,zcell,pt)
int pos,pt;
float xcell,ycell,zcell;
{
  float lv1,lv2,lvx;       /* points in the linear approx */
  float lva,lvb;           /* same */

  lv1=lineprox(sbuff[pt+loc[1]],sbuff[pt+loc[2]],zcell,zl[pos],dz);
  lv2=lineprox(sbuff[pt+loc[3]],sbuff[pt+loc[4]],zcell,zl[pos],dz);
  lva=lineprox(lv1,lv2,ycell,yl[pos],dy);
  lv1=lineprox(sbuff[pt+loc[5]],sbuff[pt+loc[6]],zcell,zl[pos],dz);
  lv2=lineprox(sbuff[pt+loc[7]],sbuff[pt+loc[8]],zcell,zl[pos],dz);
  lvb=lineprox(lv1,lv2,ycell,yl[pos],dy);
  lvx=lineprox(lva,lvb,xcell,xl[pos],dx);
  return(lvx);
}

/* linear approximation routine */
float lineprox(va,vb,a,m,d)
float va,vb;     /* values at the ends */
float a,m,d;     /* points at the ends and point between the others */
{
  float linvalue;

  if ((fabs(va)>badlim)||(fabs(vb)>badlim)||(fabs(m)>badlim))
    return(bad);
  linvalue=(((m-a)*(vb-va))/(d))+va;
  return(linvalue);
}
