/* radrp7 -- This program reads NCAR RP-7 field format tapes and
 * converts them to Candis format.  It is assumed that the tape was
 * read to disk using the Candis utility rtape.  The output is in the
 * form of one ray per variable slice.
 */

#include <stdio.h>
#include <math.h>
#include "rp7.h"
#include "cdfhdr.h"

#define MAXRAYS 40000

/* stuff for field format data */
struct rp7 header;
unsigned char *data;
unsigned char *getrp7();
float azlist[MAXRAYS],ellist[MAXRAYS],ctlist[MAXRAYS];

/* stuff for Candis */
char hbuff[HBMAX][LINE];          /* header buffer */
char cline[LINE],qline[LINE];     /* line buffers */
long nsbuff,nvbuff;               /* size of slice buffers */
float *sbuff,*vbuff;              /* slice buffer pointers */
float *fields[NFMAX];             /* field pointers */
float *fcount;                    /* record count */
float *year;                      /* scalars from field tape */
float *month;
float *day;
float *hour;
float *minute;
float *second;
float *azimuth;
float *elevation;
float *fangle;
float *altitude;
float *longitude;
float *latitude;

float *avgtrpow;
float *prf;

float *range;                     /* range index field pointer */
float range0,drange;              /* index parameter stuff */
int nr;                           /* number of gates */
FILE *infile;                     /* input file */
char *infname;                    /* input file name */
char *pgraf;                      /* pgraf routine name */
int count1;
long lenfbeg,recordlen;

main(argc,argv)
int argc;
char **argv;
{
  int loop,ir,count2,count,junk;
  char rtheader[9];
  extern long recordlen;
  float smul,sadd;
  int glitch;
  int bytes_expected,numseek;
  extern long lenfbeg;
  long firstrecl;

  /* check arguments */
  if (argc != 4 && argc != 5) {
    fprintf(stderr,"Usage: radrp7 filename count1 count2 [glitch_block]\n");
    exit(1);
  }
  infname = argv[1];
  count1 = atoi(argv[2]);
  count2 = atoi(argv[3]);
  if(argc == 5) glitch = atoi(argv[4]);
  count = count1;
  bytes_expected = 0;
  /* get pgraf routine to use */
  if ((pgraf = (char*)getenv("PG")) == NULL) {
    fprintf(stderr,"radrp7: you need to setenv PG to desired pgraf routine\n");
    exit(1);
  }

  /* open input file */
  if ((infile = fopen(infname,"r")) == NULL) {
    fprintf(stderr,"radrp7: can't open file %s\n",infname);
    exit(1);
  }

  /* get rtape header */
  for (loop = 0; loop < 8; loop++) rtheader[loop] = getc(infile);
  rtheader[8] = '\0';
  firstrecl = recordlen = atoi(rtheader) + 8;

  /* loop back for interaction */
loopback: 
  ;

  /* read a number and plot azimuth and elevation */
  loop = 0;
  header.year = 50;
  lenfbeg = firstrecl*count1;
  /* if there are no known glitches in file go straight to destination block */
  if(!(argc == 4 || count1 < glitch)){
    /* otherwise go to glitch block read new block size and continue */
    fseek(infile,(long)(glitch*firstrecl),0);
    lenfbeg = glitch*firstrecl;
    for (loop = 0; loop < 8; loop++) rtheader[loop] = getc(infile);
    rtheader[8] = '\0';
    recordlen = atoi(rtheader) + 8;
    /* now we have new record length go to destination */
    lenfbeg += (count1-glitch)*recordlen;
  }

  for (count = count1; count < count2; count += CNTSKIP) {
    /* position file pointer */
    fseek(infile,(long)lenfbeg,0);
    /* get a record */
    if ((data = getrp7(infile,'n',0,0,&header,count))
        == (unsigned char*)NULL) {
      /* glitch noticed in file: go back and find where the glitch occured *
     * if we have not been told already.                                 */
      /* rewind the file */
      fseek(infile,firstrecl,0);
      lenfbeg = firstrecl;
      numseek =0;
      while(numseek < count1){
        for (loop = 0; loop < 8; loop++) rtheader[loop] = getc(infile);
        rtheader[8] = '\0';
        if(recordlen != atoi(rtheader)+8) {
          fprintf(stderr,"change of block size in block %d\n",numseek+1);
          recordlen = atoi(rtheader)+8;
          glitch = numseek+1;
          /* trip glitch flag */
          argc = 5;
        }
        if(recordlen <= 0){
          fprintf(stderr,"unable to find reasonable record length for block ");
          fprintf(stderr," %d\n",numseek);
          exit(1);
        }
        lenfbeg += recordlen;
        fseek(infile,lenfbeg,0);
        numseek++;
      }
    }
    azlist[loop] = header.azimuth;
    ellist[loop] = header.elevation;
    ctlist[loop++] = count;
    lenfbeg += CNTSKIP*recordlen;
  }

  /* plot it */
  gopen();
  gclear();
  window(1,0.,15.,0.,5.);
  window(2,0.,15.,5.,10.);
  axes(1,(float)count1,(float)count2,11,"count",0.,360.,5,"azimuth");
  axes(2,(float)count1,(float)count2,11,"count",0.,90.,7,"elevation");
  line(1,1,loop,ctlist,azlist);
  line(2,1,loop,ctlist,ellist);
  if (header.year != 50) {
    sprintf(cline,"end time: %d:%d:%d",
        header.hour,header.minute,header.second);
    label(2,(float)count1,80.,cline);
    sprintf(cline,"date (dd:mm:yy): %d-%d-%d",
        header.day,header.month,header.year);
    label(2,(float)count1,70.,cline);
    sprintf(cline,"lon, lat, alt: %.4f %.4f %.0f",header.latitude,
        header.longitude,header.altitude);
    label(2,(float)count1,60.,cline);
    sprintf(cline,"radar id: %d",header.radar_id);
    label(2,(float)count1,50.,cline);
  }
  gpause();
  gclose();
  system(pgraf);

  /* interaction */
  fprintf(stderr,"Do you like %d to %d? (y/n) ",count1,count2);
  gets(cline);
  if (cline[0] != 'y') {
    fprintf(stderr,"So type new ones! ");
    while (!gets(cline)||sscanf(cline,"%d %d %d",&count1,&count2,&junk) != 2) {
      fprintf(stderr,"Huh? ");
    }
    goto loopback;
  }

  /* reposition file pointer and go */
  lenfbeg = firstrecl*count1;
  /* if there are no known glitches in file go straight to destination block */
  if(!(argc == 4 || count1 < glitch)){
    /* otherwise go to glitch block read new block size and continue */
    fseek(infile,(long)(glitch*firstrecl),0);
    lenfbeg = glitch*firstrecl;
    for (loop = 0; loop < 8; loop++) rtheader[loop] = getc(infile);
    rtheader[8] = '\0';
    recordlen = atoi(rtheader) + 8;
    /* now we have new record length go to destination */
    lenfbeg += (count1-glitch)*recordlen;
  }
  fseek(infile,lenfbeg,0);
  /* make header */
  nullhdr(hbuff,HBMAX,"float");
  sprintf(cline,"radrp7: %d %d\n",count1,count2);
  addcline(hbuff,HBMAX,cline);

  /* find range information */
  nr = header.num_gates;
  drange = header.delrho/1000.;

  /* the following is the range to the first gate as read off of the
tape, it is typically 0.0 */
  range0 = header.rhozero;

  /* add parameters */
  sprintf(qline,"%g",range0);
  addpar(hbuff,HBMAX,"range0",qline);
  sprintf(qline,"%g",drange);
  addpar(hbuff,HBMAX,"drange",qline);
  sprintf(qline,"%g",BAD);
  addpar(hbuff,HBMAX,"bad",qline);
  sprintf(qline,"%g",BADLIM);
  addpar(hbuff,HBMAX,"badlim",qline);

  /* add static fields */
  addfld(hbuff,HBMAX,'s',"range",1000.,0.,'l',1,"range",nr);

  /* add variable fields */
  for (loop = 0; loop < header.nfields; loop++) {
    smul = 100./header.scale[loop];
    sadd = -smul*header.bias[loop]/100.;
    addfld(hbuff,HBMAX,'v',header.fnames[loop],smul,sadd,'c',1,"range",nr);
  }
  addfld(hbuff,HBMAX,'v',"count",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"year",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"mon",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"day",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"hour",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"min",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"sec",1.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"azim",50.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"elev",50.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"fangle",50.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"altitude",1000.,0.,'s',0);
  addfld(hbuff,HBMAX,'v',"longitude",1000.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"latitude",1000.,0.,'l',0);

  addfld(hbuff,HBMAX,'v',"avgtrpow",1000.,0.,'l',0);
  addfld(hbuff,HBMAX,'v',"prf",1000.,0.,'l',0);

  /* write Candis header */
  puthdr(stdout,hbuff,HBMAX);

  /* create slice buffers */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  nvbuff = elemcnt(hbuff,HBMAX,'v');
  sbuff = getbuff(nsbuff);
  vbuff = getbuff(nvbuff);

  /* get pointers */
  range = getptr(hbuff,HBMAX,sbuff,'s','d',"range");
  for (loop = 0; loop < header.nfields; loop++) {
    fields[loop] = getptr(hbuff,HBMAX,vbuff,'v','d',header.fnames[loop]);
  }
  fcount = getptr(hbuff,HBMAX,vbuff,'v','d',"count");
  year = getptr(hbuff,HBMAX,vbuff,'v','d',"year");
  month = getptr(hbuff,HBMAX,vbuff,'v','d',"mon");
  day = getptr(hbuff,HBMAX,vbuff,'v','d',"day");
  hour = getptr(hbuff,HBMAX,vbuff,'v','d',"hour");
  minute = getptr(hbuff,HBMAX,vbuff,'v','d',"min");
  second = getptr(hbuff,HBMAX,vbuff,'v','d',"sec");
  azimuth = getptr(hbuff,HBMAX,vbuff,'v','d',"azim");
  elevation = getptr(hbuff,HBMAX,vbuff,'v','d',"elev");
  fangle = getptr(hbuff,HBMAX,vbuff,'v','d',"fangle");
  altitude = getptr(hbuff,HBMAX,vbuff,'v','d',"altitude");
  longitude = getptr(hbuff,HBMAX,vbuff,'v','d',"longitude");
  latitude = getptr(hbuff,HBMAX,vbuff,'v','d',"latitude");
  avgtrpow = getptr(hbuff,HBMAX,vbuff,'v','d',"avgtrpow");
  prf = getptr(hbuff,HBMAX,vbuff,'v','d',"prf");

  /* set up index field */
  for (loop = 0; loop < nr; loop++) {
    range[loop] = range0 + drange*loop;
  }

  /* put static slice */
  putslice(stdout,nsbuff,sbuff);

  /* loop on input rays */
  count = count1;
  while ((data = getrp7(infile,'y',0,0,&header,count))!= (unsigned char*)NULL) {
    for (ir = 0; ir < nr; ir++) {
      for (loop = 0; loop < header.nfields; loop++) {
        fields[loop][ir] = 0.01*(header.scale[loop]*
            ((float)data[loop + header.nfields*ir]) +
            header.bias[loop]);
        *fcount = count;
        *year = header.year;
        *month = header.month;
        *day = header.day;
        *hour = header.hour;
        *minute = header.minute;
        *second = header.second;
        *azimuth = header.azimuth;
        *elevation = header.elevation;
        *fangle = header.fixed_angle;
        *altitude = header.altitude/1000.;   /* change to kilometers */
        *longitude = -header.longitude;  /* probably negative (western hem) */
        *latitude = header.latitude;

        *avgtrpow = header.avgtrpow;
        *prf = header.prf;
      }
    }

    /* put variable slice */
    putslice(stdout,nvbuff,vbuff);

    /* jump out if count exceeds count2 */
    count++;
    if (count >= count2) break;
  }
  exit(0);
}
