/* radsource                                                     */
/* usage:  radsource time_name x_vel y_vel z_vel test_var        */
/* radsource accepts radar data in a common data file format and */
/* creates a file (called source) that contains the figured data */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

#define  ADD  5                /* number of aguments other than test_vars */

char buffer[HBMAX][LINE],bufferw[HBMAX][LINE];    /* buffers for header data */
float *sbuffa,*sbuffb;         /* buffers that contain slice & computed data */
float diff();
float avg();
float bd,bdl;
int whichdiff();

main(argc,argv)
int argc;
char *argv[];
{
  int flag,stat,pos;                 /* status and pos of pointers and flags */
  int posa,posb,posc;                /* indicated position of data points */
  int loop,loop1,vpos;               /* used in looping */
  int x,y,z,ds1,ds2,ds3;             /* used in cycling through dimensions */
  int ptf,ptu,ptv,ptw;               /* pointers to area that contains data */
  int numtest;                       /* number of test_var on command line */
  char time[LINE],uvel[LINE];        /* contains command line aquments */
  char vvel[LINE],wvel[LINE];        /*     "       "      "      "    */
  char xname[LINE],yname[LINE],zname[LINE]; /* contains names of dimensions */
  char dname[LINE],pvalue[LINE];     /* used in getting parameter data */
  char cl[LINE];                     /* comment line data */
  long nel,nelw;                     /* size of data to be read and written */
  float t1,t2;                       /* times for slices 1 and 2 */
  float dx,dy,dz,dt;                 /* used in computing the diff */
  float tf,uf,vf,wf;                 /* results from the diff routine */
  float avgf1,avgf2,avgf3;           /* contains averaged data */
  float avgv1,avgv2,avgv3;           /*     "        "      "  */
  struct field *fp;                  /* pointer to the field structure */

/* check command line arguments */
  if (argc < ADD+1) {
    fprintf(stderr,"Usage: radsource time_name x_vel y_vel z_vel ");
    fprintf(stderr,"test_var1 [test_var2 ...]\n");
    exit(1);
  }
  numtest = argc-ADD;
/* move stuff out of argument line */
  strcpy(time,argv[1]);
  strcpy(uvel,argv[2]);
  strcpy(vvel,argv[3]);
  strcpy(wvel,argv[4]);
/* get header from stdin */
  gethdr(stdin,buffer,HBMAX);
/* check format */
  if (getfmt(buffer,HBMAX) != 'f') {
    fprintf(stderr,"radsource: file must be float!\n");
    exit(1);
  }
/* check validity of given variables on the command line */
  flag=0;
/* check the velocity fields */
  fp=seekfld(buffer,HBMAX,'v',uvel);
  if ((fp==NULL)||(fp->dim != 3))
    flag=1;
  fp=seekfld(buffer,HBMAX,'v',vvel);
  if ((fp==NULL)||(fp->dim != 3))
    flag=1;
  fp=seekfld(buffer,HBMAX,'v',wvel);
  if ((fp==NULL)||(fp->dim != 3))
    flag=1;
  if (flag==1) {
    fprintf(stderr,"radsource: error with velocity fields, non-existant");
    fprintf(stderr," or not 3d fields\n");
    exit(1);
  }
/* check time field */
  fp=seekfld(buffer,HBMAX,'v',time);
  if ((fp==NULL)||(fp->dim != 0)) {
    fprintf(stderr,"radsource: error with time field\n");
    exit(1);
  }
/* check test_var fields */
  for (loop=0;loop<numtest;loop++) {
    fp=seekfld(buffer,HBMAX,'v',argv[loop+ADD]);
    if ((fp==NULL)||(fp->dim != 3)) {
      fprintf(stderr,"radsource: error with the test_var field");
      fprintf(stderr," %s\n",argv[loop+ADD]);
      exit(1);
    }
  }
/* make output header */
  nullhdr(bufferw,HBMAX,"float");
/* transfer comments */
  copycmt(bufferw,HBMAX,buffer,HBMAX);
/* add comment line (actually add two lines) 1st part contains all of the */
/* required arguments and the 2nd line contains all the test_var names */
  sprintf(cl,"radsource: ");
  for (loop=1;loop<ADD;loop++) {
    strcat(cl,argv[loop]);
    strcat(cl," ");
  }
  strcat(cl,"\n");
  addcline(bufferw,HBMAX,cl);
  sprintf(cl,"           ");
  for (loop=ADD;loop<argc;loop++) {
    strcat(cl,argv[loop]);
    strcat(cl," ");
  }
  strcat(cl,"\n");
  addcline(bufferw,HBMAX,cl);
/* transfer parameters */
  copypar(bufferw,HBMAX,buffer,HBMAX);
/* transfer static fields */
  copyfld(bufferw,HBMAX,buffer,HBMAX,'s');
/* transfer variable fields */
  copyfld(bufferw,HBMAX,buffer,HBMAX,'v');
  fp=seekfld(buffer,HBMAX,'v',uvel);
  for (loop=0;loop<numtest;loop++) {
    sprintf(cl,"s%s",argv[loop+ADD]);
    addfld(bufferw,HBMAX,'v',cl,fp->smul,fp->sadd,fp->bytes,fp->dim,
       fp->dname1,fp->dsize1,fp->dname2,fp->dsize2,fp->dname3,fp->dsize3);
  }
  fp=seekfld(buffer,HBMAX,'v',time);
  addfld(bufferw,HBMAX,'v',"deltime",fp->smul,fp->sadd,fp->bytes,fp->dim);
/* get x, y and z names for later reference and retrevial */
  strcpy(xname,fp->dname1);
  strcpy(yname,fp->dname2);
  strcpy(zname,fp->dname3);
/* send updated header to stdout */
  puthdr(stdout,bufferw,HBMAX);
/* find dx, dy and dz */
  sprintf(dname,"d%s",xname);
  seekpar(buffer,HBMAX,dname,pvalue);
  dx=atof(pvalue);
  sprintf(dname,"d%s",yname);
  seekpar(buffer,HBMAX,dname,pvalue);
  dy=atof(pvalue);
  sprintf(dname,"d%s",zname);
  seekpar(buffer,HBMAX,dname,pvalue);
  dz=atof(pvalue);
/* determine values for bad and badlim */
  if (seekpar(buffer,HBMAX,"bad",pvalue)==FAIL)
    bd=BAD;
  else
    bd=atof(pvalue);
  if (seekpar(buffer,HBMAX,"badlim",pvalue)==FAIL)
    bdl=BADLIM;
  else
    bdl=atof(pvalue);
/* allocate memory for slice buffer */
  nel=elemcnt(buffer,HBMAX,'s');
  sbuffa=getbuff(nel);
  getslice(stdin,nel,sbuffa);
/* send static slice to stdout w/o any changes */
  putslice(stdout,nel,sbuffa);
/* set up buffers for variable slices */
  free(sbuffa);
  nel=elemcnt(buffer,HBMAX,'v');
  nelw=elemcnt(bufferw,HBMAX,'v');
  sbuffa=getbuff(nelw);
  sbuffb=getbuff(nelw);
/* set pointers to the begining of data for each vel. field */
/* and length of dimensions to cycle through */
  fp=seekfld(buffer,HBMAX,'v',uvel);
  ptu=fp->ptr;
  fp=seekfld(buffer,HBMAX,'v',vvel);
  ptv=fp->ptr;
  fp=seekfld(buffer,HBMAX,'v',wvel);
  ptw=fp->ptr;
  ds1=fp->dsize1;
  ds2=fp->dsize2;
  ds3=fp->dsize3;
/* getslice */
  getslice(stdin,nel,sbuffa);
  stat=getslice(stdin,nel,sbuffb);
  while (stat != EOF) {
/* find the value of time and dt */
    fp=seekfld(buffer,HBMAX,'v',time);
    t1=sbuffa[fp->ptr];
    t2=sbuffb[fp->ptr];
    dt=t2-t1;
/* loop through slices and average data in consecutive times */
    vpos=0;
    while (vpos<nel) {
      avgf1=avg(sbuffa[vpos],sbuffb[vpos]);
      sbuffa[vpos++]=avgf1;
    }
/* cycle through test_var names and treat each one */
    for (loop=0;loop<numtest;loop++) {
      fp=seekfld(buffer,HBMAX,'v',argv[loop+ADD]);
      ptf=fp->ptr;
/* cycle though data */
      for (x=0;x<ds1;x++) {
        for (y=0;y<ds2;y++) {
          for (z=0;z<ds3;z++) {
/* compute the equation for df/dt d(uf)/dx d(vf)/dy and d(wf)/dz */
            pos=z+y*ds3+x*ds3*ds2;
/* find the partial of f with respect to t (note that the equation is */
/* slightly modified in order to compensate for the averaging process) */
            if ((fabs(sbuffb[ptf+pos])>bdl)||(fabs(sbuffa[ptf+pos])>bdl))
              tf = bd;
            else
              tf=((2.0*sbuffb[ptf+pos]-2.0*sbuffa[ptf+pos]))/dt;
/* find the partial of uf with respect to x */
           flag=whichdiff(x,ds1);
           posa=posb=posc=0;
           if ((flag == 0)||(flag == -1))
             posa=z+y*ds3+(x-1)*ds3*ds2;
           posb=pos;
           if ((flag == 0)||(flag == 1))
             posc=z+y*ds3+(x+1)*ds3*ds2;
/* pull averaged data from sbuffa */
            avgf1=sbuffa[ptf+posa];
            avgf2=sbuffa[ptf+posb];
            avgf3=sbuffa[ptf+posc];
            avgv1=sbuffa[ptu+posa];
            avgv2=sbuffa[ptu+posb];
            avgv3=sbuffa[ptu+posc];
            uf=diff(avgv1,avgf1,avgv2,avgf2,avgv3,avgf3,dx,flag);
/* same with vf with respect to y */
            flag=whichdiff(y,ds2);
            posa=posb=posc=0;
            if ((flag==0)||(flag == -1))
              posa=z+(y-1)*ds3+x*ds3*ds2;
            posb=pos;
            if ((flag==0)||(flag==1))
              posc=z+(y+1)*ds3+x*ds3*ds2;
            avgf1=sbuffa[ptf+posa];
            avgf3=sbuffa[ptf+posc];
            avgv1=sbuffa[ptv+posa];
            avgv2=sbuffa[ptv+posb];
            avgv3=sbuffa[ptv+posc];
            vf=diff(avgv1,avgf1,avgv2,avgf2,avgv3,avgf3,dy,flag);
/* same with wf and z */
            flag=whichdiff(z,ds3);
            posa=posb=posc=0;
            if ((flag==0)||(flag == -1))
              posa=pos-1;
            posb=pos;
            if ((flag==0)||(flag==1))
              posc=pos+1;
            avgf1=sbuffa[ptf+posa];
            avgf3=sbuffa[ptf+posc];
            avgv1=sbuffa[ptw+posa];
            avgv2=sbuffa[ptw+posb];
            avgv3=sbuffa[ptw+posc];
            wf=diff(avgv1,avgf1,avgv2,avgf2,avgv3,avgf3,dz,flag);
/* make sure the results are within limits */
            if ((fabs(tf)>bdl)||(fabs(uf)>bdl)||(fabs(vf)>bdl)||(fabs(wf)>bdl))
              sbuffa[vpos++]=bd;
            else
              sbuffa[vpos++]=tf+uf+vf+wf;
          }
        }
      }
    }
/* send the slice out (placing the value of dt at the end first) */
    sbuffa[vpos++]=dt;
    putslice(stdout,nelw,sbuffa);
/* transfer sbuffb data in sbuffa buffer and repeat above procedure */
    for (loop=0;loop<nel;loop++)
      sbuffa[loop]=sbuffb[loop];
    stat=getslice(stdin,nel,sbuffb);
  }
  exit(0);
}

/* this routine returns which form of diff to use */
/* centered or non-centered.  If non-centered it  */
/* returns which side to take the diff from       */
int whichdiff(n1,n2)
int n1,n2;                 /* n1 = where in cycle n2 = max cycle can be */
{
/* use center and right data points */
  if (n1==0)
    return(1);
/* use center and left data points */
  else if (n1==(n2-1))
    return(-1);
/* use both ends (centered) */
  else
    return(0);
}

/* this routine takes the partial of the data given.  The routine */
/* tries to use a centered diff but if one side has bad data the  */
/* routine switches to a non-centered diff.  Taking the center    */
/* point and either the left or right point.  If there is no way  */
/* to take the diff without bad data then bad is sent.            */
float diff(u1,f1,u2,f2,u3,f3,dval,fl)
float u1,f1,u2,f2,u3,f3;   /* the data at points i-1,i,i+1 */
float dval;                /* delta value */
int fl;                    /* flag for which diff to take */
{
  float aa,bb,cc;          /* computed end points for data */
  float compdiff;          /* what is the partial */

  aa=u1*f1;
  bb=u2*f2;
  cc=u3*f3;
/* see if the center diff is ok.  If not switch to non-centered */
/* and see if both ends are bad; if so then return bad          */
  if (fl == 0) {
    if ((fabs(u1)>bdl)||(fabs(f1)>bdl))
      fl = 1;
    if ((fabs(u3)>bdl)||(fabs(f3)>bdl)) {
      if (fl == 1)
        return(bd);
      else 
        fl = -1;
    }
  }
/* if center and right points of data */
  if (fl == 1) {
    if ((fabs(u2)>bdl)||(fabs(u3)>bdl)||(fabs(f2)>bdl)||(fabs(f3)>bdl))
      return(bd);
    else
      compdiff=((cc-bb)/dval);
  }
/* if center and left */
  if (fl == -1) {
    if ((fabs(u1)>bdl)||(fabs(u2)>bdl)||(fabs(f1)>bdl)||(fabs(f2)>bdl))
      return(bd);
    else
      compdiff=((bb-aa)/dval);
  }
/* if both ends (i.e. centered diff) */
  if (fl == 0)
    compdiff=((cc-aa)/(2*dval));
  return(compdiff);
}

float avg(dat1,dat2)
float dat1,dat2;
{
  float av;

  if ((fabs(dat1)>bdl)||(fabs(dat2)>bdl))
    av=bd;
  else
    av=(dat1+dat2)/2.0;
  return(av);
}
