/* raduf                                                                 */
/* usage: raduf < in_file > out_file                                     */
/* raduf reads a file from stdin written (from a universal format radar  */
/* tape) by rtape, and makes a cdf file for the data.  Bad data are      */
/* converted to our bad value and written if possible. But if the value  */
/* is a pointer to the beginning of the data section or some other value */
/* used by this program, we die with an error                            */

#include <stdio.h>
#include <ctype.h>
#include "cdfhdr.h"

#define  DAYINKS   86.4   /* length of day in kiloseconds */
#define  MAXFLDS   50     /* maximum # of variable data fields */
#define  NBUFFSIZE 50000  /* size of the buffers for files */
#define  getp(A)   getptr(buffer,HBMAX,vbuff,'v','d',A)

int status,flag;          /* eof? and misc info */
long ssize,fno;           /* number of elements to send */
char buffer[HBMAX][LINE]; /* the cdf header */
float *sbuff,*vbuff;      /* buffer for static and variable data */
float bad,badlim;         /* values for bad and badlim */
float azim,elev,sweepm;   /* azmith elevation and sweep mode */
float sweepr,fangl;       /* sweep rate and fixed angle */
float raycnt;             /* index counter */
float year,mon,day;       /* date */
float hour,min,sec;       /* time */
short *recbuff,*posns;    /* input buffer */
short bd,word1;;          /* bad data value and a pointer */

float *yearp,*monp,*dayp;   /* pointers for placement of data */
float *hourp,*minp,*secp,*azimp,*elevp,*sweepmp,*fanglp;
float *sweeprp,*polp,*raycntp,*nsampp,*nyquistp,*pdurp,*rtimep;

main()
{
  int nmrd;               /* number of values form read */
  char cl[LINE];          /* string for comments and parameter names */

/* set values */
  flag = 0;
  raycnt = 0;
  bad = 1e30;
  badlim = 9.99e+29;
/* malloc memory for buffers */
  if ((recbuff=(short*)malloc(NBUFFSIZE*sizeof(char))) == NULL)
    mfatal();
  if ((posns=(short*)malloc(10*sizeof(short))) == NULL)
    mfatal();
/* if not end of file */
  if ((status = scanf("%8ld",&fno)) != EOF) {
    if (fno > NBUFFSIZE) {
      fprintf(stderr,"raduf: input record size %d exceeds buffer size %d\n",
           fno,NBUFFSIZE);
      exit(1);
    }
    if ((nmrd = fread(recbuff,sizeof(short),fno/2,stdin)) != fno/2)
      rfatal(nmrd);
/* get bad data flag from universal header */
    bd = recbuff[44];
/* make cdf header */
    nullhdr(buffer,HBMAX,"float");
    if (isbad(word1 = recbuff[4]))
      bfatal();
/* first comment line identifies source of cdf file */
    sprintf(cl,"raduf:\n");
    addcline(buffer,HBMAX,cl);
/* read radar name and site name and add to header as comments */
    sprintf(cl,"%s\n",&recbuff[10]);
    cl[8] = '\n';
    cl[9] = '\0';
    addcline(buffer,HBMAX,cl);
    sprintf(cl,"%s\n",&recbuff[14]);
    addcline(buffer,HBMAX,cl);
/* include bad parameters */
    sprintf(cl,"%f",badlim);
    addpar(buffer,HBMAX,"badlim",cl);
    sprintf(cl,"%f",bad);
    addpar(buffer,HBMAX,"bad",cl);
/* force the longitude to be negitive */
    if (recbuff[21]>0) {
      recbuff[21] *= -1;
      recbuff[22] *= -1;
      recbuff[23] *= -1;
    }
/* read year month day hour min and sec */
    year = (float) recbuff[25];
    addfld(buffer,HBMAX,'v',"year",1.,0.,'s',0);
    mon = (float) recbuff[26];
    addfld(buffer,HBMAX,'v',"mon",1.,0.,'s',0);
    day = (float) recbuff[27];
    addfld(buffer,HBMAX,'v',"day",1.,0.,'s',0);
    hour = (float) recbuff[28];
    addfld(buffer,HBMAX,'v',"hour",1.,0.,'s',0);
    min = (float) recbuff[29];
    addfld(buffer,HBMAX,'v',"min",1.,0.,'s',0);
    sec = (float) recbuff[30];
    addfld(buffer,HBMAX,'v',"sec",1.,0.,'s',0);
/* get scalar variables, azimth */
    addfld(buffer,HBMAX,'v',"azim",100.,0.,'s',0);
    azim = (float) recbuff[32]/64;
/* elevation */
    addfld(buffer,HBMAX,'v',"elev",100.,0.,'s',0);
    elev = (float) recbuff[33]/64;
/* seep mode */
    addfld(buffer,HBMAX,'v',"sweepm",1.,0.,'s',0);
    sweepm = (float) recbuff[34];
/* fix angle */
    addfld(buffer,HBMAX,'v',"fangl",100.,0.,'s',0);
    fangl = (float) recbuff[35]/64;
/* sweep rate */
    addfld(buffer,HBMAX,'v',"sweepr",100.,0.,'s',0);
    sweepr = (float) recbuff[36]/64;
  }
/* while not end of file */
  while((status = scanf("%8ld",&fno)) != EOF) {
/* slice size includes above scalars. add size of other data as known */
    ssize = 11;
/* read data ray (may be more than one record) and reorganize into */
/* slice. readray outputs the header and the float slices */
    readray();
    if ((nmrd = fread(recbuff,sizeof(short),fno/2,stdin)) != fno/2)
      rfatal(nmrd);
/* get the hour, min and sec of the current ray */
  hour = (float) recbuff[28];
  min = (float) recbuff[29];
  sec = (float) recbuff[30];
/* assigne values */
    azim = (float) recbuff[32]/64;
    elev = (float) recbuff[33]/64;
    sweepm = (float) recbuff[34];
    fangl = (float) recbuff[35]/64;
    sweepr = (float) recbuff[36]/64;
  }
  exit(0);
}

/* readray reads the data, seeking to the nexts records as necessary */
/* to fill a slice, then writes slice */
int readray()
{
  int numray;              /* # of flds processed this ray so far */
  int nmrd;                /* return from read and write */
  int i,j,loop,count;      /* loop control */
  int currnitems,nitems;   /* number of items in var data array */
  int items[MAXFLDS];      /* table for data */
  char vname[LINE];        /* name of one variable */
  char pname[LINE];        /* name of a param */
  char varname[MAXFLDS][4];/* table for the names of data fields */
  float muls[MAXFLDS];     /* used in addfld */
  float fgrange,adj,rspacg;/* range to 1st gate, adj. to center, dr */
  float pol,nsamp;         /* polarization, wavelen, # of samples */
  float nyquist,pdur;      /* nyquist velocity, and pulse duration */
  float scale,mul;         /* scaling factors */
  float rhour,rmin,rsec;   /* used in time for rays */
  float oldtime,realtime;  /* used in time conversions */
  float *point;            /* used in placing data */
  short fperec,fperay;     /* # of fields in a record and ray */
  short offset;            /* vbuff array offset */
  struct field *fp;        /* pointer to field structure */

  currnitems=0;
  realtime=0.0;
  if (isbad(word1 = recbuff[4]))
    bfatal();
/* read number of fields this ray */
  if (isbad(fperay = recbuff[word1-1]))
    bfatal();
  if (fperay > MAXFLDS) {
    fprintf(stderr,"raduf: too many fields\n");
    exit(1);
  }
/* read number of fields this record */
  if (isbad(fperec = recbuff[word1+1]))
    bfatal();
  offset=11;
  numray=count=0;
/* while fields left to process in this ray */
  while(numray < fperay) {
/* for each var this rec */
    for(i = 0; i < fperec; i++) {
/* get variable name and pos of its header */
      sprintf(vname,"%s",&recbuff[word1+2+(2*i)]);
/* put name in lower case */
      if (isupper(vname[0]) != 0)
        vname[0] = vname[0] + 32;
      if (isupper(vname[1]) != 0)
        vname[1] = vname[1] + 32;
      vname[2] = '\0';
      if (isbad(posns[numray] = recbuff[word1+3+(2*i)]-1))
        bfatal();
/* if this variable is "ve", get nyquist, else later set to bad */
      if (strcmp("ve",vname) == 0) {
        scale = (float) recbuff[posns[numray] + 1];
        nyquist = (float) recbuff[posns[numray] + 19];
        if ((scale==bd)||(nyquist==bd))
          nyquist = scale = bad;
        else
          nyquist = nyquist/scale;
      }
/* if "dm" then get pulse duration */
      if (strcmp("dm",vname) == 0) {
        if (isbad(recbuff[posns[numray] + 24]))
          pdur = bad;
        else
          pdur = (float) recbuff[posns[numray] + 24] / 64;
      }
/* get number of items */
      if (isbad(nitems = recbuff[posns[numray]+5]))
        bfatal();
      if (currnitems == 0)
        currnitems = nitems;
/* ########################################################################## */
/* set all vars' nitems to the lowest one */
      if (nitems > currnitems)
        nitems = currnitems;
      else if (nitems < currnitems)
        currnitems = nitems;
/* bump slicesize */
      ssize = ssize + nitems;
/* ########################################################################## */
/* add this field's scale factor et al as scalar vars */
      mul = (float) recbuff[posns[numray]+1];
      if (mul < .00001)
        mul = 1.0;
/* add param of range to 1st gate (adjusted to center) & dist between */
      fgrange = recbuff[posns[0]+2]/1.0;
      adj = (recbuff[posns[0]+3])/1000.0;
      sprintf(pname,"%f",fgrange+adj);
      if ((numray==0)&&(flag==0))
        addpar(buffer,HBMAX,"range0",pname);
      rspacg = (float) recbuff[posns[numray]+4] / 1000.0;
      sprintf(pname,"%f",rspacg);
      if ((numray==0)&&(flag==0))
        addpar(buffer,HBMAX,"drange",pname);
      numray++;
/* transfer info into the table */
      strcpy(varname[count],vname);
      items[count]=nitems;
      muls[count]=mul;
      ++count;
    }
/* goto next record and get more varnames and posn's */
    if (numray < fperay) {
/* read in another record */
      scanf("%d",&fno);
      if ((nmrd = fread(recbuff,sizeof(short),fno/2,stdin)) != fno/2)
        rfatal(nmrd);
      if (isbad(word1 = recbuff[4]))
        bfatal();
      if (isbad(fperec = recbuff[word1+1]))
        bfatal();
    }
  }
/* make a static slice of range with nitems incremented by (adjustment */
/* to first gate + spacing between gates * i) and starting at range to */
/* first gate.  The values are from last var in slice. */
  if (flag == 0) {
    addfld(buffer,HBMAX,'s',"range",1.,0.,'s',1,"range",nitems);
    sbuff=getbuff(nitems);
    point=getptr(buffer,HBMAX,sbuff,'s','d',"range");
    for(i = 0; i < nitems; i++) {
      point[i] = fgrange+adj+(rspacg*i);
      if (isbad(fgrange) || isbad(adj) || isbad(rspacg))
        point[i] = bad;
    }
  }
/* add all fields to header for the variables */
  loop=0;
  while ((flag==0)&&(loop<count)) {
    nitems = items[loop];
    mul = muls[loop];
    if (isupper(varname[loop][0]) != 0)
      varname[loop][0] += 32;
    if (isupper(varname[loop][1]) != 0)
      varname[loop][1] += 32;
    addfld(buffer,HBMAX,'v',varname[loop],mul,0.,'s',1,"range",nitems);
    ++loop;
  }
/* get other field header values as static vars */
  pol = (float) recbuff[posns[0]+10];
  raycnt += 0.1;
  nsamp = (float) recbuff[posns[0]+12];
  if (flag == 0) {
    addfld(buffer,HBMAX,'v',"pol",100.0,0.0,'s',0);
    addfld(buffer,HBMAX,'v',"raycnt",1.0,0.0,'s',0);
    addfld(buffer,HBMAX,'v',"nsamp",100.0,0.0,'s',0);
    addfld(buffer,HBMAX,'v',"nyquist",scale,0.0,'s',0); 
    addfld(buffer,HBMAX,'v',"pdur",64.0,0.0,'s',0); 
    addfld(buffer,HBMAX,'v',"rtime",1000.0,0.0,'l',1,"range",nitems);
  }
/* if there is a variable "ve", get nyquist, else set to bad */
  if (seekfld(buffer,HBMAX,'v',"ve") == NULL)
    nyquist = bad;
/* if there is a variable "dm", get pulse duration, else set to bad */
  if (seekfld(buffer,HBMAX,'v',"dm") == NULL)
    pdur = bad;
/* add room for these six vars */
  ssize = ssize + 5 + nitems;
/* write header onto stdout only if first time called */
  j = 0;
  if (flag == 0)
    puthdr(stdout,buffer,HBMAX);
/* write slicesize and slice for static buffer only if first time called */
/* malloc memory for variable buffer and set up data pointers */
  if (flag == 0) {
    putslice(stdout,(long)nitems,sbuff);
    vbuff=getbuff((long)ssize);
    yearp=getp("year");
    monp=getp("mon");
    dayp=getp("day");
    hourp=getp("hour");
    minp=getp("min");
    secp=getp("sec");
    azimp=getp("azim");
    elevp=getp("elev");
    sweepmp=getp("sweepm");
    fanglp=getp("fangl");
    sweeprp=getp("sweepr");
    polp=getp("pol");
    raycntp=getp("raycnt");
    nsampp=getp("nsamp");
    nyquistp=getp("nyquist");
    pdurp=getp("pdur");
    rtimep=getp("rtime");
  }
  flag = 1;
 /* write scalar values gotten from header */
  yearp[0] = year;
  monp[0] = mon;
  dayp[0] = day;
  rhour = hour;
  rmin = min;
  rsec = sec;
/* note: this hour, min and sec contain just the time of the first ray */
/* rtime contains the time (in ksec) of each point */
  hourp[0] = rhour;
  minp[0] = rmin;
  secp[0] = rsec;
  azimp[0] = azim;
  elevp[0] = elev;
  sweepmp[0] = sweepm;
  fanglp[0] = fangl;
  sweeprp[0] = sweepr;
/* for each var, get from input buffer, test for bad and divide by mul */
  count=0;
  for(i = 0; i < fperay; i++) {
    point=getptr(buffer,HBMAX,vbuff,'v','d',varname[count]);
    for(j = 0; j < items[count]; j++) {
      if (isbad(recbuff[recbuff[posns[i]]+j-1]))
        point[j] = bad;
      else
        point[j]=(float)recbuff[recbuff[posns[i]]+j-1]/muls[count];
    }
    offset = offset + items[count];
    ++count;
  }
  polp[0] = pol;
  raycntp[0] = raycnt;
  nsampp[0] = nsamp;
  nyquistp[0] = nyquist;
  pdurp[0] = pdur;
/* place the time each point (all the same) along the ray into rtime */
  oldtime = realtime;
  realtime = rhour*3.6+rmin*.06+rsec/1000.;
  if (realtime < oldtime) realtime += DAYINKS;
  for (loop=0;loop<nitems;loop++)
    rtimep[loop] = realtime;
/* write the data slice to stdout */
  putslice(stdout,(long)ssize,vbuff);
  return(1);
}

/* isbad accepts a short from the input buffer and returns true if it */
/*  equals the bad data flag */
isbad(val)
short val;
{
  if (val == bd)
    return(1);
  return(0);
}

/* bfatal is called if  some value which is needed for this program's */
/*  calculations is bad. */
bfatal()
{
  fprintf(stderr,"raduf: bad value from uftape file, cannot continue\n");
  exit(1);
}

/* rfatal is called when an unexpected number of bytes is read */
rfatal(n)
int n;  /* number actually read */
{
  fprintf(stderr,"raduf: unexpected number of bytes read: %d\n",n);
  exit(1);
}

/* mfatal is called when malloc fails */
mfatal()
{
  fprintf(stderr,"raduf: can't get enough memory\n");
  exit(1);
}
