/* rp7.h -- This contains information about data from the rp7 radar
 * processor.
 */

#define HSIZE 100         /* input header size (shorts) */
#define MAXDATA 100000    /* maximum size of data buffer */
#define CF 182.044444     /* correction factor */
#define NFMAX 6           /* max number of fields */
#define NAMELENMAX 30     /* max length of field names */
#define CNTSKIP 30

struct rp7 {
  int log_rec_num;
  int tape_seq_num;
  int record_type;        /* 0 = data; 1 = prim pow cal; 2 = vel cal;
                             3 = sec pow cal; */
  int year;               /* 2 digits */
  int month;
  int day;
  int hour;
  int minute;
  int second;
  float azimuth;
  float elevation;
  float rhozero;          /* range (km) to leading edge of first gate */
  float delrho;           /* gate spacing (m) */
  int num_gates;
  int num_samples;        /* samples per gate */
  float pulse_width;      /* nanoseconds */
  float prf;              /* pulse rep freq (Hz) */
  float wavelength;       /* cm */
  int sweep_num;
  int scan_mode;          /* 0 = calib; 1 = PPI; 2 = coplane; 3 = RHI;
                             4 = vertical; 5 = target; 6 = manual;
                             7 = idle; 8 = surveillance */
  float clock_az_lim;     /* degrees */
  float counter_az_lim;   /* degrees */
  float up_el_lim;
  float low_el_lim;
  float fixed_angle;
  float prim_cop_b_angle;
  float sec_cop_b_angle;
  float scan_rate;        /* degrees per sec */
  int volume_count;
  int clutter_filter;
  int polarization;       /* 0 = horiz; 1 = vert */
  int mod_rec_count;      /* record count modulo 32767 */
  float altitude;         /* station altitude (m) */
  float longitude;        /* degrees */
  float latitude;         /* degrees */
  
  float avgtrpow;
  int transition_flag;    /* 1 = on; 0 = in a scan */
  int radar_id;           /* 2 = CP-2; 3 = CP-3; 4 = CP-4; 0x4d48 = MH */
  int nfields;            /* number of fields at each gate (<= NFMAX) */
  char fnames[NFMAX][NAMELENMAX]; /* names of fields */
  int fparams[NFMAX];     /* field parameters */
  int scale[NFMAX];       /* scale parameters for fields */
  int bias[NFMAX];        /* bias parameters for fields */
};
