/* rafdap								      *
 * convert ncar DAP format aircraft data to candis format		      *
 * no provision for the Cxxxxx file has been taken into account               *
 * 28 may 1990                                                                */

/* Modified 17 Nov 92 -- increased NVAR from 300 to 1000 per NCAR
 * modification to DAP format. -- D. J. Raymond
 */

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "cdfhdr.h"

/* max number of variable fields in DAP file (from original fortran code) */
#define NVAR 1000
#define LL 30
#define BIGHDR (2*HBMAX)

#define freadi(A) fread(A,sizeof(int),1,fh);
#define freadc(B,C) fread(B,sizeof(char),C,fh);

/* for files: Hxxxxx (header), Dxxxxx (data) */
FILE *fopen(),*fh,*fd;
long swap4();

/* names of stuff in header
   nmuser   => name of the user
   idatef   => date of last file access
   itimef   => time of last file access
   notes    => user note space
   idated   => date data was taken
   iproj    => project number
   iarcft   => aircraft number
   iflght   => flight number
   nmrecs   => number of records in data file
   nwords   => number of words in a data file record
   ideltt   => time between samples in the data file (milliseconds)
   ntmseg   => total number of time segments in the data file
   nvar     => total number of variables
   nvolt    => number of mode 1 (voltages) variables in the file
   names    => 6 character array of variable names
   units    => 4 char array containing ascii units of each variable
   qcx      => name of qc for computing tas,winds
   psx      => name of static press. for tas,winds
   ttx      => name of temperature for tas,winds
   avane    => name of attack vane for winds
   bvane    => name of sideslip vane for winds
   dpx      => name of dew point used for humidity
   itmseg   => start time of segment #n in isegtm(1,n),(2,n),(3,n)
               end time of segment #n in isegtm(4.n),(5,n),(6,n) 
*/
int  nmrecs,idated[3],iproj,iflght,nwords,ideltt,ntmseg,nvar,nvolt;
int itmseg[6][50];
char qcx[LL],psx[LL],ttx[LL],avane[LL],bvane[LL],dpx[LL];
char nmuser[17],names[NVAR][LL],units[NVAR][5],idatef[3][3],itimef[3][3];
char iarcft[5],notes[81];
float oldtime,newtime,delt;
int iday;

/* stuff in data file
   ihr      => time of record hours
   imin     => time of record minutes
   isec     => time of record seconds
   imsec    => time of record milli-seconds
   values   => values of each variable in hp floating point
*/
int ihr,imin,isec,imsec;
float values[NVAR];

float *sbuff;   /* buffer for slice data */

main(argc,argv)
int argc;
char *argv[];
{
  int stat,stats;                      /* status flags */
  int poop,loop,i,j;                   /* loop vars */
  long nel;                            /* # of elements in slice */
  char hfile[LINE],dfile[LINE];        /* names of input files */
  char hbuff[BIGHDR][LINE],cline[LINE]; /* candis header and comments */
  char namebuff[LINE];                 /* buffer for dup name fix */
  char lcnames[NVAR][LL];              /* names from DAP header in lowercase */
  float *dout[NVAR],*out[5];           /* pointers for out data */

/* check arguments */
  if ((argc == 1)||(argc > 3)) {
    fprintf(stderr,"Usage: %s [ Hxxxxx [ Dxxxxx | - ]]\n",argv[0]);
    fprintf(stderr,"       Hxxxxx is the header file\n");
    fprintf(stderr,"       Dxxxxx is the data file\n");
    fprintf(stderr,"          -   signifies data comes in via standard in\n");
    exit(1);
  }

/* move command line arguments into variable names and open files */
  if (argc >= 2) {
    strcpy(hfile,argv[1]);
    if ((fh=fopen(hfile,"r")) == NULL) {
      fprintf(stderr,"rafdap:  couldn't open %s\n",hfile);
      exit(1);
    }
  }

/* read header if no other arguments given then send header format as output */
  read_dap_header(argc);

/* open data file */
  strcpy(dfile,argv[2]);
  if (strcmp(dfile,"-")==0) fd = stdin;
  else if ((fd=fopen(dfile,"r")) == NULL) {
    fprintf(stderr,"rafdap:  couldn't open %s\n",dfile);
    exit(1);
  }

/* create null header */
  nullhdr(hbuff,BIGHDR,"float");

/* add cline */
  sprintf(cline,"%s: %s %s\n",argv[0],hfile,dfile);
  addcline(hbuff,BIGHDR,cline);

/* cycle thru variable names and convert to lowercase and get rid of spaces */
  for (i=0;i<nvar;i++) {
    for (j=0;j<7;j++) {
      stat=isupper(names[i][j]);
      stats=isspace(names[i][j]);
      if (stat!=0) lcnames[i][j]=tolower(names[i][j]);
      else if (stats!=0) lcnames[i][j] = '\0';
      else lcnames[i][j]=names[i][j];
    }
  }

/* tranfser pertinent stuff to cdf header: add comment lines */
  sprintf(cline,"user name: %s\n",nmuser);
  addcline(hbuff,BIGHDR,cline);
  sprintf(cline,"data taken: %d %d %d\n",idated[0],idated[1],idated[2]);
  addcline(hbuff,BIGHDR,cline);
  sprintf(cline,"     project# %d\n",iproj);
  addcline(hbuff,BIGHDR,cline);
  sprintf(cline,"     aircraft# %s\n",iarcft);
  addcline(hbuff,BIGHDR,cline);
  sprintf(cline,"     flight# %d\n",iflght);
  addcline(hbuff,BIGHDR,cline);

/* add variable fields   tptime is added so as to be compatible with existing
   programs */
  addfld(hbuff,BIGHDR,'v',"tptime",100.0,0.0,'s',0);
  addfld(hbuff,BIGHDR,'v',"hour",100.0,0.0,'s',0);
  addfld(hbuff,BIGHDR,'v',"min",100.0,0.0,'s',0);
  addfld(hbuff,BIGHDR,'v',"sec",100.0,0.0,'s',0);
  addfld(hbuff,BIGHDR,'v',"msec",100.0,0.0,'s',0);
  for (loop=0;loop<nvar;loop++) {

/* test for name duplication -- add a "." if duplicated */
    for (poop = 0; poop < loop; poop++) {
      if (strcmp(lcnames[loop],lcnames[poop]) == 0) {
        sprintf(cline,"%s.",lcnames[poop]);
        if (strlen(cline) > LL - 2) {
          fprintf(stderr,"rafdap: variable %s duplicated too much!\n",cline);
          exit(1);
        }
        strcpy(lcnames[loop],cline);
      }
    }
    addfld(hbuff,BIGHDR,'v',lcnames[loop],10000.0,0.0,'l',0);
  }

/* output header */
  puthdr(stdout,hbuff,BIGHDR);

/* send out empty static slice and set aside memory for variable slice */
  nel=elemcnt(hbuff,BIGHDR,'s');
  sbuff=getbuff(nel);
  putslice(stdout,nel,sbuff);
  free(sbuff);
  nel=elemcnt(hbuff,BIGHDR,'v');
  sbuff=getbuff(nel);

/* get pointers to the the data */
  for (loop=0;loop<nvar;loop++)
    dout[loop]=getptr(hbuff,BIGHDR,sbuff,'v','d',lcnames[loop]); 
  out[0]=getptr(hbuff,BIGHDR,sbuff,'v','d',"hour");
  out[1]=getptr(hbuff,BIGHDR,sbuff,'v','d',"min");
  out[2]=getptr(hbuff,BIGHDR,sbuff,'v','d',"sec");
  out[3]=getptr(hbuff,BIGHDR,sbuff,'v','d',"msec");
  out[4]=getptr(hbuff,BIGHDR,sbuff,'v','d',"tptime");
  newtime = -1.;                /* initialize for monotonicity check */
  delt = 1.;
  iday = 0;

/* attempt to read data */
  while ((stat=fread(&ihr,sizeof(int),1,fd))!=0) {
    fread(&imin,sizeof(int),1,fd);
    fread(&isec,sizeof(int),1,fd);
    fread(&imsec,sizeof(int),1,fd);

/* for vax's only */
#if VAX
    imin=swap4(imin);
    isec=swap4(imin);
    imsec=swap4(imin);
#endif

    for (loop=0;loop<nvar;loop++) {
      fread(&values[loop],sizeof(float),1,fd);

/* place data in sbuff */
      dout[loop][0]=values[loop];      
    }
    out[0][0]=(float)ihr;
    out[1][0]=(float)imin;
    out[2][0]=(float)isec;
    out[3][0]=(float)imsec;

/* compute seconds after midnight of first day */
    oldtime = newtime;
    newtime=(float)(ihr*3600+imin*60+isec + imsec/1000.0 + 86400*iday);
    if (newtime > oldtime) {
      delt = newtime - oldtime;
      if (delt > 1.) delt = 1.;   /* protect against record gaps */
    }
    else {

/* don't let time decrease */
      if (newtime - oldtime < -86390. && newtime - oldtime > -86410.) {
        iday++;                   /* we changed days */
        newtime += 86400.;
      }
      else newtime = oldtime + delt;    /* probably a data glitch */
    }
    out[4][0]=newtime;

/* write data slice */
    putslice(stdout,nel,sbuff);
  }
  fclose(fd);
  exit(0);
}

read_dap_header(narg)
int narg;
{
  int loop,i,j;
  long fsize1,fsize2;
  long fsize1p,fsize2p;

/* read data from header (part I) */
  fread(&fsize1,sizeof(long),1,fh); /* get size of block 1 (beginning #) */
  freadc(nmuser,16);                /* get name of user  */
  freadc(idatef[0],2);              /* get date */
  freadc(idatef[1],2);              /*    of last */
  freadc(idatef[2],2);              /*    file access */
  freadc(itimef[0],2);              /* get time */
  freadc(itimef[1],2);              /*    of last */
  freadc(itimef[2],2);              /*    file access */
  freadc(notes,80);                 /* user note space */
  freadi(&idated[0]);               /* date that */
  freadi(&idated[1]);               /*    data was */
  freadi(&idated[2]);               /*    taken */
  freadi(&iproj);                   /* project number */
  freadc(iarcft,4);                 /* aircraft number */
  freadi(&iflght);                  /* flight number */
  freadi(&nmrecs);                  /* # of records in data file */
  freadi(&nwords);                  /* # of words in a data file record */
  freadi(&ideltt);                  /* time between samples (ms) */
  freadi(&ntmseg);                  /* # of time segments */
  freadi(&nvar);                    /* total # of variables */
  freadi(&nvolt);                   /* # of mode 1 variables */

#if VAX
  idated[0]=swap4(idated[0]);               /* date that */
  idated[1]=swap4(idated[1]);               /*    data was */
  idated[2]=swap4(idated[2]);               /*    taken */
  iproj=swap4(iproj);                   /* project number */
  iflght=swap4(iflght);                  /* flight number */
  nmrecs=swap4(nmrecs);                  /* # of records in data file */
  nwords=swap4(nwords);                  /* # of words in a data file record */
  ideltt=swap4(ideltt);                  /* time between samples (ms) */
  ntmseg=swap4(ntmseg);                  /* # of time segments */
  nvar=swap4(nvar);                    /* total # of variables */
  nvolt=swap4(nvolt);                   /* # of mode 1 variables */
#endif

/* set last bit in strings to null so as not to cause future problem */
  notes[80]=iarcft[4]=nmuser[16]=idatef[0][2]=idatef[1][2]=idatef[2][2]='\0';
  itimef[0][2]=itimef[1][2]=itimef[2][2]='\0';

/* get names and units */
  for (loop=0;loop<NVAR;loop++) {
    freadc(names[loop],6);          /* get the names of the variables */
    names[loop][6]='\0';
  }
  for (loop=0;loop<NVAR;loop++)  {
    freadc(units[loop],4);          /* get the units for the variables */
    units[loop][4]='\0';
  }

/* get names used for computations */
  freadc(qcx,6);                    /* name of QC for computing TAS winds */
  freadc(psx,6);                    /* name of static press for TAS winds */
  freadc(ttx,6);                    /* name of temp for TAS winds */
  freadc(avane,6);                  /* name of attack vane for winds */
  freadc(bvane,6);                  /* name of sideslip vane for winds */
  freadc(dpx,6);                    /* name of dew point used for humidity */
  fread(&fsize1p,sizeof(long),1,fh);/* get size of block 1 (ending #) */

/* if only the header is input then output the format */
  if (narg == 2) {
    fprintf(stdout,"user name: %s\n",nmuser);
    fprintf(stdout,"date/time of last access: %s %s %s/%s %s %s\n\n",
      idatef[0],idatef[1],idatef[2],itimef[0],itimef[1],itimef[2]);
    fprintf(stdout,"NOTES:\n%s\n",notes);
    fprintf(stdout,"\ndate data taken: %d %d %d\n",
      idated[0],idated[1],idated[2]);
    fprintf(stdout,"project # =%d  aircraft # =%s  flight #   =%d\n",
      iproj,iarcft,iflght);
    fprintf(stdout,"# of recs =%d  # of words =%d  ideltt     =%d\n",
      nmrecs,nwords,ideltt);
    fprintf(stdout,"# of times=%d  # of vars  =%d  mode1 vars =%d\n",
      ntmseg,nvar,nvolt);
    fprintf(stdout,"\n");
    fprintf(stdout,"var    units         var    units         var    units\n");
    fprintf(stdout,"\n");
    for (loop=0;loop<nvar;loop+=3)
      fprintf(stdout,"%s %s          %s %s          %s %s\n",names[loop],
       units[loop],names[loop+1],units[loop+1],names[loop+2],units[loop+2]);
    fprintf(stdout,"name of QC for computing TAS winds    %s\n",qcx);
    fprintf(stdout,"name of static press for TAS winds    %s\n",psx);
    fprintf(stdout,"name of temp for TAS winds            %s\n",ttx);
    fprintf(stdout,"name of attack vane for winds         %s\n",avane);
    fprintf(stdout,"name of sideslip vane for winds       %s\n",bvane);
    fprintf(stdout,"name of dew point used for humidity   %s\n",dpx);
  }

  fread(&fsize2,sizeof(long),1,fh);  /* get size of block 2 (beginning #) */

/* get itmseg from header part II */
  for (i=0;i<50;i++) {
    for (j=0;j<6;j++) {
      freadi(&itmseg[j][i]);

#if VAX
      itmseg[j][i]=swap4(itmseg[j][i]);
#endif

    }
  }
  fread(&fsize2p,sizeof(long),1,fh);  /* get size of block 2 (ending #) */
  fclose(fh);

/* if only header given then output itmesg to stdout */
  if (narg == 2 ) {
    for (i=0;i<50;i++)
      fprintf(stdout,"start %6d %6d %6d     end %6d %6d %6d\n",itmseg[0][i],
        itmseg[1][i],itmseg[2][i],itmseg[3][i],itmseg[4][i],itmseg[5][i]);
  }

#if VAX
  fsize1=swap4(fsize1);
  fsize1p=swap4(fsize1p);
  fsize2=swap4(fsize2);
  fsize2p=swap4(fsize2p);
#endif

/* if beginning and ending block size #'s don't match report possible error */
  if (fsize1!=fsize1p) {
    fprintf(stderr,"rafdap: possible error in header (part I) please check\n");
    exit(2);
  }
  if (fsize2!=fsize2p) { 
    fprintf(stderr,"rafdap: possible error in itmseg (header part II)\n");
    exit(2);
  }
  if (narg == 2) exit(0);
  return;
}

#if VAX
long swap4(lval)
long lval;
{
  long temp;
  *(((char*)&temp) + 3) = *(((char*)&lval) + 0);
  *(((char*)&temp) + 2) = *(((char*)&lval) + 1);
  *(((char*)&temp) + 1) = *(((char*)&lval) + 2);
  *(((char*)&temp) + 0) = *(((char*)&lval) + 3);
  return(temp);
}
#endif
