%{
/* rafhdr -- Lex function to interpret RAF headers and create a
 * common data format file.  This is called by the main program
 * rafread.
 */
#include <math.h>
#undef input    /* our input and unput files are in rafread.c */
#undef unput
%}
%Start project day month year time skip initial vars vlist name ilist
%%
				{BEGIN initial;}
				{rafinit();}
<initial>"PROJECT"		{BEGIN project;}
<project>\"[-+0-9A-Z()\., ]+\"	{rafproj(yytext);}
<project>"PRDATE"		{BEGIN day;}
<day>\"[0-9 ]+\"		{rafday(yytext); BEGIN month;}
<month>\"[0-9A-Z ]+\"		{rafmonth(yytext); BEGIN year;}
<year>\"[0-9 ]+\"		{rafyear(yytext);}
<year>"BEGSNP"|"ENDSNP"		{BEGIN time;}
<time>[0-9\.]+			{raftime(yytext);}
<year,time>"COMMENT"			{BEGIN skip;}
<skip>"EXDATE = ("[0-9A-Z," ]+")"	{BEGIN vars;}
<skip>"ORDVAR = TITLE "		{BEGIN ilist; rafpush("TITLE");}
<vars>"ORDVAR"			{BEGIN ilist;}
<ilist>"LETVAR"			{rafilist(); BEGIN vlist;}
<vars>"LETVAR"			{BEGIN vlist;}
<ilist>[A-Z0-9]+		{rafpush(yytext);}
<vlist>"%FOR"			{BEGIN name;}
<vlist>\"[-+0-9a-zA-Z#\(\)\.,/&=:>< ]+\"	{rafpush(yytext);}
<vlist>[0-9\.E+]+			{rafpush(yytext);}
<name>[A-Z0-9]+			{rafvlist(yytext); BEGIN vars;}
.				;
%%
/* subroutines to help generate cdf header from RAF header */

#include "cdfhdr.h"
#include "rafread.h"

char hbuff[HBBIG][LINE];                /* header buffer */
char cline[LINE],temp[LINE];            /* temporary line buffers */
char stack[10][LINE];                   /* stack for reception of stuff */
char astack[10][LINE];                  /* attribute stack */
int tcall;                              /* number of time calls */
int stackptr;                           /* stack pointer */
int astackptr;                          /* attribute stack pointer */
int vcounter;                           /* variable counter */
char cbuff[MAXVARS][LINE];              /* comment buffer */
char nbuff[MAXVARS][LINE];              /* name buffer */
char ubuff[MAXVARS][LINE];              /* units buffer */
int rbuff[MAXVARS];                     /* rate buffer */
int rlist[MAXVARS];                     /* unique rate buffer */
int rcounter;                           /* unique rate counter */
int bitbuff[MAXVARS];                   /* no. bits buffer */
int fstbuff[MAXVARS];                   /* position buffer */
int slip;                               /* skip bits */
int bits;                               /* no. bits */
float factor[MAXVARS];                  /* smul */
float term[MAXVARS];                    /* sadd */
int datbytes;                           /* number of bytes in RAF record */
char byte;                              /* cdf precision */
char dname[LINE];                       /* dimension name */
int nomatch;                            /* test flag for static field gen */
char cfile;                             /* flag from rafread for cfile gen */
int loop,loop2;                         /* looping variables */

/* collect the information list that helps us interpret variable chars */
rafilist()
{
  long i;
/* copy stack to attribute stack */
  for (i = 0; i < 10*LINE; i++) astack[0][i] = stack[0][i];
  astackptr = stackptr;
/* zero variable counter and stack pointer */
  vcounter = 0;
  stackptr = 0;
}

/* collect all of the data on variables from the RAF header */
rafvlist(text)
char *text;
{
  char temp[LINE];
/* test for variable number overflow */
  if (vcounter >= MAXVARS) {
    fprintf(stderr,"rafread: more than %d variables\n",MAXVARS);
    exit(1);
  }
/* test for consistency between vlist and ilist */
  if (astackptr != stackptr) {
    fprintf(stderr,"rafread: astackptr %d != stackptr %d for %s\n",
         astackptr,stackptr,text);
    exit(1);
  }
/* get variable name */
  strcpy(nbuff[vcounter],text);
/* loop on information list */
  for (loop = 0; loop < stackptr; loop++) {
/* check for various possibilities */
/* comment or title */
    if (strcmp(astack[loop],"TITLE") == 0) {
      strcpy(cbuff[vcounter],stack[loop]);
      lcase(cbuff[vcounter]);
      quotstrip(cbuff[vcounter]);
    }
/* skip (spelled slip to avoid name conflict) */
    else if (strcmp(astack[loop],"SKIP") == 0) {
      slip = atoi(stack[loop]);
      if (slip != 0) {
        fprintf(stderr,"rafread: skip = %d is non-zero!\n",slip);
        exit(1);
      }
    }
/* location of first bit in buffer */
    else if (strcmp(astack[loop],"FSTBIT") == 0) {
      fstbuff[vcounter] = atoi(stack[loop]);
    }
/* number of bits per word */
    else if (strcmp(astack[loop],"BITS") == 0) {
      bitbuff[vcounter] = atoi(stack[loop]);
    }
/* rate */
    else if (strcmp(astack[loop],"RATE") == 0) {
      rbuff[vcounter] = atoi(stack[loop]);
    }
/* units */
    else if (strcmp(astack[loop],"UNITS") == 0) {
      strcpy(ubuff[vcounter],stack[loop]);
      quotstrip(ubuff[vcounter]);
    }
/* smul */
    else if (strcmp(astack[loop],"FACTOR") == 0) {
      factor[vcounter] = atof(stack[loop]);
    }
/* sadd */
    else if (strcmp(astack[loop],"TERM") == 0) {
      term[vcounter] = atof(stack[loop]);
    }
  }
/* increment variable count and zero stack pointer */
  vcounter++;
  stackptr = 0;
}

/* lower the case of a text string */
lcase(text)
char *text;
{
  int loop;
  loop = 0;
  while (text[loop] != '\0') {
    if ((text[loop] >= 'A') && (text[loop] <= 'Z')) text[loop] += 'a' - 'A';
    loop++;
  }
}

/* push a string onto stack */
rafpush(text)
char *text;
{
  strcpy(stack[stackptr],text);
  if (stackptr < 9) stackptr++;
}

/* initialize stuff for header generation */
rafinit()
{
  nullhdr(hbuff,HBBIG,"float");
  addcline(hbuff,HBBIG,"rafread:\n");
  rcounter = datbytes = stackptr = tcall = 0;
}

/* snatch project name */
rafproj(text)
char *text;
{
  quotstrip(text);
  sprintf(cline,"%s\n",text);
  if (cfile == 'n') addcline(hbuff,HBBIG,cline);
  else printf("%s",cline);
}

/* snatch day */
rafday(text)
char *text;
{
  quotstrip(text);
  strcpy(cline,text);
}

/* snatch month */
rafmonth(text)
char *text;
{
  quotstrip(text);
  strcat(cline," ");
  strcat(cline,text);
}

/* snatch year */
rafyear(text)
char *text;
{
  quotstrip(text);
  strcat(cline," ");
  strcat(cline,text);
  strcat(cline,"\n");
  if (cfile == 'n') addcline(hbuff,HBBIG,cline);
  else printf("%s",cline);
}

/* snatch time elements */
raftime(text)
char *text;
{
  tcall++;
  sprintf(temp,"%d",(int)atof(text));
  if (tcall == 1) strcpy(cline,temp);
  else strcat(cline,temp);
  if ((tcall != 3) && (tcall != 6)) strcat(cline,":");
  if (tcall == 3) strcat(cline," to ");
  if (tcall == 6) {
    strcat(cline,"\n");
    if (cfile == 'n') addcline(hbuff,HBBIG,cline);
    else printf("%s",cline);
  }
}

/* eliminate double quotes from string */
quotstrip(text)
char *text;
{
  int i,j;
  i = 0;
  j = 0;
  while (text[i] != '\0') {
    if (text[i] != '"') {
      text[j] = text[i];
      j++;
    }
    i++;
  }
  text[j] = '\0';
}

/* end it all */
yywrap()
{
/* assuming info tables full, loop on variable */
  for (loop = 0; loop < vcounter; loop++) {
/* check the number of bits */
    bits = bitbuff[loop];
    if (bits == 16) byte = 's';
    else if (bits == 32) byte = 'l';
    else {
      fprintf(stderr,"rafread: nonstandard word length = %d\n",bits);
      exit(1);
    }
/* lower the case of the variable name (an obsession of Raymond's!) */
    lcase(nbuff[loop]);
/* only add fields if we have to */
    if (cfile == 'n') {
/* set up dimension name */
      sprintf(dname,"time%d",rbuff[loop]);
/* add an entry to the variable field section of the header buffer */
      if (rbuff[loop] == 1) {
        addfld(hbuff,HBBIG,'v',nbuff[loop],factor[loop],
             term[loop],byte,0);
      }
      else {
        addfld(hbuff,HBBIG,'v',nbuff[loop],factor[loop],
             term[loop],byte,1,dname,rbuff[loop]);
      }
/* increment byte count */
      datbytes += rbuff[loop]*bitbuff[loop]/8;
/* add index field if needed to static field section of header buffer */
/* first check to see if we already have one of the same name */
      nomatch = 1;
      for (loop2 = 0; loop2 < rcounter; loop2++) {
        if (rlist[loop2] == rbuff[loop]) {
          nomatch = 0;
          break;
        }
      }
/* if we don't, add it to the list ... */
      if (nomatch) {
        rlist[rcounter++] = rbuff[loop];
      }
/* ... and add an index field */
      if (nomatch && (rbuff[loop] > 1)) {
        addfld(hbuff,HBBIG,'s',dname,100.,0.,'s',1,dname,rbuff[loop]);
      }
    }
  }
/* output either a variable table or a cdf header */
  if (cfile == 'y') {
    for (loop = 0; loop < vcounter; loop++) {
      printf("%10s: %3d Hz %10s %s\n",
           nbuff[loop],rbuff[loop],ubuff[loop],cbuff[loop]);
    }
  }
  else {
    puthdr(stdout,hbuff,HBBIG);
  }
  return(1);
}
