/* rafread -- This program reads an RAF file and data file and
 * produces a cdf file.  First two arguments are the respective files.
 * Last argument is name of variable to be used as time.
 * The two files are assumed to have been read from tape onto disk
 * using rtape, which heads each physical tape record with an 8 byte
 * decimal ascii integer telling how many bytes there are in the
 * record.
 */
#include <stdio.h>
#include "cdfhdr.h"
#include "rafread.h"
#define MAXBUFF 1000       /* size of input character buffer for header */

static FILE *ifile;               /* header and data input file */
static char ibuff[MAXBUFF];       /* character buffer for lex input routine */
static long iptr;                 /* buffer pointer for ibuff */
static char *dataptr,*dataget();  /* pointer to data record and routine that gets it */
static float *vslice,*vtemp;      /* pointers to variable slice */
static long nvslice;              /* # elements in variable slice */
static float *sslice;             /* pointer to static slice */
static long nsslice;              /* # elements in static slice */
static long callnum;              /* call number indicator on data input */
static char *databuff;            /* pointer to data buffer */
static long datapos;              /* internal pointer for dataget */
static long datarec;              /* physical record length in dataget */
static long datarec1;             /* first physical record length in dataget */
static struct field *fp;          /* temporary pointer */
static short shorty;              /* buffer for short word */
static long longy;                /* buffer for long word */
static short swap2();
static long swap4();

/* all externals defined in rafhdr.l */

extern char hbuff[HBBIG][LINE];  /* cdf header buffer */
extern int bitbuff[MAXVARS];  /* number of bits per word */
extern int rbuff[MAXVARS]; /* data rate = # sequential samples */
extern float factor[MAXVARS];  /* smul */
extern float term[MAXVARS];  /* sadd */
extern char nbuff[MAXVARS][LINE];  /* defined in rafhdr */
extern int vcounter;       /* number of variables */
extern int datbytes;       /* total number of bytes in logical record */
extern char cfile;         /* flag indicating comment file only to be made */

main(argc,argv)
int argc;
char *argv[];
{
  int loop,rloop;          /* looping variables */
/* check args */
  if (argc < 2) {
    fprintf(stderr,"Usage: rafread headerfile [datafile | -]\n");
    exit(1);
  }
/* set cfile flag for benefit of rafhdr */
  cfile = (argc == 2) ? 'y' : 'n';
/* open header file */
  if ((ifile = fopen(argv[1],"r")) == NULL) {
    fprintf(stderr,"rafread: can't open %s\n",argv[1]);
    exit(1);
  }
/* zero input buffer pointer so that "input" starts off correctly */
  iptr = 0;
/* read header file and make a cdf file */
  yylex();
/* close header file */
  fclose(ifile);
/* terminate if no data file specified */
  if (cfile == 'y') exit(1);
/* set up index fields */
  nsslice = elemcnt(hbuff,HBBIG,'s');
  if (nsslice > 0) {
    sslice = getbuff(nsslice);
    loop = 0;
    while ((fp = getfld(hbuff,HBBIG,'s',loop++)) != NULL) {
      for (rloop = 0; rloop < fp->dsize1; rloop++) {
        *(sslice + fp->ptr + rloop) = ((float)rloop)/((float)fp->dsize1);
      }
    }
  }
/* write a static slice */
  putslice(stdout,nsslice,sslice);
/* open data file for reading (standard input if -) */
  if (strcmp(argv[2],"-") == 0) ifile = stdin;
  else {
    if ((ifile = fopen(argv[2],"r")) == NULL) {
      fprintf(stderr,"rafread: can't open data file %s\n",argv[2]);
      exit(1);
    }
  }
/* allocate variable slice output buffer */
  vslice = getbuff(nvslice = elemcnt(hbuff,HBBIG,'v'));
/* zero things so that dataget works right */
  datarec1 = datapos = datarec = callnum = 0;
/* loop on logical records */
  while ((dataptr = dataget(datbytes)) != NULL) {
/* loop through RAF variables, transferring and converting */
    vtemp = vslice;
    for (loop = 0; loop < vcounter; loop++) {
/* 16 bit RAF data is equated to short integer, 2 bytes */
      if (bitbuff[loop] == 16) {
        for (rloop = 0; rloop < rbuff[loop]; rloop++) {
          shorty = *((short*)dataptr);
/* this is where machine dependence occurs */
#if VAX
          shorty = swap2(shorty);
#endif
          *vtemp = (float)shorty/factor[loop] - term[loop];
          vtemp++;
          dataptr += 2;
        }
      }
/* 32 bit RAF data is equated to long integer, 4 bytes */
      if (bitbuff[loop] == 32) {
        for (rloop = 0; rloop < rbuff[loop]; rloop++) {
          longy = *((long*)dataptr);
/* more machine dependence */
#if VAX
          longy = swap4(longy);
#endif
          *vtemp = (float)longy/factor[loop] - term[loop];
          vtemp++;
          dataptr += 4;
        }
      }
    }
/* write a variable slice to standard output */
    putslice(stdout,nvslice,vslice);
  }
  exit(0);
}

/* substitute lex input routine -- assumes ifile is open */
char input()
{
  char c,ichar;
  char header[9];
  int loop,nbytes;
/* fill the stack with an input block if it is empty */
  if (iptr <= 0) {
/* read the header, assuming rtape format */
    for (loop = 0; loop < 8; loop++) {
      if ((header[loop] = getc(ifile)) == EOF) return(0);
    }
    header[8] = '\0';
    nbytes = atoi(header);
/* then read the body, placing in correct order for popping off stack */
    iptr += nbytes;
    if (iptr > MAXBUFF) {
      fprintf(stderr,"rafread(input): input header buffer overflow\n");
      exit(1);
    }
    for (loop = 0; loop < nbytes; loop++) {
      if ((c = getc(ifile)) == EOF) {
        fprintf(stderr,"rafread: unexpected eof in header file\n");
        exit(1);
      }
/* turn nulls into spaces */
      ibuff[iptr - loop - 1] = (c == '\0' ? ' ' : c);
    }
  }
/* assuming a non-empty stack, return the top element */
  return(ibuff[--iptr]);
}

/* substitute lex unput routine */
unput(c)
char c;
{
  if (iptr >= MAXBUFF) {
    fprintf(stderr,"radfread(unput): input header buffer overflow\n");
    exit(1);
  }
  ibuff[iptr++] = c;
}

/* file that handles RAF data input */
char *dataget(dbytes)
int dbytes;
{
  char header[9];
  int i;
  int hold;         /*added rar 10/24/92 */
/* read rtape file if at end of physical record */
  if (datapos + dbytes > datarec) {
/* header first */
    for (i = 0; i < 8; i++) {
/*****if ((header[i] = getc(ifile)) == EOF) return(NULL);********/
      hold = getc (ifile);  /*added rar 10/24/92 */
      if (hold == EOF) {    /*added rar 10/24/92 */
        return (NULL);      /*added rar 10/24/92 */
      } else {              /*added rar 10/24/92 */
        header [i] = hold;  /*added rar 10/24/92 */
      }  /* EndIf */        /*added rar 10/24/92 */
    }
    header[8] = '\0';
    datarec = atoi(header);
    if ((callnum > 0) && (datarec > datarec1)) {
      fprintf(stderr,"rafread: datarec > datarec1\n");
      exit(1);
    }
/* malloc input buffer on first call */
    if (callnum++ == 0) {
      datarec1 = datarec;
      databuff = (char*)malloc(sizeof(char)*datarec1);
      if (databuff == NULL) {
        fprintf(stderr,"rafread: can't get enough memory for data buffer\n");
        exit(1);
      }
    }
/* then physical record */
    datapos = 0;
    if (fread(databuff,sizeof(char),datarec,ifile) == 0) {
      fprintf(stderr,"rafread: unexpected eof in reading data file\n");
      exit(1);
    }
  }
/* we have a valid buffer -- lets return a pointer */
  datapos += dbytes;
  return(databuff + datapos - dbytes);
}

/* routines that are only needed on VAX */
#if VAX
short swap2(sval)
short sval;
{
  short temp;
  swab(&sval,&temp,2);
  return(temp);
}

long swap4(lval)
long lval;
{
  long temp;
  *(((char*)&temp) + 3) = *(((char*)&lval) + 0);
  *(((char*)&temp) + 2) = *(((char*)&lval) + 1);
  *(((char*)&temp) + 1) = *(((char*)&lval) + 2);
  *(((char*)&temp) + 0) = *(((char*)&lval) + 3);
  return(temp);
}
#endif
