/* raftheform -- compute fields needed for a thetae-p chart */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

/* needed constants */
#define LV0 2.501e6
#define P0 1000.
#define RD 287.04
#define RV 461.5
#define CPD 1005.
#define CL 4190.
#define CTOK 273.15
#define EPS .622
#define KAPPA 0.286

main(argc,argv)
int argc;
char *argv[];
{
  float p1,p2,the1,the2,delp,delthe;
  float *p,*the,*theta;
  long ip,ithe,np,nthe;
  int handle,dhthe,dhp;
  float *floatbuff();
  float thcalc();

/* do command line */
  if (argc != 7) {
    fprintf(stderr,"Usage: raftheform p1 p2 np the1 the2 nthe\n");
    exit(1);
  }
  p1 = atof(argv[1]);
  p2 = atof(argv[2]);
  np = atoi(argv[3]);
  if (np <= 1) {
    fprintf(stderr,"raftheform: np = %d must be > 1\n",np);
    exit(1);
  }
  the1 = atof(argv[4]);
  the2 = atof(argv[5]);
  nthe = atoi(argv[6]);
  if (nthe <= 1) {
    fprintf(stderr,"raftheform: nthe = %d must be > 1\n",nthe);
    exit(1);
  }

/* allocate space */
  p = floatbuff(np);
  the = floatbuff(nthe);
  theta = floatbuff(np*nthe);

/* fill index fields */
  delp = (p2 - p1)/(np - 1);
  for (ip = 0; ip < np; ip++) {
    p[ip] = p1 + ip*delp;
  }
  delthe = (the2 - the1)/(nthe - 1);
  for (ithe = 0; ithe < nthe; ithe++) {
    the[ithe] = the1 + ithe*delthe;
  }

/* loop on grid */
  for (ip = 0; ip < np; ip++) {
    for (ithe = 0; ithe < nthe; ithe++) {
      theta[ip + np*ithe] = thcalc(p[ip],the[ithe]);
    }
  }

/* put it out in a candis file on the standard output */
  handle = candiscreate("-",0);
  candiscomment(handle,"raftheform:");
  dhthe = candisidim(handle,"thetae",nthe,the);
  dhp = candisidim(handle,"pressure",np,p);
  candisfld(handle,"theta",2,dhthe,dhp,0,0,theta);
  candiswrite(handle);

/* done! */
  exit(0);
}

/* calculate theta, given the pressure (mb) and thetae (K) */
float thcalc(p,the)
float p,the;
{
  float th,dthedth,t2,t1,rs,es,the2,the1;
  int i;
  float thecalc();

/* initialize iteration loop */
  th = 300.;
  rs = 1.;     /* choose a large value and thecalc won't let exceed sat */

/* loop till we poop */
  for (i = 0; i < 8; i++) {
    t2 = th*pow(p/P0,KAPPA);
    t1 = (th - 1.)*pow(p/P0,KAPPA);
    the2 = thecalc(t2,p,rs,0.);
    the1 = thecalc(t1,p,rs,0.);
    dthedth = the2 - the1;
    th += (the - the2)/dthedth;
  }

/* return result */
  return(th);
}

/* compute thetae given temp (K), pres (mb), and mixrat (g/g) */
float thecalc(t,p,r,l)
float t,p,r,l;
{
  float es,rh,x1,x2,a1,a2,lv,rs;
  float escalc();

/* don't let r exceed saturation value */
  es = escalc(t);
  rs = EPS*es/(p - es);
  if (r > rs) r = rs;
  rh = r/rs;

/* get actual L */
  lv = LV0 - 2320.*(t - CTOK);
  if (lv > LV0) lv = LV0;

/* do the rest */
  x1 = RD/(CPD + (r + l)*CL);
  x2 = -r*RV/(CPD + (r + l)*CL);
  a1 = P0*(1. + 1.608*r)/p;
  a2 = lv*r/((CPD + (r + l)*CL)*t);
  return(t*pow(a1,x1)*pow(rh,x2)*exp(a2));
}

/* compute saturation vapor pressure from dewpoint */
float escalc(td)
float td;
{
  return(6.112*exp(17.67*(td - CTOK)/(td - 29.65)));
}

/* floatbuff -- This routine uses malloc to allocate float buffer
 * space.  The argument nelem defines the number of floating
 * point elements.  It returns a float pointer to the space.  If malloc
 * fails, getbuff dies with an error message.
 */
float *floatbuff(nelem)
long nelem;
{
  float *point;
  if ((point = (float*)malloc((unsigned)(sizeof(float)*nelem)))
       == (float*)NULL) {
    fprintf(stderr,"floatbuff: can't get %d words\n",nelem);
    exit(1);
  }
  return(point);
}
