/* Do the calculation of reversible theta-e given temp (K), pres (mb),
 * mixrat (g/g), and liquid water mixrat (g/g).  The last factor
 * is generally not very important, and can be set to zero in most
 * cases without much loss of accuracy.
 * Algorithm from Kerry Emanuel.
 * Coded and checked by Dave Raymond.
 * Please send comments, error reports, fixes, etc. to
 * raymond@kestrel.nmt.edu
 */

#define LV0 2.501e6
#define P0 1000.
#define RD 287.04
#define RV 461.5
#define CPD 1005.
#define CL 4190.
#define CTOK 273.15
#define EPS .622

float the(t,p,r,l)
float t,p,r,l;
{
  float es,rh,x1,x2,a1,a2,lv,rs;
  float escalc();

/* don't let r exceed saturation value */
  es = escalc(t);
  rs = EPS*es/(p - es);
  if (r > rs) r = rs;
  rh = r/rs;

/* get actual L */
  lv = LV0 - 2320.*(t - CTOK);
  if (lv > LV0) lv = LV0;

/* do the rest */
  x1 = RD/(CPD + (r + l)*CL);
  x2 = -r*RV/(CPD + (r + l)*CL);
  a1 = P0*(1. + 1.608*r)/p;
  a2 = lv*r/((CPD + (r + l)*CL)*t);
  return(t*pow(a1,x1)*pow(rh,x2)*exp(a2));
}

/* compute saturation vapor pressure from dewpoint */
float escalc(td)
float td;
{
  return(6.112*exp(17.67*(td - CTOK)/(td - 29.65)));
}
