/* cdffibre.c -- Convert a Candis file to a Sisal cdf record in
 * Fibre format.
 */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

char hbuff[HBMAX][LINE];
char comment[HBMAX][LINE];
float *sbuff,*vbuff;
long nsbuff,nvbuff;
struct field *fp;
struct field ixlist[4];
char *name;
int instance;
int nclines;
int poop,loop;
int length;
int nparam;
char pname[LINE];
char pvalue[LINE];
int ndims;
int nflds;

main(argc,argv)
int argc;
char *argv[];
{

/* check arguments */
  if (argc != 3) {
    fprintf(stderr,"Usage: cdffibre name instance\n");
    exit(1);
  }
  name = argv[1];
  instance = atoi(argv[2]);

/* get header */
  gethdr(stdin,hbuff,HBMAX);
  if (getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"cdffibre: float format expected\n");
    exit(1);
  }

/* compute input buffer lengths */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  nvbuff = elemcnt(hbuff,HBMAX,'v');

/* allocate buffers */
  sbuff = getbuff(nsbuff);
  vbuff = getbuff(nvbuff);

/* read static and first (and hopefully only) variable slices */
  getslice(stdin,nsbuff,sbuff);
  getslice(stdin,nvbuff,vbuff);

/* write the Fibre format file to standard output */

/* start record, do name and instance */
  printf("<\n  \"%s\"\n  %d\n",name,instance);

/* do comments */
  nclines = getcmt(hbuff,HBMAX,comment);
  printf("  [ 1,%d:\n",nclines);
  for (loop = 0; loop < nclines; loop++) {
    length = strlen(comment[loop]) - 1;
    comment[loop][length] = '\0';
    printf("    \"%s\"\n",comment[loop]);
  }
  printf("  ]\n");

/* count the parameters */
  nparam = 0;
  while (getpar(hbuff,HBMAX,nparam,pname,pvalue) == OK) nparam++;

/* do parameters */
  printf("  [ 1,%d:\n",nparam);
  for (loop = 0; loop < nparam; loop++) {
    getpar(hbuff,HBMAX,loop,pname,pvalue);
    printf("    < \"%s\" %e >\n",pname,atof(pvalue));
  }
  printf("  ]\n");

/* count number of index fields */
  loop = 0;
  ndims = 0;
  while ((fp = getfld(hbuff,HBMAX,'s',loop++)) != NULL) {
    if (fp->dim == 1 && strcmp(fp->fname,fp->dname1) == 0) {
      if (ndims > 3) {
        fprintf(stderr,"cdffibre: too many index fields\n");
        exit(1);
      }
      ixlist[ndims] = *fp;
      ndims++;
    }
  }

/* do dimensions */
  printf("  [ 1,%d:\n",ndims);
  for (loop = 0; loop < ndims; loop++) {
    printf("    <\n");
    printf("      \"%s\"\n",ixlist[loop].fname);
    printf("      %d\n",ixlist[loop].tsize);
    printf("      [ 0,%d:\n",ixlist[loop].tsize - 1);
    for (poop = 0; poop < ixlist[loop].tsize; poop++) {
      printf("        %e\n",sbuff[poop + ixlist[loop].ptr]);
    }
    printf("      ]\n    >\n");
  }
  printf("  ]\n");

/* count the fields besides index fields */
  nflds = 0;
  loop = 0;
  while ((fp = getfld(hbuff,HBMAX,'s',loop++)) != NULL) {
    if (fp->dim != 1 || strcmp(fp->fname,fp->dname1) != 0) {
      nflds++;
    }
  }
  loop = 0;
  while ((fp = getfld(hbuff,HBMAX,'v',loop++)) != NULL) nflds++;

/* put other static fields in variable slice */
  printf("  [ 1,%d:\n",nflds);
  doflds(sbuff,'s');

/* do variable fields */
  doflds(vbuff,'v');
  printf("  ]\n>\n");

/* finished */
  exit(0);
}

/* doflds -- add the fields to the fibre output */
doflds(buff,type)
float *buff;
char type;
{
  int dh0,dh1,dh2,dh3;

/* loop on non-index fields */
  loop = 0;
  while ((fp = getfld(hbuff,HBMAX,type,loop++)) != NULL) {
    if (fp->dim != 1 || strcmp(fp->fname,fp->dname1) != 0) {

/* get index field associated with each dimension */
      dh0 = dh1 = dh2 = dh3 = -1;
      if (fp->dim > 0) dh0 = dimcheck(fp->dname1,ixlist);
      if (fp->dim > 1) dh1 = dimcheck(fp->dname2,ixlist);
      if (fp->dim > 2) dh2 = dimcheck(fp->dname3,ixlist);
      if (fp->dim > 3) dh3 = dimcheck(fp->dname4,ixlist);

/* write the fibre output */
      printf("    <\n      \"%s\" %d [ 1,4: %d %d %d %d] %d\n",
           fp->fname,fp->dim,dh0,dh1,dh2,dh3,fp->tsize);
      printf("      [ 0,%d:\n",fp->tsize - 1);
      for (poop = 0; poop < fp->tsize; poop++) {
        printf("        %e\n",buff[poop + fp->ptr]);
      }
      printf("      ]\n");
      printf("    >\n");
    }
  }
}

/* dimcheck -- check to see if a dimension has an index field */
int dimcheck(dname,ixl)
char *dname;
struct field *ixl;
{
  int i;
  for (i = 0; i < ndims; i++) {
    if (strcmp(dname,ixl[i].fname) == 0) return(i);
  }
  fprintf(stderr,"cdffibre: dimension %s doesn't have an index field\n",dname);
  exit(1);
}
