/* addcline -- This routine adds a comment line to a header buffer.
 * If available buffer space is exceeded, the routine dies, as is
 * also true if the comment line cline is too long.  Cline must have
 * a newline at the end, and be terminated with a null as are normal
 * C language strings.
 */
#include <stdio.h>
#include "cdfhdr.h"

addcline(buffer,max,cline)
char buffer[][LINE],*cline;
int max;
{
  int loop,length;
  char temp[LINE];
/* check for excessive line length */
  if ((length = strlen(cline)) >= LINE) {
    fprintf(stderr,"addcline: Comment line too long\n");
    exit(1);
  }
/* check for no terminating newline */
  if (cline[length - 1] != '\n') {
    fprintf(stderr,"addcline: No terminating newline on comment\n");
    exit(1);
  }
/* find end of buffer */
  length = 0;
  for (loop = 6; loop < max - 1; loop++) {
    if (ETEST(loop)) {
      length = loop;
      break;
    }
  }
/* test for bad format */
  if (length == 0) {
    fprintf(stderr,
      "addcline: Bad header buffer format or no more buffer space\n");
    exit(1);
  }
/* ripple down to make room for new insert */
  for (loop = length; loop > 0; loop--) {
    strcpy(buffer[loop + 1],buffer[loop]);
    sscanf(buffer[loop],"%s",temp);
    if (strcmp(temp,PSH) == 0) {
      strcpy(buffer[loop],cline);
      return;
    }
  }
/* error if we couldn't find paramater list sub-header */
  fprintf(stderr,"addcline: Bad header buffer format\n");
  exit(1);
}
