/* addfld -- This routine adds a field to a header buffer.  The type is
 * either 's' for static or 'v' for variable.  The elements of the field 
 * structure are arguments 4, 5, 6, ... of the function call.  The total number
 * of arguments is variable depending on the dimension of the field.
 */
#include <stdio.h>
#include "cdfhdr.h"

/* VARARGS8 */
addfld(buffer,max,type,fname,smul,sadd,precis,dim,d1,n1,d2,n2,d3,n3,d4,n4)
char buffer[][LINE],type,precis,*fname,*d1,*d2,*d3,*d4;
float smul,sadd;
int max,dim,n1,n2,n3,n4;
{
  int loop,length,stat;
  char line[2*LINE],temp[LINE];
/* check for proper type */
  if ((type != 's') && (type != 'v')) {
    fprintf(stderr,"addfld: Unrecognized type = %c\n",type);
    exit(1);
  }
  stat = (type == 's');
/* check for existence of field of this name */
  if (seekfld(buffer,max,type,fname) != NULL) {
    fprintf(stderr,"addfld: field %s already exists\n",fname);
    exit(1);
  }
/* check for array dimension out of range */
  if ((dim < 0) || (dim > 4)) {
    fprintf(stderr,"addfld: dim = %d out of range on field = %s\n",dim,fname);
    exit(1);
  }
/* check for pixel field consistency */
  if (precis == 'p') {
    if (smul != 1. && smul != 2. && smul != 4. && smul != 8. &&
         smul != 16. && smul != 32.) {
      fprintf(stderr,"addfld: smul = %g not a proper pixel value\n",smul);
      exit(1);
    }
    if (smul*sadd != 8.*sizeof(float)) {
      fprintf(stderr,"addfld: smul = %g times sadd = %g != float size\n",
           smul,sadd);
      exit(1);
    }
  }
/* make up line */
  sprintf(line,"%s %g %g %c %d",fname,smul,sadd,precis,dim);
  if (dim > 0) {
    sprintf(temp," %s %d",d1,n1);
    strcat(line,temp);
  }
  if (dim > 1) {
    sprintf(temp," %s %d",d2,n2);
    strcat(line,temp);
  }
  if (dim > 2) {
    sprintf(temp," %s %d",d3,n3);
    strcat(line,temp);
  }
  if (dim > 3) {
    sprintf(temp," %s %d",d4,n4);
    strcat(line,temp);
  }
  strcat(line,"\n");
/* check for excessive length */
  if (strlen(line) > LINE - 1) {
    fprintf(stderr,"addfld: field too long: %s\n",line);
    exit(1);
  }
/* find end of buffer */
  length = 0;
  for (loop = 6; loop < max - 1; loop++) {
    if (ETEST(loop)) {
      length = loop;
      break;
    }
  }
/* test for bad format */
  if (length == 0) {
    fprintf(stderr,
      "addfld: Bad header buffer format or no more buffer space\n");
    exit(1);
  }
/* ripple down to make room for new insert */
  for (loop = length; loop > 0; loop--) {
    strcpy(buffer[loop + 1],buffer[loop]);
    sscanf(buffer[loop],"%s",temp);
    if ((stat && (strcmp(temp,VSH) == 0)) ||
      (!stat && (strcmp(temp,FSH) == 0))) {
      strcpy(buffer[loop],line);
      return;
    }
  }
/* error if we couldn't find field list sub-header */
  fprintf(stderr,"addfld: Bad header buffer format\n");
  exit(1);
}
